using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using Ato.EN.IntegrationServices.CodeGenerationIDS;

namespace Ato.CD.Inbound.TRT202202
{
    internal class CrossFormValidatorIDS : ICrossFormValidator
    {
        private TRT2022 ParentDocument { get; }

        private IDS2022 ChildDocument { get; }

        internal CrossFormValidatorIDS(TRT2022 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (IDS2022)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOIDS440670(response);
            VRATOTRT432649(response);

            return response;
        }

        #region VR.ATO.IDS.440670

        /*  VR.ATO.IDS.440670
                    Your supplied TFN does not match the TFN supplied on the form it was submitted with

                Legacy Rule Format:
                    IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                    RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^TFN <> ^ParentTFN

        Data Elements:
            
                    ^ParentTFN = ParentTFN
            
                    ^TFN = RP
                    */
        private void VRATOIDS440670(List<ProcessMessageDocument> response)
        {
            bool assertion = ! ChildDocument.IDS500.Equals(ParentDocument.RPIdentifierTFN );

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/tns:IDS/tns:RP/tns:TaxFileNumberId",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IDS.440670" } }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440670

        #region VR.ATO.TRT.432649

        /*  VR.ATO.TRT.432649
                    If there is a yes response to question 4 label A on the IDS, there must also be a yes response to question 29 label C (transactions with specified countries) on the trust or partnership return form.

                Legacy Rule Format:
                    IF [TRT151] = FALSE AND [IDS9] = TRUE
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS9] = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
                    [TRT151] = TRT:RP:rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator

            Technical Business Rule Format:
                    ^TRT151 = FALSE AND ^IDS9 = TRUE

        Data Elements:
            
                    RP:^TRT151 = tns:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
            
                    RP:^IDS9 = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
                    */
        private void VRATOTRT432649(List<ProcessMessageDocument> response )
        {
            bool assertion = ParentDocument.TRT151.GetValueOrDefault() == false && ChildDocument.IDS9.GetValueOrDefault() == true;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432649",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Transactions with specified countries question incorrect",
                    LongDescription = @"If there is a 'True' response at 'Did you have dealings with international related parties located in specified countries?' on the IDS then the response at Overseas transactions - Transactions with specified countries on the trust return must be 'True'.",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432649" },
                        new ProcessMessageParameter { Name = "TRT151", Value = ParentDocument.TRT151.ToString () }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432649
    }
}