using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using Ato.EN.IntegrationServices.CodeGenerationIEE;

namespace Ato.CD.Inbound.TRT202202
{ 
    internal class CrossFormValidatorIEE : ICrossFormValidator
    {
        private TRT2022 ParentDocument { get; }

        private IEE2021 ChildDocument { get; } 

        internal CrossFormValidatorIEE(TRT2022 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (IEE2021)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN410324(response);
            VRATOGEN410396(response);
            VRATOIEE410410(response);            
            VRATOIEE410051(response);

            return response;
        }

        #region VR.ATO.IEE.410324

        /* VR.ATO.IEE.410324
        Reporting period on Schedule must match reporting period on main form

        WHERE PARENT RETURN EXISTS
        IF [IEE201] <> PARENT RETURN:RP:period.startDate OR [IEE202] <> PARENT RETURN:RP:period.endDate 
            RETURN VALIDATION MESSAGE
        ENDIF
        
        IEE201 = tns:PeriodStartD

        IEE202 = tns:PeriodEndD

        */
        private void VRATOGEN410324(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE201 != ParentDocument.RPStartDate || ChildDocument.IEE202 != ParentDocument.RPEndDate;
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410324",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Reporting period does not match the reporting period supplied on the form it was submitted with",                    
                    Location = "//tns:IEE/tns:RP/tns:PeriodStartD",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410324" },
                        new ProcessMessageParameter { Name = "IEE201", Value = ChildDocument.IEE201.ToString() },
                        new ProcessMessageParameter { Name = "IEE202", Value = ChildDocument.IEE202.ToString() },
                        new ProcessMessageParameter { Name = "RP:period.startDate", Value = ParentDocument.RPStartDate.ToString() },
                        new ProcessMessageParameter { Name = "RP:period.endDate", Value = ParentDocument.RPEndDate.ToString() }
                    }
                };

                response.Add(processMessage);
            }
        }

        #endregion VR.ATO.IEE.410324

        #region VR.ATO.IEE.410396

        /*  VR.ATO.IEE.410396

        ABN on Schedule must match ABN on main form

            WHERE PARENT RETURN EXISTS
            IF (^IEE162 <> BLANK) AND ([TRT7] <> BLANK) AND (^IEE162 <> [TRT7])
                RETURN VALIDATION MESSAGE
            ENDIF

        Data Elements:
            
                    IEE162 = tns:AustralianBusinessNumberId
            
                    RP:^ParentABN = ParentABN
                    */
        private void VRATOGEN410396(List<ProcessMessageDocument> response )
        {
            bool assertion =  !string.IsNullOrWhiteSpace(ChildDocument.IEE162)  && !string.IsNullOrWhiteSpace(ParentDocument.TRT7 )  &&  !ParentDocument.TRT7.Equals (ChildDocument.IEE162);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied ABN does not match the ABN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "//tns:IEE/tns:RP/tns:AustralianBusinessNumberId",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410396" },
                        new ProcessMessageParameter { Name = "IEE162", Value = ChildDocument.IEE162 }
                    }
                };
                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410396

        #region VR.ATO.IEE.410410

        /*  VR.ATO.IEE.410410
        TFN on Schedule must match TFN on main form

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS 
                    IF ([IEE3] <> PARENT RETURN:RP:entity.identifier.TFN)
                         RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^TFN <> BLANK) AND (^TFN <> ^ParentTFN)

        Data Elements:
            
                    ^ParentTFN = ParentTFN
           
                    ^TFN = RP
                    */
        private void VRATOIEE410410(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.IEE3) && !string.IsNullOrWhiteSpace(ParentDocument.RPIdentifierTFN) && !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.IEE3);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.410115",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN on Schedule must match TFN on main form",
                    Location = "//tns:IEE/tns:RP/tns:TaxFileNumberId",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410410" },
                        new ProcessMessageParameter { Name = "IEE3", Value = ChildDocument.IEE3.ToString() },

                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410410

        #region VR.ATO.IEE.410051

        /*  VR.ATO.IEE.410051
                    Election Commencement Time must not be entered if Income Year Specified is different to the parent return.

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF ([IEE49] <> NULL) AND ([IEE45] <> PARENT RETURN:Report.TargetFinancial.Year) 
                        RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE49 <> NULL) AND (^IEE45 <> ^ParentTargetFinancialYear)

        Data Elements:
            
                    RP:^IEE49 = tns:ElectionsCommencementD
            
                    RP:^ParentTargetFinancialYear = ParentTargetFinancialYear
            
                    RP:^IEE45 = tns:ElectionsInterposedEntityElectionStatusY
                    */
        private void VRATOIEE410051(List<ProcessMessageDocument> response )
        {
            bool assertion =   ChildDocument.IEE49.HasValue  && !ParentDocument.TRT1.GetValueOrDefault ().Equals(ChildDocument.IEE45.GetValueOrDefault());

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410235",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Election Commencement Time must not be present if Income Year Specified is different to the parent return target financial year",
                    Location = "//tns:IEE/tns:RP/tns:InterposedEntityElectionRevocationDetails/tns:ElectionsCommencementD",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410051" },
                        new ProcessMessageParameter { Name = "IEE49", Value = ChildDocument.IEE49.GetValueOrDefault().ToString ()},
                        new ProcessMessageParameter { Name = "IEE45", Value = ChildDocument.IEE45.GetValueOrDefault().ToString() }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410051
    }
}