using System.Linq;
using System.Collections.Generic;
using VaTS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRTAMI;
using Ato.EN.IntegrationServices.CodeGenerationTRTAMIS;
using Ato.EN.IntegrationServices.CodeGenerationCGTS;
using Ato.EN.IntegrationServices.CodeGenerationIDS;
using Ato.EN.IntegrationServices.CodeGenerationPSS;
using Ato.EN.IntegrationServices.CodeGenerationRS;

namespace Ato.CD.Inbound.TRTAMI202202
{
    internal class CrossFormValidator
    {
        private TRTAMI2022 ParentDocument { get; }
        private BusinessDocumentCollection ChildDocuments { get; }
        private IEnumerable<CGTS2018> CGTSChildDocuments { get; }
        private IEnumerable<IDS2022> IDSChildDocuments { get; }
        private IEnumerable<PSS2018> PSSChildDocuments { get; }
        private IEnumerable<RS2018> RSChildDocuments { get; }
        private IEnumerable<TRTAMIS2022> TRTAMISChildDocuments { get; }

        public CrossFormValidator(TRTAMI2022 report, BusinessDocumentCollection childDocuments)
        {
            ParentDocument = report;
            ChildDocuments = childDocuments ?? new BusinessDocumentCollection();

            CGTSChildDocuments = ChildDocuments
                    .Where(s => s.DocumentName.Equals(Schedule.CGTS.ToString()))
                    .Select(p => (CGTS2018)p.ConsumedReport);

            IDSChildDocuments = ChildDocuments
                    .Where(s => s.DocumentName.Equals(Schedule.IDS.ToString()))
                    .Select(p => (IDS2022)p.ConsumedReport);

            PSSChildDocuments = ChildDocuments
                    .Where(s => s.DocumentName.Equals(Schedule.PSS.ToString()))
                    .Select(p => (PSS2018)p.ConsumedReport);

            RSChildDocuments = ChildDocuments
                    .Where(s => s.DocumentName.Equals(Schedule.RS.ToString()))
                    .Select(p => (RS2018)p.ConsumedReport);

            TRTAMISChildDocuments = ChildDocuments
                    .Where(s => s.DocumentName.Equals(Schedule.TRTAMIS.ToString()))
                    .Select(p => (TRTAMIS2022)p.ConsumedReport);
        }

        public List<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOTRTAMI000244(response);
            VRATOTRTAMI000234(response);
            VRATOTRTAMI000236(response);
            VRATOTRTAMI000314(response);
            VRATOTRTAMI000315(response);
            VRATOTRTAMI000235(response);
            VRATOTRTAMI000203(response);
            VRATOTRTAMI000174(response);
            VRATOTRTAMI000240(response);
            VRATOTRTAMI000151(response);
            VRATOTRTAMI000152(response);
            VRATOTRTAMI000153(response);
            VRATOTRTAMI000154(response);
            VRATOTRTAMI000199(response);
            VRATOTRTAMI000201(response);
            VRATOTRTAMI000205(response);
            VRATOTRTAMI000246(response);
            VRATOTRTAMI000330(response);
            VRATOGEN438017(response);

            return response;
        }

        #region VR.ATO.TRTAMI.000244
        //Rule ID: VR.ATO.TRTAMI.000244

        //IF[TRTAMI102] = FALSE AND ANY OCCURRENCE OF([TRTAMIS313]) <> NULLORBLANK
        //  RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOTRTAMI000244(List<ProcessMessageDocument> response)
        {
            bool assertion = ParentDocument.TRTAMI102.GetValueOrDefault() == false &&
                ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.TRTAMIS.ToString())).
                Select(c => (TRTAMIS2022)c.ConsumedReport).Any(t => t.TRTAMIS313 != null);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000244",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "'Have you chosen to treat separate classes in the trust as a separate AMIT?' response is incorrect",
                    LongDescription = "When the Name of AMIT Class has been provided for any Trust Attribution Managed Investments Schedule (TRTAMIS), the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' must be 'True' (yes)",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MultiClassTreatment.Indicator",
                    Parameters = new ProcessMessageParameters {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000244" },
                        new ProcessMessageParameter { Name = "TRTAMI102", Value = ParentDocument.TRTAMI102.ToString() }
                    }
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000234
        //Rule ID: VR.ATO.TRTAMI.000234

        //    IF COUNT(SCHEDULE = "TRTAMIS") > 1 AND[TRTAMIS313] = ANY OTHER OCCURRENCE OF[TRTAMIS313]
        //       RETURN VALIDATION MESSAGE
        //    ENDIF
        private void VRATOTRTAMI000234(List<ProcessMessageDocument> response)
        {
            IEnumerable<TRTAMIS2022> duplicates = ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.TRTAMIS.ToString()))
               .Select(c => (TRTAMIS2022)c.ConsumedReport).Where(s => !string.IsNullOrEmpty(s.TRTAMIS313)).GroupBy(s => s.TRTAMIS313).Where(s => s.Skip(1).Any())
               .SelectMany(c => c);

            response.AddRange(from duplicate in duplicates
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.TRTAMI.000234",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = "Name of AMIT Class for TRTAMIS schedule is invalid",
                                  LongDescription = "When multiple Trust Attribution Managed Investments Schedules (TRTAMIS) are provided, a unique Name of AMIT Class must be provided for each",
                                  Location = "/xbrli:xbrl/tns:IncomeTax.AttributionManagedInvestmentTrustClass.Description",
                                  Parameters = new ProcessMessageParameters {
                                      new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000234" },
                                      new ProcessMessageParameter { Name = "TRTAMIS313", Value = duplicate.TRTAMIS313 }
                                  }
                              });
        }
        #endregion

        #region VR.ATO.TRTAMI.000236
        //IF[TRTAMI102] = FALSE AND COUNT(SCHEDULE = "TRTAMIS") > 1
        //   RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOTRTAMI000236(List<ProcessMessageDocument> response)
        {
            bool assertion = (ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.TRTAMIS.ToString())) > 1)
                && !ParentDocument.TRTAMI102.GetValueOrDefault();

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000236",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "'Have you chosen to treat separate classes in the trust as a separate AMIT?' response is incorrect",
                    LongDescription = "When multiple Trust Attribution Managed Investments Schedules (TRTAMIS) are provided, the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' must be 'True' (yes)",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MultiClassTreatment.Indicator",
                    Parameters = new ProcessMessageParameters {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000236" },
                        new ProcessMessageParameter { Name = "TRTAMI102", Value = ParentDocument.TRTAMI102.ToString() }
                    }
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000314

        //Rule ID: VR.ATO.TRTAMI.000314
        //IF[TRTAMI60] <> NULL AND COUNT(SCHEDULE = "TRTAMIS") > 0 AND ALL OCCURRENCES OF([TRTAMIS334]) = NULL
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOTRTAMI000314(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.TRTAMIS.ToString())) > 0 &&
                ParentDocument.TRTAMI60 != null &&
                ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.TRTAMIS.ToString())).
                Select(c => (TRTAMIS2022)c.ConsumedReport).All(t => t.TRTAMIS334 == null);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000314",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "At least one occurrence of Total Liabilities should be present if the corresponding label in AMIT tax return has a value",
                    Location = "/xbrli:xbrl/tns:Liabilities.Total.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000314" } }
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000315

        //Rule ID: VR.ATO.TRTAMI.000315
        //IF[TRTAMI58] <> NULL AND COUNT(SCHEDULE = "TRTAMIS") > 0 AND ALL OCCURRENCES OF([TRTAMIS333]) = NULL
        //  RETURN VALIDATION MESSAGE
        //ENDIF

        //[TRTAMI58] = TRTAMI:RP(Instant):bafpo1.02.13:Assets.Total.Amount
        //[TRTAMIS333] = TRTAMIS:RP(Instant):bafpo1.02.13:Assets.Total.Amount
        private void VRATOTRTAMI000315(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.TRTAMIS.ToString())) > 0 &&
                ParentDocument.TRTAMI58 != null &&
                ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.TRTAMIS.ToString())).
                Select(c => (TRTAMIS2022)c.ConsumedReport).All(t => t.TRTAMIS333 == null);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000315",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "At least one occurrence of Total Assets should be present if the corresponding label in AMIT tax return has a value",
                    Location = "/xbrli:xbrl/tns:Assets.Total.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000315" } }
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000235
        /*  VR.ATO.TRTAMI.000235
            IF COUNT(SCHEDULE = "CGTS") > 1 AND [CGTS153] = ANY OTHER OCCURRENCE OF [CGTS153]
               RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOTRTAMI000235(List<ProcessMessageDocument> response)
        {
            IEnumerable<CGTS2018> duplicates = ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.CGTS.ToString()))
                .Select(c => (CGTS2018)c.ConsumedReport).GroupBy(s => s.CGTS153).Where(s => s.Skip(1).Any())
                .SelectMany(c => c);

            response.AddRange(from duplicate in duplicates
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.TRTAMI.000235",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = "Name of AMIT Class for CGT schedule is invalid",
                                  LongDescription = "When multiple Capital Gains Tax (CGT) schedules are provided, a unique Name of AMIT Class must be provided for each",
                                  Location = "/xbrli:xbrl/xbrli:context",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000235" } }
                              });
        }
        #endregion // VR.ATO.TRTAMI.000235

        #region  VR.ATO.TRTAMI.000203
        /*
           Rule ID: VR.ATO.TRTAMI.000203
           IF COUNT(SCHEDULE = "PSS") > 1
             RETURN VALIDATION MESSAGE
           ENDIF
       */
        private void VRATOTRTAMI000203(List<ProcessMessageDocument> response)
        {
            bool assertion = (ChildDocuments.Count(c => c.DocumentName.Equals("PSS")) > 1);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000203",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of PAYG Payment Summary Schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "CMN.ATO.TRTAMI.000203" } }
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000174
        /* 
          Rule ID: VR.ATO.TRTAMI.000174
          IF COUNT(SCHEDULE = "TRTAMIS") = 0
            RETURN VALIDATION MESSAGE
          ENDIF
        */
        private void VRATOTRTAMI000174(List<ProcessMessageDocument> response)
        {
            bool assertion = !ChildDocuments.Any(c => c.DocumentName.Equals(Schedule.TRTAMIS.ToString()));

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000174",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"At least one Trust Attribution Managed Investments Schedule must be present",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000174" } }
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000240
        private void VRATOTRTAMI000240(List<ProcessMessageDocument> response)
        {
            var cgts153 = ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.CGTS.ToString()))
                .Select(c => (CGTS2018)c.ConsumedReport).Where(c => !string.IsNullOrWhiteSpace(c.CGTS153)).Select(c => c.CGTS153);

            if (cgts153.Any())
            {
                var trtamis313 = ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.TRTAMIS.ToString()))
                    .Select(t => (TRTAMIS2022)t.ConsumedReport).Select(t => t.TRTAMIS313);

                response.AddRange(cgts153.Where(c => !trtamis313.Any(c.Equals)).Select(c => new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000240",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Name of AMIT Class for CGT schedule is invalid",
                    LongDescription = "Name of AMIT Class provided on each Capital Gains Tax (CGT) schedule, must have a matching Trust Attribution Managed Investments Schedule (TRTAMIS)",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000240"},
                        new ProcessMessageParameter {Name = "CGTS153", Value = c}
                    }
                }));
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000151
        /* 
            Rule ID: VR.ATO.TRTAMI.000151

            IF([TRTAMI49] > 0) AND(COUNT(SCHEDULE = "IDS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF
        */
        protected void VRATOTRTAMI000151(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = !ChildDocuments.Any(c => c.DocumentName.Equals(Schedule.IDS.ToString()))
                && ParentDocument.TRTAMI49 > 0;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000151",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule must be attached",
                    LongDescription = @"An International Dealings Schedule (IDS) must be attached when an Interest Expenses Overseas amount above zero is provided",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000151" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI000152
        /*  
        Rule ID: VR.ATO.TRTAMI.000152

        IF([TRTAMI50] = SET(NULL, 0)) AND(COUNT(SCHEDULE = "IDS") = 1) AND([IDS22] > 0)
            RETURN VALIDATION MESSAGE
        ENDIF
        -->*/
        protected void VRATOTRTAMI000152(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.IDS.ToString())) == 1
                && (ParentDocument.TRTAMI50 == 0 || ParentDocument.TRTAMI50 == null)
                && ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.IDS.ToString())).Select(c => (IDS2022)c.ConsumedReport).All(i => i.IDS22 > 0);

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000152",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalty expenses overseas required",
                    LongDescription = @"When a Royalties deduction amount above zero is present on the International Dealings Schedule (IDS), a Royalty Expenses Overseas amount above zero must also be provided",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000152" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000153

        /*    
        IF([TRTAMI50] > 0) AND(COUNT(SCHEDULE = "IDS") = 1) AND([TRTAMI50] < [IDS22])
        RETURN VALIDATION MESSAGE
        ENDIF
        [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
        [TRTAMI50] = TRTAMI:RP:bafpr1.02.27:Expense.Royalties.Amount
        */
        protected void VRATOTRTAMI000153(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;
            var ids22 = ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.IDS.ToString()))
               .Select(c => (IDS2022)c.ConsumedReport).Where(c => (c.IDS22) != null).Select(c => c.IDS22);

            assertion = ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.IDS.ToString())) == 1
                && (ParentDocument.TRTAMI50 < ids22.FirstOrDefault())
                && (ParentDocument.TRTAMI50 > 0);

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000153",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalties deductions incorrect",
                    LongDescription = @"When a Royalties deduction amount is present on the International Dealings Schedule (IDS), it must not exceed the Royalty Expenses Overseas amount",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000153" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000154
        /*
        IF([TRTAMI50] > 0) AND(COUNT(SCHEDULE = "IDS") = 0)
           RETURN VALIDATION MESSAGE
        ENDIF
        [TRTAMI50] = TRTAMI:RP:bafpr1.02.27:Expense.Royalties.Amount
        */
        protected void VRATOTRTAMI000154(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.IDS.ToString())) == 0
                && (ParentDocument.TRTAMI50 > 0);

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000154",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule must be attached",
                    LongDescription = @"An International Dealings Schedule (IDS) must be attached when a Royalty Expenses Overseas amount above zero is provided",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000154" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000199
        /*
        IF ([TRTAMI47] = FALSE AND [TRTAMI48] = FALSE) AND ([TRTAMI49] = SET(NULL,0) AND [TRTAMI50] = SET(NULL,0)) AND (COUNT(Schedule = ""IDS"") = 1)
           RETURN VALIDATION MESSAGE
        ENDIF
        [TRTAMI47] = TRTAMI:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
        [TRTAMI48] = TRTAMI:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
        [TRTAMI49] = TRTAMI:RP:bafpr1.02.05:Expense.Interest.Amount
        [TRTAMI50] = TRTAMI:RP:bafpr1.02.27:Expense.Royalties.Amount
        */
        protected void VRATOTRTAMI000199(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ParentDocument.TRTAMI47 == false && ParentDocument.TRTAMI48 == false
                    && (ParentDocument.TRTAMI49 == null || ParentDocument.TRTAMI49 == 0)
                    && (ParentDocument.TRTAMI50 == null || ParentDocument.TRTAMI50 == 0)
                    && ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.IDS.ToString())) == 1;

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000199",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not required",
                    LongDescription = @"When the value of dealings with international related parties was not above $2 million, thin capitalisation provisions did not affect you, and no amounts have been provided at Interest expenses overseas and Royalty expenses overseas, an International Dealings schedule must not be attached",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000199" } },
                };

                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000201
        /* 
        IF COUNT(SCHEDULE = "IDS") > 1
           RETURN VALIDATION MESSAGE
        ENDIF
        */
        protected void VRATOTRTAMI000201(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.IDS.ToString())) > 1;

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000201",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of International Dealings Schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000201" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000205

        /*
        IF [TRTAMI137] = FALSE AND [TRTAMI138] = FALSE AND COUNT(SCHEDULE = ""IDS"") = 1 AND [IDS9] = TRUE
        RETURN VALIDATION MESSAGE
        ENDIF
        [IDS9] = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
        [TRTAMI137] = TRTAMI:RP:rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
        [TRTAMI138] = TRTAMI:RP:rvctc4.02.11:InternationalDealings.TransactionControlTaxHavenCountries.Indicator 
        */
        protected void VRATOTRTAMI000205(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;
            var ids9 = ChildDocuments.Where(s => s.DocumentName.Equals(Schedule.IDS.ToString()))
               .Select(c => (IDS2022)c.ConsumedReport).Where(c => (c.IDS9) != null).Select(c => c.IDS9);


            assertion =
                ParentDocument.TRTAMI137 == false && ParentDocument.TRTAMI138 == false
                && ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.IDS.ToString())) == 1
                && ids9.FirstOrDefault() == true;

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000205",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Transactions with specified countries question incorrect",
                    LongDescription = "When the response to 'Did you have dealings with international related parties located in specified countries?' on the International Dealings Schedule (IDS) is 'True' (yes), the response at either 'Funds or property, whether directly or indirectly, have been sent to, or received from, any of the specified countries', or 'Trustee has the ability or expectation to control, whether directly or indirectly, the disposition of any funds, property, assets or investments located in, or located elsewhere but controlled or managed from any of the specified countries', must also be 'True' (yes)",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000205" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000246
        /*
        IF [TRTAMI47] = TRUE AND COUNT(SCHEDULE = ""IDS"") = 0
        RETURN VALIDATION MESSAGE
        ENDIF
        [TRTAMI47] = TRTAMI:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
        */
        protected void VRATOTRTAMI000246(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ParentDocument.TRTAMI47 == true &&
                        ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.IDS.ToString())) == 0;

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000246",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule (IDS) must be attached",
                    LongDescription = "When the value of dealings with international related parties was above $2 million, the International Dealings Schedule (IDS) must be attached",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000246" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.TRTAMI.000330

        /*  VR.ATO.TRTAMI.000330
                Small business entity question on the International Dealings Schedule (IDS) form must not be 'TRUE'.

            Legacy Rule Format:
                IF [TRTAMI162] = TRUE AND (COUNT(SCHEDULE = "IDS") = 1 AND [IDS425] = TRUE)
                    RETURN VALIDATION MESSAGE
                ENDIF

            Data Elements:
            
                [TRTAMI162] = TRTAMI:RP:pyde.02.43:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
                [IDS425] = IDS:RP:rvctc4.02.18:InternationalDealings.SmallBusinessConcessionsEligibility.Indicator

        */
        private void VRATOTRTAMI000330(List<ProcessMessageDocument> response)
        {
            if (ParentDocument.TRTAMI162.GetValueOrDefault() == true &&
               IDSChildDocuments.Count() == 1 &&
               IDSChildDocuments.FirstOrDefault().IDS425.GetValueOrDefault() == true)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000330",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business entity question on the International Dealings Schedule (IDS) form must not be 'TRUE'.",
                    LongDescription = @"If the trust has self-assessed that they are a Significant Global Entity (SGE), then 'Are you a small business entity, not a significant global entity and your international related party dealings do not exceed $5 million and 50% of your current year aggregated turnover?' must not be 'TRUE' on the International Dealings Schedule (IDS) form.",
                    Location = $"/xbrli:xbrl/tns:OrganisationDetails.SignificantGlobalEntityStatus.Indicator",
                    Parameters = new ProcessMessageParameters
                {
                    new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000330" }
                }
                });
            }
        }
        #endregion

        #region VR.ATO.GEN438017
        /*
            IF(RP:bafpo6.xx.xx:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator = TRUE) AND(COUNT(SCHEDULE = "IDS") = 0)
            RETURN VALIDATION MESSAGE
            ENDIF
        */
        protected void VRATOGEN438017(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ParentDocument.TRTAMI48 == true &&
                        ChildDocuments.Count(c => c.DocumentName.Equals(Schedule.IDS.ToString())) == 0;

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = "If the thin capitalisation rules apply  then International Dealings Schedule must be attached",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438017" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion


    }
}