using System.Linq;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using DataContracts;

namespace Ato.EN.IntegrationServices.CodeGenerationTRTAMI
{
    public partial class TRTAMI2022Validator
    {
        #region VR.ATO.TRTAMI.000004
        /*  VR.ATO.TRTAMI.000004
                    The Sequence Container within the Statement of Attribution Sequence Dimension context instance must not be duplicated

                Legacy Rule Format:
                    IF RP.{SoASeqNum}:SequenceContainer = ANY OTHER OCCURRENCE OF RP.{SoASeqNum}:SequenceContainer
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    RP.{SoASeqNum}:SequenceContainer = ANY OTHER OCCURRENCE OF RP.{SoASeqNum}:SequenceContainer

        Data Elements:
        */
        protected void VRATOTRTAMI000004(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            bool assertion = report.RPSoASeqNumContextCollection == null ? false :
                                report.RPSoASeqNumContextCollection.GroupBy(seq => seq.SequenceNumberDimensionValue)
                                    .Any(seq => seq.Count() > 1);
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000004",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Sequence Container must not be duplicated",
                    LongDescription = "The Sequence Container within the Statement of Attribution Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSoASeqNumContextCollection == null ? 0 : report.RPSoASeqNumContextCollection.Last().LastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000004" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRTAMI.000004

        #region VR.ATO.TRTAMI.000190

        /*  VR.ATO.TRTAMI.000190
        If all Trustee Liabilities amounts and all Statement of Attribution (SA) Taxable Determined Member Component amounts are either blank or zero, then 'Is any tax payable by the trustee?' must be 'no'.

        Legacy Rule Format:
        IF [TRTAMI33] <> "0" AND (([TRTAMI35] = NULL OR [TRTAMI35] = 0) AND ([TRTAMI36] = NULL OR [TRTAMI36] = 0) AND ([TRTAMI39] = NULL OR [TRTAMI39] = 0) AND ([TRTAMI40] = NULL OR [TRTAMI40] = 0) AND ALL OCCURRENCES OF ([TRTAMI82] = NULL OR [TRTAMI82] = 0))
           RETURN VALIDATION MESSAGE
        ENDIF

        [TRTAMI33] = TRTAMI:RP:pyin.02.00:Report.Statement.Summary.PaymentRefundOrNil.Code
        [TRTAMI35] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustComponentDeficitOffset.Amount
        [TRTAMI36] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustShortfallIncome.Amount
        [TRTAMI39] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustExcessOffset.Amount
        [TRTAMI40] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustComponentNotReflected.Amount
        [TRTAMI82] = TRTAMI:RP.{SoASeqNum}:rvctc4.02.10:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount

        Technical Business Rule Format:
        ^TRTAMI33 <> '0' AND ((^TRTAMI35 = NULL OR ^TRTAMI35 = 0) AND (^TRTAMI36 = NULL OR ^TRTAMI36 = 0) AND (^TRTAMI39 = NULL OR ^TRTAMI39 = 0) AND (^TRTAMI40 = NULL OR ^TRTAMI40 = 0) AND
        (^TRTAMI82 = NULL OR ^TRTAMI82 = 0))
        Data Elements:
        RP:^TRTAMI33 = tns:Report.Statement.Summary.PaymentRefundOrNil.Code
        RP:^TRTAMI35 = tns:IncomeTax.TrustComponentDeficitOffset.Amount
        RP:^TRTAMI36 = tns:IncomeTax.TrustShortfallIncome.Amount
        RP:^TRTAMI39 = tns:IncomeTax.TrustExcessOffset.Amount
        RP:^TRTAMI40 = tns:IncomeTax.TrustComponentNotReflected.Amount
        RP.{SoASeqNum}:^TRTAMI82 = tns:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount        */
        protected void VRATOTRTAMI000190(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {

            bool assertion = (
                    report.TRTAMI33 != @"0"
                    && (
                        (report.TRTAMI35 == null || report.TRTAMI35 == 0)
                        &&
                        (report.TRTAMI36 == null || report.TRTAMI36 == 0)
                        &&
                        (report.TRTAMI39 == null || report.TRTAMI39 == 0)
                        &&
                        (report.TRTAMI40 == null || report.TRTAMI40 == 0)
                        &&
                        (
                            report.RPSoASeqNumContextCollection == null ||
                            report.RPSoASeqNumContextCollection.Sum(seq => seq.TRTAMI82) == 0)
                         )
                       );

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000190",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Is any tax payable by the Trustee?' must be set to ""0"" (Nil)",
                    LongDescription = @"When no positive values have been provided for any Trustee Liabilities amounts or any Statement of Attribution (SA) Taxable Determined Member Component amounts, the 'Is any tax payable by the trustee?' must be set to ""0"" (Nil)",
                    Location = "/xbrli:xbrl/tns:Report.Statement.Summary.PaymentRefundOrNil.Code",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000190" },
                        new ProcessMessageParameter { Name = "TRTAMI33", Value = report.TRTAMI33 },
                        new ProcessMessageParameter { Name = "TRTAMI35", Value = report.TRTAMI35.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "TRTAMI36", Value = report.TRTAMI36.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "TRTAMI39", Value = report.TRTAMI39.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "TRTAMI40", Value = report.TRTAMI40.GetValueOrDefault().ToString() }
                    },
                };

                /*
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI82", Value = report.RPSoASeqNumRESContextCollection.TRTAMI82.GetValueOrDefault().ToString() });
                */

                response.Add(processMessage);
            }

        }
        #endregion // VR.ATO.TRTAMI.000190

        #region VR.ATO.TRTAMI.000249

        /*  VR.ATO.TRTAMI.000249
                    Address Line 1 must be present

                Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                    IF [TRTAMI125] = NULLORBLANK AND [TRTAMI77] <> NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF

                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                    [TRTAMI125] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
                    ^TRTAMI125 = BLANK AND ^TRTAMI77 <> BLANK

        Data Elements:

                    RP.{SoASeqNum}.RES:^TRTAMI125 = tns:AddressDetails.Line1.Text

                    RP.{SoASeqNum}:^TRTAMI77 = tns:PersonNameDetails.FamilyName.Text
                    */
        protected void VRATOTRTAMI000249(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            if (report.RPSoASeqNumContextCollection != null && report.RPSoASeqNumRESContextCollection != null)
                response.AddRange(report.RPSoASeqNumContextCollection.Join(report.RPSoASeqNumRESContextCollection,
                    r => r.SequenceNumberDimensionValue, s => s.SequenceNumberDimensionValue,
                    (r, s) => new { rp = r, res = s }).Where(t =>
                      string.IsNullOrWhiteSpace(t.res.TRTAMI125) && !string.IsNullOrWhiteSpace(t.rp.TRTAMI77)).Select(t =>
                      new ProcessMessageDocument
                      {
                          Code = "CMN.ATO.TRTAMI.000249",
                          Severity = ProcessMessageSeverity.Error,
                          Description = "Member Residential Address - Address Line 1 must be present for an individual",
                          Location = $"/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='{t.res.Id}']",
                          Parameters = new ProcessMessageParameters
                          {
                            new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000249"},
                            new ProcessMessageParameter {Name = "TRTAMI77", Value = t.rp.TRTAMI77},
                            new ProcessMessageParameter {Name = "TRTAMI125", Value = t.res.TRTAMI125}
                          }
                      }));
        }
        #endregion // VR.ATO.TRTAMI.000249

        #region VR.ATO.TRTAMI.000258

        /*  VR.ATO.TRTAMI.000258
                    Suburb/Town/City is mandatory

                Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                    IF [TRTAMI127] = NULLORBLANK AND [TRTAMI77] <> NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                    [TRTAMI127] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.LocalityName.Text

            Technical Business Rule Format:
                    ^TRTAMI127 = BLANK AND ^TRTAMI77 <> BLANK

        Data Elements:
            
                    RP.{SoASeqNum}.RES:^TRTAMI127 = tns:AddressDetails.LocalityName.Text
            
                    RP.{SoASeqNum}:^TRTAMI77 = tns:PersonNameDetails.FamilyName.Text
                    */
        protected void VRATOTRTAMI000258(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            if (report.RPSoASeqNumContextCollection != null && report.RPSoASeqNumRESContextCollection != null)
                response.AddRange(report.RPSoASeqNumContextCollection.Join(report.RPSoASeqNumRESContextCollection,
                    r => r.SequenceNumberDimensionValue, s => s.SequenceNumberDimensionValue,
                    (r, s) => new { rp = r, res = s }).Where(t =>
                    string.IsNullOrWhiteSpace(t.res.TRTAMI127) && !string.IsNullOrWhiteSpace(t.rp.TRTAMI77)).Select(t =>
                    new ProcessMessageDocument
                    {
                        Code = "CMN.ATO.TRTAMI.000258",
                        Severity = ProcessMessageSeverity.Error,
                        Description = "Member Residential Address - City/Town must be provided for an individual",
                        Location = $"/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='{t.res.Id}']",
                        Parameters = new ProcessMessageParameters
                        {
                            new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000258"},
                            new ProcessMessageParameter {Name = "TRTAMI77", Value = t.rp.TRTAMI77}
                        }
                    }
                ));
        }
        #endregion // VR.ATO.TRTAMI.000258

        #region VR.ATO.TRTAMI.000264

        /*  VR.ATO.TRTAMI.000264
        Invalid context. The number of Residential contexts per Reporting Party Statement of Attribution context must not exceed 1.

        Legacy Rule Format:
        WHERE IN CONTEXT(RP.{SoASeqNum})
        IF COUNT(CONTEXT(RP.{SoASeqNum}.RES)) > 1
           RETURN VALIDATION MESSAGE
        ENDIF

        Technical Business Rule Format:
        Count(^Context) > 1

        Data Elements:

        ^Context = RP.{SoASeqNum}:RP.{SoASeqNum}.RES
        */
        protected void VRATOTRTAMI000264(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {



            if (report.RPSoASeqNumContextCollection != null && Count(report.RPSoASeqNumContextCollection) > 0)
            {
                foreach (TRTAMI2022.RPSoASeqNumContext seqNum in report.RPSoASeqNumContextCollection)
                {
                    int resCount = 0;
                    if (report.RPSoASeqNumRESContextCollection != null)
                    {
                        foreach (TRTAMI2022.RPSoASeqNumRESContext res in report.RPSoASeqNumRESContextCollection)
                        {
                            if (seqNum.SequenceNumberDimensionValue.ToString() == res.SequenceNumberDimensionValue.ToString())
                                resCount++;
                        }
                    }
                    if (resCount > 1)
                    {
                        ProcessMessageDocument processMessage = new ProcessMessageDocument
                        {
                            Code = "CMN.ATO.TRTAMI.000264",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid context. The number of Residential contexts per Reporting Party Statement of Attribution context must not exceed 1.",
                            Location = $"/xbrli:xbrl/xbrli:context[@id='" + seqNum.Id.ToString() + "']",
                            Parameters = new ProcessMessageParameters
                          {
                            new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000264"}
                          }
                        };
                        response.Add(processMessage);
                    }
                }
            }
        }
        #endregion // VR.ATO.TRTAMI.000264

        #region VR.ATO.TRTAMI.000265

        /*  VR.ATO.TRTAMI.000265
            Invalid context. The number of Business contexts per Reporting Party Statement of Attribution context must not exceed 1.
    
            Legacy Rule Format:
            WHERE IN CONTEXT(RP.{SoASeqNum})
            IF COUNT(CONTEXT(RP.{SoASeqNum}.BUS)) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.{SoASeqNum}:RP.{SoASeqNum}.BUS
            */
        protected void VRATOTRTAMI000265(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            if (report.RPSoASeqNumContextCollection != null && Count(report.RPSoASeqNumContextCollection) > 0)
            {
                foreach (TRTAMI2022.RPSoASeqNumContext seqNum in report.RPSoASeqNumContextCollection)
                {
                    int busCount = 0;
                    if (report.RPSoASeqNumBUSContextCollection != null)
                    {
                        foreach (TRTAMI2022.RPSoASeqNumBUSContext bus in report.RPSoASeqNumBUSContextCollection)
                        {
                            if (seqNum.SequenceNumberDimensionValue.ToString() == bus.SequenceNumberDimensionValue.ToString())
                                busCount++;
                        }
                    }
                    if (busCount > 1)
                    {
                        ProcessMessageDocument processMessage = new ProcessMessageDocument
                        {
                            Code = "CMN.ATO.TRTAMI.000265",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid context. The number of Business contexts per Reporting Party Statement of Attribution context must not exceed 1.",
                            Location = $"/xbrli:xbrl/xbrli:context[@id='" + seqNum.Id.ToString() + "']",
                            Parameters = new ProcessMessageParameters
                          {
                            new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000265"}
                          }
                        };
                        response.Add(processMessage);
                    }
                }
            }
        }
        #endregion // VR.ATO.TRTAMI.000265

        #region VR.ATO.TRTAMI.000268

        /*  VR.ATO.TRTAMI.000268
                    Address Line 1 must be present

                Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                    IF [TRTAMI131] = NULLORBLANK AND [TRTAMI80] <> NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF


                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    [TRTAMI131] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
                    ^TRTAMI131 = BLANK AND ^TRTAMI80 <> BLANK

        Data Elements:

                    RP.{SoASeqNum}.BUS:^TRTAMI131 = tns:AddressDetails.Line1.Text

                    RP.{SoASeqNum}:^TRTAMI80 = tns:OrganisationNameDetails.OrganisationalName.Text
                    */
        protected void VRATOTRTAMI000268(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            if (report.RPSoASeqNumContextCollection != null && report.RPSoASeqNumBUSContextCollection != null)
                response.AddRange(report.RPSoASeqNumContextCollection.Join(report.RPSoASeqNumBUSContextCollection,
                    r => r.SequenceNumberDimensionValue, s => s.SequenceNumberDimensionValue,
                    (r, s) => new { rp = r, bus = s }).Where(t =>
                    string.IsNullOrWhiteSpace(t.bus.TRTAMI131) && !string.IsNullOrWhiteSpace(t.rp.TRTAMI80)).Select(t =>
                    new ProcessMessageDocument
                    {
                        Code = "CMN.ATO.TRTAMI.000268",
                        Severity = ProcessMessageSeverity.Error,
                        Description = "Member Business Address - Address Line 1 must be present for a non-individual",
                        Location = $"/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='{t.bus.Id}']",
                        Parameters = new ProcessMessageParameters
                        {
                            new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000268"},
                            new ProcessMessageParameter {Name = "TRTAMI80", Value = t.rp.TRTAMI80}
                        }
                    }
                ));
        }
        #endregion // VR.ATO.TRTAMI.000268

        #region VR.ATO.TRTAMI.000277

        /*  VR.ATO.TRTAMI.000277
                    Suburb/Town/City is mandatory

                Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                    IF [TRTAMI133] = NULLORBLANK AND [TRTAMI80] <> NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    [TRTAMI133] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.LocalityName.Text

            Technical Business Rule Format:
                    ^TRTAMI133 = BLANK AND ^TRTAMI80 <> BLANK

        Data Elements:
            
                    RP.{SoASeqNum}.BUS:^TRTAMI133 = tns:AddressDetails.LocalityName.Text
            
                    RP.{SoASeqNum}:^TRTAMI80 = tns:OrganisationNameDetails.OrganisationalName.Text
                    */
        protected void VRATOTRTAMI000277(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            if (report.RPSoASeqNumContextCollection != null && report.RPSoASeqNumBUSContextCollection != null)
                response.AddRange(report.RPSoASeqNumContextCollection.Join(report.RPSoASeqNumBUSContextCollection,
                    r => r.SequenceNumberDimensionValue, s => s.SequenceNumberDimensionValue,
                    (r, s) => new { rp = r, bus = s }).Where(t =>
                    string.IsNullOrWhiteSpace(t.bus.TRTAMI133) && !string.IsNullOrWhiteSpace(t.rp.TRTAMI80)).Select(t =>
                    new ProcessMessageDocument
                    {
                        Code = "CMN.ATO.TRTAMI.000277",
                        Severity = ProcessMessageSeverity.Error,
                        Description = "Member Business Address - City/Town must be provided for a non-individual",
                        Location = $"/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='{t.bus.Id}']",
                        Parameters = new ProcessMessageParameters
                        {
                            new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000277"},
                            new ProcessMessageParameter {Name = "TRTAMI80", Value = t.rp.TRTAMI80}
                        }
                    }
                ));
        }
        #endregion // VR.ATO.TRTAMI.000277

        #region VR.ATO.TRTAMI.000305

        /*  VR.ATO.TRTAMI.000305
                    For an Australian address, Postal/Zip Code is mandatory

                Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                    IF [TRTAMI143] = NULLORBLANK AND [TRTAMI77] <> NULLORBLANK AND ([TRTAMI130] = NULL OR [TRTAMI130] = "au")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                    [TRTAMI130] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.08:AddressDetails.Country.Code
                    [TRTAMI143] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.27:AddressDetails.PostalCode.Text

            Technical Business Rule Format:
                    ^TRTAMI143 = BLANK AND ^TRTAMI77 <> BLANK AND (^TRTAMI130 = NULL OR ^TRTAMI130 = 'au')

        Data Elements:
            
                    RP.{SoASeqNum}.RES:^TRTAMI143 = tns:AddressDetails.PostalCode.Text
            
                    RP.{SoASeqNum}:^TRTAMI77 = tns:PersonNameDetails.FamilyName.Text
            
                    RP.{SoASeqNum}.RES:^TRTAMI130 = tns:AddressDetails.Country.Code
                    */
        protected void VRATOTRTAMI000305(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            if (report.RPSoASeqNumContextCollection != null && report.RPSoASeqNumRESContextCollection != null)
                response.AddRange(report.RPSoASeqNumContextCollection.Join(report.RPSoASeqNumRESContextCollection,
                    r => r.SequenceNumberDimensionValue, s => s.SequenceNumberDimensionValue,
                    (r, s) => new { rp = r, res = s }).Where(t =>
                      string.IsNullOrWhiteSpace(t.res.TRTAMI143) && !string.IsNullOrWhiteSpace(t.rp.TRTAMI77) &&
                      (t.res.TRTAMI130 == null || t.res.TRTAMI130.ToLower().Equals("au"))).Select(t =>
                      new ProcessMessageDocument
                      {
                          Code = "CMN.ATO.TRTAMI.000305",
                          Severity = ProcessMessageSeverity.Error,
                          Description = "Member Residential Address - Postal/Zip Code is required for an Australian address",
                          LongDescription = @"For an Australian address, Member Residential Address - Postal/Zip Code is required for an individual. Determine whether the address is Australian or overseas and correct accordingly.",
                          Location = $"/xbrli:xbrl/tns:AddressDetails.PostalCode.Text[@contextRef='{t.res.Id}']",
                          Parameters = new ProcessMessageParameters
                          {
                            new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000305"},
                            new ProcessMessageParameter {Name = "TRTAMI77", Value = t.rp.TRTAMI77}
                          }
                      }
                ));
        }
        #endregion // VR.ATO.TRTAMI.000305

        #region VR.ATO.TRTAMI.000306

        /*  VR.ATO.TRTAMI.000306
                    For an Australian address, Member Residential Address - Postal/Zip Code is invalid. Valid range is 0200 - 9999.

                Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                    IF [TRTAMI77] <> NULLORBLANK AND ([TRTAMI130] = NULL OR [TRTAMI130] = "au") AND [TRTAMI143] <> NULLORBLANK AND (LENGTH([TRTAMI143]) <> 4 OR [TRTAMI143] <> SET(0200-9999))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                    [TRTAMI130] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.08:AddressDetails.Country.Code
                    [TRTAMI143] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.27:AddressDetails.PostalCode.Text

            Technical Business Rule Format:
                    ^TRTAMI77 <> BLANK AND (^TRTAMI130 = NULL OR ^TRTAMI130 = 'au') AND ^TRTAMI143 <> BLANK AND (Length(^TRTAMI143) <> 4 OR NotInSet(^TRTAMI143, '0200-9999'))

        Data Elements:
            
                    RP.{SoASeqNum}.RES:^TRTAMI143 = tns:AddressDetails.PostalCode.Text
            
                    RP.{SoASeqNum}:^TRTAMI77 = tns:PersonNameDetails.FamilyName.Text
            
                    RP.{SoASeqNum}.RES:^TRTAMI130 = tns:AddressDetails.Country.Code
                    */
        protected void VRATOTRTAMI000306(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            if (report.RPSoASeqNumContextCollection != null && report.RPSoASeqNumRESContextCollection != null)
                response.AddRange(report.RPSoASeqNumContextCollection.Join(report.RPSoASeqNumRESContextCollection,
                        r => r.SequenceNumberDimensionValue, s => s.SequenceNumberDimensionValue,
                        (r, s) => new { rp = r, res = s }).Where(t =>
                          !string.IsNullOrWhiteSpace(t.rp.TRTAMI77) &&
                          (t.res.TRTAMI130 == null || t.res.TRTAMI130.ToLower().Equals("au")) &&
                          (!string.IsNullOrWhiteSpace(t.res.TRTAMI143) && Length(t.res.TRTAMI143) != 4 ||
                          !IsMatch(t.res.TRTAMI143, @"^([0][2-9][0-9][0-9]|[1-9][0-9][0-9][0-9])*$", RegexOptions.IgnoreCase)))
                    .Select(t =>
                        new ProcessMessageDocument
                        {
                            Code = "CMN.ATO.TRTAMI.000306",
                            Severity = ProcessMessageSeverity.Error,
                            Description = "Member Residential Address - Postal/Zip Code is invalid",
                            LongDescription = "For an Australian address, Member Residential Address - Postal/Zip Code is invalid. Valid range is 0200 - 9999.",
                            Location = $"/xbrli:xbrl/tns:AddressDetails.PostalCode.Text[@contextRef='{t.res.Id}']",
                            Parameters = new ProcessMessageParameters
                            {
                                new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000306"},
                                new ProcessMessageParameter {Name = "TRTAMI143", Value = t.res.TRTAMI143}
                            }
                        }
                    ));
        }
        #endregion // VR.ATO.TRTAMI.000306

        #region VR.ATO.TRTAMI.000308

        /*  VR.ATO.TRTAMI.000308
                    For an Australian address, Postal/Zip Code is mandatory

                Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                    IF [TRTAMI144] = NULLORBLANK AND [TRTAMI80] <> NULLORBLANK AND ([TRTAMI136] = NULL OR [TRTAMI136] = "au")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    [TRTAMI136] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.08:AddressDetails.Country.Code
                    [TRTAMI144] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.27:AddressDetails.PostalCode.Text

            Technical Business Rule Format:
                    ^TRTAMI144 = BLANK AND ^TRTAMI80 <> BLANK AND (^TRTAMI136 = NULL OR ^TRTAMI136 = 'au')

        Data Elements:
            
                    RP.{SoASeqNum}.BUS:^TRTAMI144 = tns:AddressDetails.PostalCode.Text
            
                    RP.{SoASeqNum}:^TRTAMI80 = tns:OrganisationNameDetails.OrganisationalName.Text
            
                    RP.{SoASeqNum}.BUS:^TRTAMI136 = tns:AddressDetails.Country.Code
                    */
        protected void VRATOTRTAMI000308(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            if (report.RPSoASeqNumContextCollection != null && report.RPSoASeqNumBUSContextCollection != null)
                response.AddRange(report.RPSoASeqNumContextCollection.Join(report.RPSoASeqNumBUSContextCollection,
                    r => r.SequenceNumberDimensionValue, s => s.SequenceNumberDimensionValue,
                    (r, s) => new { rp = r, bus = s }).Where(t =>
                    string.IsNullOrWhiteSpace(t.bus.TRTAMI144) && !string.IsNullOrWhiteSpace(t.rp.TRTAMI80) &&
                    (t.bus.TRTAMI136 == null || t.bus.TRTAMI136.ToLower().Equals("au"))).Select(t =>
                    new ProcessMessageDocument
                    {
                        Code = "CMN.ATO.TRTAMI.000308",
                        Severity = ProcessMessageSeverity.Error,
                        Description = "Member Business Address - Postal/Zip Code is required for an Australian address",
                        LongDescription = "For an Australian address, Member Business Address - Postal/Zip Code is required for a non-individual. Determine whether the address is Australian or overseas and correct accordingly.",
                        Location = $"/xbrli:xbrl/tns:AddressDetails.PostalCode.Text[@contextRef='{t.bus.Id}']",
                        Parameters = new ProcessMessageParameters
                        {
                            new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000308"},
                            new ProcessMessageParameter {Name = "TRTAMI80", Value = t.rp.TRTAMI80}
                        }
                    }
                ));
        }
        #endregion // VR.ATO.TRTAMI.000308

        #region VR.ATO.TRTAMI.000309

        /*  VR.ATO.TRTAMI.000309
                    For an Australian address, Member Business Address - Postal/Zip Code is invalid. Valid range is 0200 - 9999.

                Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                    IF [TRTAMI80] <> NULLORBLANK AND ([TRTAMI136] = NULL OR [TRTAMI136] = "au") AND [TRTAMI144] <> NULLORBLANK AND (LENGTH([TRTAMI144]) <> 4 OR [TRTAMI144] <> SET(0200-9999))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    [TRTAMI136] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.08:AddressDetails.Country.Code
                    [TRTAMI144] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.27:AddressDetails.PostalCode.Text

            Technical Business Rule Format:
                    ^TRTAMI80 <> BLANK AND (^TRTAMI136 = NULL OR ^TRTAMI136 = 'au') AND ^TRTAMI144 <> BLANK AND (Length(^TRTAMI144) <> 4 OR NotInSet(^TRTAMI144, '0200-9999'))

        Data Elements:
            
                    RP.{SoASeqNum}.BUS:^TRTAMI144 = tns:AddressDetails.PostalCode.Text
            
                    RP.{SoASeqNum}:^TRTAMI80 = tns:OrganisationNameDetails.OrganisationalName.Text
            
                    RP.{SoASeqNum}.BUS:^TRTAMI136 = tns:AddressDetails.Country.Code
                    */
        protected void VRATOTRTAMI000309(List<ProcessMessageDocument> response, TRTAMI2022 report)
        {
            if (report.RPSoASeqNumContextCollection != null && report.RPSoASeqNumBUSContextCollection != null)
                response.AddRange(report.RPSoASeqNumContextCollection.Join(report.RPSoASeqNumBUSContextCollection,
                        r => r.SequenceNumberDimensionValue, s => s.SequenceNumberDimensionValue,
                        (r, s) => new { rp = r, bus = s }).Where(t =>
                          !string.IsNullOrWhiteSpace(t.rp.TRTAMI80) &&
                          (t.bus.TRTAMI136 == null || t.bus.TRTAMI136.ToLower().Equals("au")) &&
                          (!string.IsNullOrWhiteSpace(t.bus.TRTAMI144) && (Length(t.bus.TRTAMI144) != 4 ||
                          !IsMatch(t.bus.TRTAMI144, @"^([0][2-9][0-9][0-9]|[1-9][0-9][0-9][0-9])*$", RegexOptions.IgnoreCase))))
                    .Select(t =>
                        new ProcessMessageDocument
                        {
                            Code = "CMN.ATO.TRTAMI.000309",
                            Severity = ProcessMessageSeverity.Error,
                            Description = "Member Business Address - Postal/Zip Code is invalid",
                            LongDescription = "For an Australian address, Member Business Address - Postal/Zip Code is invalid. Valid range is 0200 - 9999.",
                            Location = $"/xbrli:xbrl/tns:AddressDetails.PostalCode.Text[@contextRef='{t.bus.Id}']",
                            Parameters = new ProcessMessageParameters
                            {
                                new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000309"},
                                new ProcessMessageParameter {Name = "TRTAMI80", Value = t.rp.TRTAMI80},
                                new ProcessMessageParameter {Name = "TRTAMI144", Value = t.bus.TRTAMI144}
                            }
                        }
                    ));
        }
        #endregion // VR.ATO.TRTAMI.000309
    }
}
