using System;
using System.IO;
using System.Runtime.CompilerServices;
using System.Text;
using System.Xml;
using System.Linq;
using System.Collections.Generic;
using DataContracts;
using System.Text.RegularExpressions;

namespace Ato.EN.IntegrationServices.CodeGenerationTRTAMI
{

    public partial class TRTAMI2022Validator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="TRTAMI2022Validator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public TRTAMI2022Validator(int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }


        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString().ToLower() : string.Empty;
        }

        private static string GetValueOrEmptyDate(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-dd") : string.Empty;
        }

        private static string GetValueOrEmptyDate(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-dd");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-ddTHH:mm:ssZ");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-ddTHH:mm:ssZ") : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Get string value between [first] a and [last] b.
        /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

        /// <summary>
        /// Get string value after [first] a.
        /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

        /// <summary>
        /// Get string value after [last] a.
        /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
                return 0;
            else
                return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return (from x in nodes select x).Distinct().Count() != 1;
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            nodes = nodes.Where(x => x != null).ToArray();
            return !((from x in nodes select x).Distinct().Count() == nodes.Count());

        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

        public DateTime Date(DateTime? datetime)
        {
            return datetime.GetValueOrDefault().Date;
        }

        //The logic in After function expects "---" for day and "--" for month. 
        //Since hyphen was missing the date always returned null
        public DateTime? ConvertToDate(int day, int month, int year)
        {
            return ConvertToDate(day == 0 ? null : "---" + day.ToString(), month == 0 ? null : "--" + month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }

        private static DateTime? AddMonthsToDate(DateTime? dateTime, int months)
        {
            return dateTime == null ? null : (DateTime?)dateTime.Value.AddMonths(months);
        }

        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response = true;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length == 14)
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 3), out numeric))
                        response = false;

                }
                else if (usi.Length == 9)
                {

                    if (Regex.IsMatch(usi, @"^([a-zA-Z]{3}\d{4}[a-zA-Z]{2})$"))
                        response = false;
                }
            }
            return response;
        }

        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
            return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
            decimal numberValue;
            decimal.TryParse(value, out numberValue);
            return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':
                        options = options | RegexOptions.IgnoreCase;
                        break;
                    case 'm':
                        options = options | RegexOptions.Multiline;
                        break;
                    case 's':
                        options = options | RegexOptions.Singleline;
                        break;
                    case 'n':
                        options = options | RegexOptions.ExplicitCapture;
                        break;
                    case 'x':
                        options = options | RegexOptions.IgnorePatternWhitespace;
                        break;
                }
            }

            return options;
        }

        private static string Substring(object field, int start, int length)
        {
            return field.ToString().Substring(start, length);
        }

        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public TRTAMI2022 ConsumedReport { get; private set; }

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string, ProcessMessageDocument>();

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(TRTAMI2022 report)
        {

            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            bool assertion;

            this.ConsumedReport = report;

            // ------------------------------------------------------------------------------
            // Validations are now done in code.
            // This version supports full validation of a report - for those rules that have successfully parsed and could be generated.
            // Generated Validations include:
            //   - Validation logic in C#.
            //   - Production of errors in EBMS, SBR and SWS format.
            // 
            // The generation at this stage does not support the following - and has to be completed manually (later versions of the generation will do this for you)
            //   - Business Rules that did not parse.  The ESR team will help support you where this happens as an effort is being made to rectify these
            //   - You will get TODO tasks for those that the parser could not cope with
            // ------------------------------------------------------------------------------

            #region Manual Rules - Rules marked for manual coding

            #region VR.ATO.TRTAMI.000004

            /*  VR.ATO.TRTAMI.000004
                The Sequence Container within the Statement of Attribution Sequence Dimension context instance must not be duplicated

            Legacy Rule Format:
                IF RP.{SoASeqNum}:SequenceContainer = ANY OTHER OCCURRENCE OF RP.{SoASeqNum}:SequenceContainer
                   RETURN VALIDATION MESSAGE
                ENDIF

            Technical Business Rule Format:
                HasDuplicateValues(^TRTAMI150)

            Data Elements:

            ^TRTAMI150 = TRTAMI150
            */
            VRATOTRTAMI000004(response, report);
            #endregion // VR.ATO.TRTAMI.000004

            #region VR.ATO.TRTAMI.000190

            /*  VR.ATO.TRTAMI.000190
                        If all Trustee Liabilities amounts and all Statement of Attribution (SA) Taxable Determined Member Component amounts are either blank or zero, then 'Is any tax payable by the trustee?' must be 'no'.

                    Legacy Rule Format:
                        IF [TRTAMI33] <> "0" AND (([TRTAMI35] = NULL OR [TRTAMI35] = 0) AND ([TRTAMI36] = NULL OR [TRTAMI36] = 0) AND ([TRTAMI39] = NULL OR [TRTAMI39] = 0) AND ([TRTAMI40] = NULL OR [TRTAMI40] = 0) AND ALL OCCURRENCES OF ([TRTAMI82] = NULL OR [TRTAMI82] = 0))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [TRTAMI33] = TRTAMI:RP:pyin.02.00:Report.Statement.Summary.PaymentRefundOrNil.Code
                        [TRTAMI35] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustComponentDeficitOffset.Amount
                        [TRTAMI36] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustShortfallIncome.Amount
                        [TRTAMI39] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustExcessOffset.Amount
                        [TRTAMI40] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustComponentNotReflected.Amount
                        [TRTAMI82] = TRTAMI:RP.{SoASeqNum}:rvctc4.02.10:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount

                Technical Business Rule Format:
                        ^TRTAMI33 <> '0' AND ((^TRTAMI35 = NULL OR ^TRTAMI35 = 0) AND (^TRTAMI36 = NULL OR ^TRTAMI36 = 0) AND (^TRTAMI39 = NULL OR ^TRTAMI39 = 0) AND (^TRTAMI40 = NULL OR ^TRTAMI40 = 0) AND (AllOccurrences(^TRTAMI82, ^TRTAMI82 = NULL) OR ^TRTAMI82 = 0))

            Data Elements:

                ^TRTAMI33 = RP:Report.Statement.Summary.PaymentRefundOrNil.Code

                ^TRTAMI35 = RP:IncomeTax.TrustComponentDeficitOffset.Amount

                ^TRTAMI36 = RP:IncomeTax.TrustShortfallIncome.Amount

                ^TRTAMI39 = RP:IncomeTax.TrustExcessOffset.Amount

                ^TRTAMI40 = RP:IncomeTax.TrustComponentNotReflected.Amount

                ^TRTAMI82 = RP.{SoASeqNum}:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount
                */
            VRATOTRTAMI000190(response, report);
            #endregion // VR.ATO.TRTAMI.000190

            #region VR.ATO.TRTAMI.000249

            /*  VR.ATO.TRTAMI.000249
                        Address Line 1 must be present

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                        IF [TRTAMI125] = NULLORBLANK AND [TRTAMI77] <> NULLORBLANK
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                        [TRTAMI125] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line1.Text

                Technical Business Rule Format:
                        ^TRTAMI125 = BLANK AND ^TRTAMI77 <> BLANK

            Data Elements:

                ^TRTAMI125 = RP.{SoASeqNum}.RES:AddressDetails.Line1.Text

                ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                */
            VRATOTRTAMI000249(response, report);
            #endregion // VR.ATO.TRTAMI.000249

            #region VR.ATO.TRTAMI.000258

            /*  VR.ATO.TRTAMI.000258
                        Suburb/Town/City is mandatory

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                        IF [TRTAMI127] = NULLORBLANK AND [TRTAMI77] <> NULLORBLANK
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                        [TRTAMI127] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.LocalityName.Text

                Technical Business Rule Format:
                        ^TRTAMI127 = BLANK AND ^TRTAMI77 <> BLANK

            Data Elements:

                ^TRTAMI127 = RP.{SoASeqNum}.RES:AddressDetails.LocalityName.Text

                ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                */
            VRATOTRTAMI000258(response, report);
            #endregion // VR.ATO.TRTAMI.000258

            #region VR.ATO.TRTAMI.000264

            /*  VR.ATO.TRTAMI.000264
                        Invalid context. The number of Residential contexts per Reporting Party Statement of Attribution context must not exceed 1.

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum})
                        IF COUNT(CONTEXT(RP.{SoASeqNum}.RES)) > 1
                           RETURN VALIDATION MESSAGE
                        ENDIF

                Technical Business Rule Format:
                        Count(^Context) > 1

            Data Elements:

                ^Context = RP.{SoASeqNum}.RES
                */
            VRATOTRTAMI000264(response, report);
            #endregion // VR.ATO.TRTAMI.000264

            #region VR.ATO.TRTAMI.000265

            /*  VR.ATO.TRTAMI.000265
                        Invalid context. The number of Business contexts per Reporting Party Statement of Attribution context must not exceed 1.

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum})
                        IF COUNT(CONTEXT(RP.{SoASeqNum}.BUS)) > 1
                           RETURN VALIDATION MESSAGE
                        ENDIF

                Technical Business Rule Format:
                        Count(^Context) > 1

            Data Elements:

                ^Context = RP.{SoASeqNum}.BUS
                */
            VRATOTRTAMI000265(response, report);
            #endregion // VR.ATO.TRTAMI.000265

            #region VR.ATO.TRTAMI.000268

            /*  VR.ATO.TRTAMI.000268
                        Address Line 1 must be present

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                        IF [TRTAMI131] = NULLORBLANK AND [TRTAMI80] <> NULLORBLANK
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                        [TRTAMI131] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line1.Text

                Technical Business Rule Format:
                        ^TRTAMI131 = BLANK AND ^TRTAMI80 <> BLANK

            Data Elements:

                ^TRTAMI131 = RP.{SoASeqNum}.BUS:AddressDetails.Line1.Text

                ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                */
            VRATOTRTAMI000268(response, report);
            #endregion // VR.ATO.TRTAMI.000268

            #region VR.ATO.TRTAMI.000277

            /*  VR.ATO.TRTAMI.000277
                        Suburb/Town/City is mandatory

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                        IF [TRTAMI133] = NULLORBLANK AND [TRTAMI80] <> NULLORBLANK
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                        [TRTAMI133] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.LocalityName.Text

                Technical Business Rule Format:
                        ^TRTAMI133 = BLANK AND ^TRTAMI80 <> BLANK

            Data Elements:

                ^TRTAMI133 = RP.{SoASeqNum}.BUS:AddressDetails.LocalityName.Text

                ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                */
            VRATOTRTAMI000277(response, report);
            #endregion // VR.ATO.TRTAMI.000277

            #region VR.ATO.TRTAMI.000305

            /*  VR.ATO.TRTAMI.000305
                        For an Australian address, Postal/Zip Code is mandatory

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                        IF [TRTAMI143] = NULLORBLANK AND [TRTAMI77] <> NULLORBLANK AND ([TRTAMI130] = NULL OR [TRTAMI130] = "au")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                        [TRTAMI130] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.08:AddressDetails.Country.Code
                        [TRTAMI143] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.27:AddressDetails.PostalCode.Text

                Technical Business Rule Format:
                        ^TRTAMI143 = BLANK AND ^TRTAMI77 <> BLANK AND (^TRTAMI130 = NULL OR ^TRTAMI130 = 'au')

            Data Elements:

                ^TRTAMI143 = RP.{SoASeqNum}.RES:AddressDetails.PostalCode.Text

                ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text

                ^TRTAMI130 = RP.{SoASeqNum}.RES:AddressDetails.Country.Code
                */
            VRATOTRTAMI000305(response, report);
            #endregion // VR.ATO.TRTAMI.000305

            #region VR.ATO.TRTAMI.000306

            /*  VR.ATO.TRTAMI.000306
                        For an Australian address, Member Residential Address - Postal/Zip Code is invalid. Valid range is 0200 - 9999.

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                        IF [TRTAMI77] <> NULLORBLANK AND ([TRTAMI130] = NULL OR [TRTAMI130] = "au") AND [TRTAMI143] <> NULLORBLANK AND (LENGTH([TRTAMI143]) <> 4 OR [TRTAMI143] <> SET(0200-9999))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                        [TRTAMI130] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.08:AddressDetails.Country.Code
                        [TRTAMI143] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.27:AddressDetails.PostalCode.Text

                Technical Business Rule Format:
                        ^TRTAMI77 <> BLANK AND (^TRTAMI130 = NULL OR ^TRTAMI130 = 'au') AND ^TRTAMI143 <> BLANK AND (Length(^TRTAMI143) <> 4 OR (^TRTAMI143 < 0200 OR ^TRTAMI143 > 9999))

            Data Elements:

                ^TRTAMI143 = RP.{SoASeqNum}.RES:AddressDetails.PostalCode.Text

                ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text

                ^TRTAMI130 = RP.{SoASeqNum}.RES:AddressDetails.Country.Code
                */
            VRATOTRTAMI000306(response, report);
            #endregion // VR.ATO.TRTAMI.000306

            #region VR.ATO.TRTAMI.000308

            /*  VR.ATO.TRTAMI.000308
                        For an Australian address, Postal/Zip Code is mandatory

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                        IF [TRTAMI144] = NULLORBLANK AND [TRTAMI80] <> NULLORBLANK AND ([TRTAMI136] = NULL OR [TRTAMI136] = "au")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                        [TRTAMI136] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.08:AddressDetails.Country.Code
                        [TRTAMI144] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.27:AddressDetails.PostalCode.Text

                Technical Business Rule Format:
                        ^TRTAMI144 = BLANK AND ^TRTAMI80 <> BLANK AND (^TRTAMI136 = NULL OR ^TRTAMI136 = 'au')

            Data Elements:

                ^TRTAMI144 = RP.{SoASeqNum}.BUS:AddressDetails.PostalCode.Text

                ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text

                ^TRTAMI136 = RP.{SoASeqNum}.BUS:AddressDetails.Country.Code
                */
            VRATOTRTAMI000308(response, report);
            #endregion // VR.ATO.TRTAMI.000308

            #region VR.ATO.TRTAMI.000309

            /*  VR.ATO.TRTAMI.000309
                        For an Australian address, Member Business Address - Postal/Zip Code is invalid. Valid range is 0200 - 9999.

                    Legacy Rule Format:
                        WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                        IF [TRTAMI80] <> NULLORBLANK AND ([TRTAMI136] = NULL OR [TRTAMI136] = "au") AND [TRTAMI144] <> NULLORBLANK AND (LENGTH([TRTAMI144]) <> 4 OR [TRTAMI144] <> SET(0200-9999))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                        [TRTAMI136] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.08:AddressDetails.Country.Code
                        [TRTAMI144] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.27:AddressDetails.PostalCode.Text

                Technical Business Rule Format:
                        ^TRTAMI80 <> BLANK AND (^TRTAMI136 = NULL OR ^TRTAMI136 = 'au') AND ^TRTAMI144 <> BLANK AND (Length(^TRTAMI144) <> 4 OR (^TRTAMI144 < 0200 OR ^TRTAMI144 > 9999))

            Data Elements:

                ^TRTAMI144 = RP.{SoASeqNum}.BUS:AddressDetails.PostalCode.Text

                ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text

                ^TRTAMI136 = RP.{SoASeqNum}.BUS:AddressDetails.Country.Code
                */
            VRATOTRTAMI000309(response, report);
            #endregion // VR.ATO.TRTAMI.000309
            #endregion Manual Rules - Rules marked for manual coding

            #region VR.ATO.GEN.000209

            /*  VR.ATO.GEN.000209
            Invalid context. The number of Reporting party contexts must equal 1
    
            Legacy Rule Format:
            IF COUNT(RP) <> 1
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) <> 1
    
            Data Elements:
    
            ^Context = RP
            */
            assertion = (Count(report.RPCount) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430296",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party contexts must equal 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000209" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000209

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP(Instant)
    
            ^OtherTFN = RP(Instant)
    
            ^TFN = RP
            */
            assertion = (report.RPInstantExists != false && report.RPInstantIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.POS
    
            ^OtherTFN = RP:RP.POS
    
            ^TFN = RP
            */
            assertion = (report.RPPOSExists != false && report.RPPOSIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPOSOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.POS:RP.POS.Prev
    
            ^OtherTFN = RP.POS:RP.POS.Prev
    
            ^TFN = RP
            */
            assertion = (report.RPPOSPrevExists != false && report.RPPOSPrevIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPOSPrevOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Prev
    
            ^OtherTFN = RP:RP.Prev
    
            ^TFN = RP
            */
            assertion = (report.RPPrevExists != false && report.RPPrevIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrevOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.StapledEntity
    
            ^OtherTFN = RP:RP.StapledEntity
    
            ^TFN = RP
            */
            assertion = (report.RPStapledEntityExists != false && report.RPStapledEntityIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPStapledEntityOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.TrueAndCorrect
    
            ^OtherTFN = RP:RP.TrueAndCorrect
    
            ^TFN = RP
            */
            assertion = (report.RPTrueAndCorrectExists != false && report.RPTrueAndCorrectIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTrueAndCorrectOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Trustee
    
            ^OtherTFN = RP:RP.Trustee
    
            ^TFN = RP
            */
            assertion = (report.RPTrusteeExists != false && report.RPTrusteeIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTrusteeOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.010028

            /*  VR.ATO.GEN.010028
            If the return is an amendment, sequence number, type and reason must be present
    
            Legacy Rule Format:
            IF pyin.xx.xx:Report.Amendment.Indicator = TRUE AND (pyin.xx.xx:Report.AmendmentSequence.Number = NULL OR pyin.xx.xx:Report.AmendmentType.Code = NULLORBLANK OR pyin.xx.xx:Report.AmendmentReason.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRTAMI6 = TRUE AND (^TRTAMI9 = NULL OR ^TRTAMI7 = BLANK OR ^TRTAMI8 = BLANK)
    
            Data Elements:
    
            ^TRTAMI6 = RP:Report.Amendment.Indicator
    
            ^TRTAMI9 = RP:Report.AmendmentSequence.Number
    
            ^TRTAMI7 = RP:Report.AmendmentType.Code
    
            ^TRTAMI8 = RP:Report.AmendmentReason.Text
            */
            assertion = (report.TRTAMI6 == true && (report.TRTAMI9 == null || string.IsNullOrWhiteSpace(report.TRTAMI7) == true || string.IsNullOrWhiteSpace(report.TRTAMI8) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010028",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If the return is an amendment, sequence number, type and reason must be present",
                    LongDescription = @"If the Amendment Indicator is 'yes' (true), then the Amendment Sequence Number, Amendment Type and Amendment Reason must be present and not blank.",
                    Location = "/xbrli:xbrl/tns:Report.Amendment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.010028" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI6", Value = GetValueOrEmpty(report.TRTAMI6) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI9", Value = report.TRTAMI9.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI7", Value = GetValueOrEmpty(report.TRTAMI7) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI8", Value = GetValueOrEmpty(report.TRTAMI8) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.010028

            #region VR.ATO.GEN.010029

            /*  VR.ATO.GEN.010029
            If the return is an amendment, Amendment Indicator must be 'yes' (true)
    
            Legacy Rule Format:
            IF pyin.xx.xx:Report.Amendment.Indicator <> TRUE AND (pyin.xx.xx:Report.AmendmentSequence.Number <> NULL OR pyin.xx.xx:Report.AmendmentType.Code <> NULLORBLANK OR pyin.xx.xx:Report.AmendmentReason.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRTAMI6 <> TRUE AND (^TRTAMI9 <> NULL OR ^TRTAMI7 <> BLANK OR ^TRTAMI8 <> BLANK)
    
            Data Elements:
    
            ^TRTAMI6 = RP:Report.Amendment.Indicator
    
            ^TRTAMI9 = RP:Report.AmendmentSequence.Number
    
            ^TRTAMI7 = RP:Report.AmendmentType.Code
    
            ^TRTAMI8 = RP:Report.AmendmentReason.Text
            */
            assertion = (report.TRTAMI6 != true && (report.TRTAMI9 != null || string.IsNullOrWhiteSpace(report.TRTAMI7) != true || string.IsNullOrWhiteSpace(report.TRTAMI8) != true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If the return is an amendment, Amendment Indicator must be 'yes' (true)",
                    LongDescription = @"If an Amendment Sequence Number, Amendment Type or Amendment Reason is present, then the Amendment Indicator must be 'yes' (true)",
                    Location = "/xbrli:xbrl/tns:Report.Amendment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.010029" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI6", Value = GetValueOrEmpty(report.TRTAMI6) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI9", Value = report.TRTAMI9.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI7", Value = GetValueOrEmpty(report.TRTAMI7) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI8", Value = GetValueOrEmpty(report.TRTAMI8) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.010029

            #region VR.ATO.GEN.010030

            /*  VR.ATO.GEN.010030
            Amendment sequence number must be a number between 1 and 9
    
            Legacy Rule Format:
            IF pyin.xx.xx:Report.AmendmentSequence.Number <> NULL AND pyin.xx.xx:Report.AmendmentSequence.Number <> SET(1-9)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRTAMI9 <> NULL AND (^TRTAMI9 < 1 OR ^TRTAMI9 > 9)
    
            Data Elements:
    
            ^TRTAMI9 = RP:Report.AmendmentSequence.Number
            */
            assertion = (report.TRTAMI9 != null && (report.TRTAMI9.GetValueOrDefault() < 1 || report.TRTAMI9.GetValueOrDefault() > 9));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Amendment sequence number must be a number between 1 and 9",
                    Location = "/xbrli:xbrl/tns:Report.AmendmentSequence.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.010030" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI9", Value = report.TRTAMI9.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.010030

            #region VR.ATO.GEN.428247

            /*  VR.ATO.GEN.428247
            Your Tax Agent number has failed the Tax Agent registration algorithm check. Input the correct Tax Agent registration number.
    
            Legacy Rule Format:
            IF ((pyid.xx.xx:Identifiers.TaxAgentNumber.Identifier) <> NULLORBLANK) AND (TANALGORITHM(pyid.xx.xx:Identifiers.TaxAgentNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI91 <> BLANK) AND (FailsTANAlgorithm(^TRTAMI91))
    
            Data Elements:
    
            ^TRTAMI91 = INT.TrueAndCorrect:Identifiers.TaxAgentNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI91) != true && FailsTANAlgorithm(report.TRTAMI91));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Tax Agent number has failed the Tax Agent number algorithm check",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxAgentNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428247" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI91", Value = GetValueOrEmpty(report.TRTAMI91) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428247

            #region VR.ATO.GEN.430206

            /*  VR.ATO.GEN.430206
            Hours taken to prepare and complete form must be 1 to 999
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Report.CompletionHours.Number <> NULL) AND (pyin.xx.xx:Report.CompletionHours.Number <> SET (1-999))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI74 <> NULL) AND ((^TRTAMI74 < 1 OR ^TRTAMI74 > 999))
    
            Data Elements:
    
            ^TRTAMI74 = RP:Report.CompletionHours.Number
            */
            assertion = (report.TRTAMI74 != null && (report.TRTAMI74.GetValueOrDefault() < 1 || report.TRTAMI74.GetValueOrDefault() > 999));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430206",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Hours taken to prepare and complete form must be 1 to 999",
                    Location = "/xbrli:xbrl/tns:Report.CompletionHours.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430206" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI74", Value = report.TRTAMI74.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430206

            #region VR.ATO.GEN.432035

            /*  VR.ATO.GEN.432035
            BSB Number must be six digits and greater than 012000
    
            Legacy Rule Format:
            IF pyid.xx.xx:FinancialInstitutionAccount.BankStateBranch.Number <> NULLORBLANK AND 
            (pyid.xx.xx:FinancialInstitutionAccount.BankStateBranch.Number <> SET(012001-999999)) OR (LENGTH(pyid.xx.xx:FinancialInstitutionAccount.BankStateBranch.Number) <> 6)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRTAMI41 <> BLANK AND (IsNumeric(^TRTAMI41) = 'FALSE' OR AsNumeric(^TRTAMI41) < 12001 OR AsNumeric(^TRTAMI41) > 999999 OR Length(^TRTAMI41) <> 6)
    
            Data Elements:
    
            ^TRTAMI41 = RP:FinancialInstitutionAccount.BankStateBranch.Number
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI41) != true && (IsNumeric(report.TRTAMI41) == false || AsNumeric(report.TRTAMI41) < 12001 || AsNumeric(report.TRTAMI41) > 999999 || Length(report.TRTAMI41) != 6));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432035",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"BSB Number must be six digits and greater than 012000",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount.BankStateBranch.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.432035" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI41", Value = GetValueOrEmpty(report.TRTAMI41) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.432035

            #region VR.ATO.GEN.432041

            /*  VR.ATO.GEN.432041
            Financial institution account name cannot be greater than 32 characters
    
            Legacy Rule Format:
            IF LENGTH(pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text) > 32
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRTAMI43) > 32
    
            Data Elements:
    
            ^TRTAMI43 = RP:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text
            */
            assertion = (Length(report.TRTAMI43) > 32);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432041",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Bank account name must not be longer than 32 characters",
                    LongDescription = @"32 characters is the Banking Standard and set by the Banking Industry",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.432041" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI43", Value = GetValueOrEmpty(report.TRTAMI43) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.432041

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP
    
            ^Context = RP
            */
            assertion = (report.RPExists != false && report.RPIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP(Instant)
    
            ^Context = RP(Instant)
            */
            assertion = (report.RPInstantExists != false && report.RPInstantIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.POS
    
            ^Context = RP:RP.POS
            */
            assertion = (report.RPPOSExists != false && report.RPPOSIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPOSOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.POS:RP.POS.Prev
    
            ^Context = RP.POS:RP.POS.Prev
            */
            assertion = (report.RPPOSPrevExists != false && report.RPPOSPrevIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPOSPrevOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Prev
    
            ^Context = RP:RP.Prev
            */
            assertion = (report.RPPrevExists != false && report.RPPrevIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrevOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.StapledEntity
    
            ^Context = RP:RP.StapledEntity
            */
            assertion = (report.RPStapledEntityExists != false && report.RPStapledEntityIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPStapledEntityOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.TrueAndCorrect
    
            ^Context = RP:RP.TrueAndCorrect
            */
            assertion = (report.RPTrueAndCorrectExists != false && report.RPTrueAndCorrectIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTrueAndCorrectOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Trustee
    
            ^Context = RP:RP.Trustee
            */
            assertion = (report.RPTrusteeExists != false && report.RPTrusteeIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTrusteeOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438045

            /*  VR.ATO.GEN.438045
            Year of return must be supplied
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Report.TargetFinancial.Year = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRTAMI1 = NULL
    
            Data Elements:
    
            ^TRTAMI1 = RP:Report.TargetFinancial.Year
            */
            assertion = (report.TRTAMI1 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438045",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of return must be supplied",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438045" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI1", Value = GetValueOrEmpty(report.TRTAMI1) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438045

            #region VR.ATO.GEN.438070

            /*  VR.ATO.GEN.438070
            If an Intermediary is present, Intermediary Declarer contact name must be provided.
    
            Legacy Rule Format:
            IF ((CONTEXT(INT.TrueAndCorrect)<>NULL) AND (INT.TrueAndCorrect:pyde.xx.xx:PersonUnstructuredName.FullName.Text = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((^Context <> NULL) AND (^TRTAMI83 = BLANK))
    
            Data Elements:
    
            ^TRTAMI83 = INT.TrueAndCorrect:PersonUnstructuredName.FullName.Text
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists != false && string.IsNullOrWhiteSpace(report.TRTAMI83) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438070",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer contact name must be provided",
                    LongDescription = @"If an Intermediary is present, Intermediary Declarer contact name must be provided.",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName.FullName.Text[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438070" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI83", Value = report.TRTAMI83 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438070

            #region VR.ATO.GEN.438071

            /*  VR.ATO.GEN.438071
            If an Intermediary is not present, Reporting Party Declarer contact name must be provided.
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect)=NULL) AND (RP.TrueAndCorrect:pyde.xx.xx:PersonUnstructuredName.FullName.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context=NULL) AND (^TRTAMI94 = BLANK)
    
            Data Elements:
    
            ^TRTAMI94 = RP.TrueAndCorrect:PersonUnstructuredName.FullName.Text
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists == false && string.IsNullOrWhiteSpace(report.TRTAMI94) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438071",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer contact name must be provided",
                    LongDescription = @"If an Intermediary is not present, Reporting Party Declarer contact name must be provided.",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName.FullName.Text[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438071" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI94", Value = report.TRTAMI94 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438071

            #region VR.ATO.GEN.438072

            /*  VR.ATO.GEN.438072
            If an area code is supplied and Intermediary is present, Intermediary Telephone number is required.
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect) <> NULL) AND (INT.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number = NULLORBLANK) AND (INT.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Area.Code <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^TRTAMI86 = BLANK) AND (^TRTAMI85 <> BLANK)
    
            Data Elements:
    
            ^TRTAMI85 = INT.TrueAndCorrect:ElectronicContact.Telephone.Area.Code
    
            ^TRTAMI86 = INT.TrueAndCorrect:ElectronicContact.Telephone.Minimal.Number
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists != false && string.IsNullOrWhiteSpace(report.TRTAMI86) == true && string.IsNullOrWhiteSpace(report.TRTAMI85) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438072",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Telephone number is required, if an area code is provided",
                    LongDescription = @"If an area code is supplied and Intermediary is present, Intermediary Telephone number is required.",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Area.Code[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438072" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI86", Value = report.TRTAMI86 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI85", Value = report.TRTAMI85 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438072

            #region VR.ATO.GEN.438073

            /*  VR.ATO.GEN.438073
            If an area code is supplied and Intermediary is not present, Reporting Party Telephone number is required.
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect) = NULL) AND (RP.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number = NULLORBLANK) AND (RP.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Area.Code <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context = NULL) AND (^TRTAMI97 = BLANK) AND (^TRTAMI96 <> BLANK)
    
            Data Elements:
    
            ^TRTAMI96 = RP.TrueAndCorrect:ElectronicContact.Telephone.Area.Code
    
            ^TRTAMI97 = RP.TrueAndCorrect:ElectronicContact.Telephone.Minimal.Number
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists == false && string.IsNullOrWhiteSpace(report.TRTAMI97) == true && string.IsNullOrWhiteSpace(report.TRTAMI96) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438073",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Telephone number is required, if an area code is provided",
                    LongDescription = @"If an area code is supplied and Intermediary is not present, Reporting Party Telephone number is required.",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Area.Code[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438073" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI97", Value = report.TRTAMI97 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI96", Value = report.TRTAMI96 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438073

            #region VR.ATO.GEN.438074

            /*  VR.ATO.GEN.438074
            If a telephone number is supplied and Intermediary is present, Intermediary area code is required.
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect) <> NULL) AND (INT.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number <> NULL) AND (INT.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Area.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^TRTAMI86 <> NULL) AND (^TRTAMI85 = BLANK)
    
            Data Elements:
    
            ^TRTAMI85 = INT.TrueAndCorrect:ElectronicContact.Telephone.Area.Code
    
            ^TRTAMI86 = INT.TrueAndCorrect:ElectronicContact.Telephone.Minimal.Number
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists != false && report.TRTAMI86 != null && string.IsNullOrWhiteSpace(report.TRTAMI85) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438074",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Area code is required, if a Telephone number is provided",
                    LongDescription = @"If a telephone number is supplied and Intermediary is present, Intermediary area code is required.",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Area.Code[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438074" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI86", Value = report.TRTAMI86 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI85", Value = report.TRTAMI85 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438074

            #region VR.ATO.GEN.438075

            /*  VR.ATO.GEN.438075
            If a telephone number is supplied and Intermediary is not present, Reporting Party area code is required.
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect) = NULL) AND (RP.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number <> NULL) AND (RP.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Area.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context = NULL) AND (^TRTAMI97 <> NULL) AND (^TRTAMI96 = BLANK)
    
            Data Elements:
    
            ^TRTAMI96 = RP.TrueAndCorrect:ElectronicContact.Telephone.Area.Code
    
            ^TRTAMI97 = RP.TrueAndCorrect:ElectronicContact.Telephone.Minimal.Number
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists == false && report.TRTAMI97 != null && string.IsNullOrWhiteSpace(report.TRTAMI96) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438075",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Area code is required, if a Telephone number is provided",
                    LongDescription = @"If a telephone number is supplied and Intermediary is not present, Reporting Party area code is required.",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Area.Code[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438075" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI97", Value = report.TRTAMI97 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI96", Value = report.TRTAMI96 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438075

            #region VR.ATO.GEN.438076

            /*  VR.ATO.GEN.438076
            If an Intermediary is present, Intermediary Declarer Telephone number must be provided.
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect)<>NULL) AND (INT.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^TRTAMI86 = BLANK)
    
            Data Elements:
    
            ^TRTAMI86 = INT.TrueAndCorrect:ElectronicContact.Telephone.Minimal.Number
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists != false && string.IsNullOrWhiteSpace(report.TRTAMI86) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438076",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Contact telephone details must be provided",
                    LongDescription = @"If an Intermediary is present, Intermediary Declarer Telephone number must be provided.",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Minimal.Number[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438076" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI86", Value = report.TRTAMI86 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438076

            #region VR.ATO.GEN.438077

            /*  VR.ATO.GEN.438077
            If an Intermediary is not present, Reporting Party Declarer Telephone number must be provided.
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect)=NULL) AND (RP.TrueAndCorrect:pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context=NULL) AND (^TRTAMI97 = BLANK)
    
            Data Elements:
    
            ^TRTAMI97 = RP.TrueAndCorrect:ElectronicContact.Telephone.Minimal.Number
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists == false && string.IsNullOrWhiteSpace(report.TRTAMI97) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438077",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Contact telephone details must be provided",
                    LongDescription = @"If an Intermediary is not present, Reporting Party Declarer Telephone number must be provided.",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Minimal.Number[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438077" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI97", Value = report.TRTAMI97 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438077

            #region VR.ATO.GEN.438078

            /*  VR.ATO.GEN.438078
            If an Intermediary is present, Intermediary Declarer Signature Date must be present.
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect)<>NULL) AND (INT.TrueAndCorrect:pyin.xx.xx:Declaration.Signature.Date = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^TRTAMI87 = NULL)
    
            Data Elements:
    
            ^TRTAMI87 = INT.TrueAndCorrect:Declaration.Signature.Date
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists != false && report.TRTAMI87 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438078",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration Signature Date is required",
                    LongDescription = @"If an Intermediary is present, Intermediary Declarer Signature Date must be present.",
                    Location = "/xbrli:xbrl/tns:Declaration.Signature.Date[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438078" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI87", Value = GetValueOrEmptyDate(report.TRTAMI87) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438078

            #region VR.ATO.GEN.438079

            /*  VR.ATO.GEN.438079
            If an Intermediary is not present, Reporting Party Declarer Signature Date must be present.
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect)=NULL) AND (RP.TrueAndCorrect:pyin.xx.xx:Declaration.Signature.Date = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context=NULL) AND (^TRTAMI98 = NULL)
    
            Data Elements:
    
            ^TRTAMI98 = RP.TrueAndCorrect:Declaration.Signature.Date
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists == false && report.TRTAMI98 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438079",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration Signature Date is required",
                    LongDescription = @"If an Intermediary is not present, Reporting Party Declarer Signature Date must be present.",
                    Location = "/xbrli:xbrl/tns:Declaration.Signature.Date[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438079" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI98", Value = GetValueOrEmptyDate(report.TRTAMI98) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438079

            #region VR.ATO.GEN.500030

            /*  VR.ATO.GEN.500030
            Invalid context. Period start date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^StartDate1 = INT.TrueAndCorrect
            */
            assertion = ((report.INTTrueAndCorrectExists == true) && (report.INTTrueAndCorrectStartDate != null && report.INTTrueAndCorrectStartDate != report.RPStartDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500030

            #region VR.ATO.GEN.500030

            /*  VR.ATO.GEN.500030
            Invalid context. Period start date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^StartDate1 = RP:RP.POS
            */
            assertion = ((report.RPPOSExists == true) && (report.RPPOSStartDate != null && report.RPPOSStartDate != report.RPStartDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500030

            #region VR.ATO.GEN.500030

            /*  VR.ATO.GEN.500030
            Invalid context. Period start date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^StartDate1 = RP.POS:RP.POS.Prev
            */
            assertion = ((report.RPPOSPrevExists == true) && (report.RPPOSPrevStartDate != null && report.RPPOSPrevStartDate != report.RPStartDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500030

            #region VR.ATO.GEN.500030

            /*  VR.ATO.GEN.500030
            Invalid context. Period start date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^StartDate1 = RP:RP.Prev
            */
            assertion = ((report.RPPrevExists == true) && (report.RPPrevStartDate != null && report.RPPrevStartDate != report.RPStartDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500030

            #region VR.ATO.GEN.500030

            /*  VR.ATO.GEN.500030
            Invalid context. Period start date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^StartDate1 = RP:RP.StapledEntity
            */
            assertion = ((report.RPStapledEntityExists == true) && (report.RPStapledEntityStartDate != null && report.RPStapledEntityStartDate != report.RPStartDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500030

            #region VR.ATO.GEN.500030

            /*  VR.ATO.GEN.500030
            Invalid context. Period start date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^StartDate1 = RP:RP.TrueAndCorrect
            */
            assertion = ((report.RPTrueAndCorrectExists == true) && (report.RPTrueAndCorrectStartDate != null && report.RPTrueAndCorrectStartDate != report.RPStartDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500030

            #region VR.ATO.GEN.500030

            /*  VR.ATO.GEN.500030
            Invalid context. Period start date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^StartDate1 = RP:RP.Trustee
            */
            assertion = ((report.RPTrusteeExists == true) && (report.RPTrusteeStartDate != null && report.RPTrusteeStartDate != report.RPStartDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500030

            #region VR.ATO.GEN.500033

            /*  VR.ATO.GEN.500033
            Invalid context. Period end date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^EndDate1 = INT.TrueAndCorrect
            */
            assertion = ((report.INTTrueAndCorrectExists == true) && (report.INTTrueAndCorrectEndDate != null && report.INTTrueAndCorrectEndDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500033

            #region VR.ATO.GEN.500033

            /*  VR.ATO.GEN.500033
            Invalid context. Period end date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^EndDate1 = RP:RP.POS
            */
            assertion = ((report.RPPOSExists == true) && (report.RPPOSEndDate != null && report.RPPOSEndDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500033

            #region VR.ATO.GEN.500033

            /*  VR.ATO.GEN.500033
            Invalid context. Period end date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^EndDate1 = RP.POS:RP.POS.Prev
            */
            assertion = ((report.RPPOSPrevExists == true) && (report.RPPOSPrevEndDate != null && report.RPPOSPrevEndDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500033

            #region VR.ATO.GEN.500033

            /*  VR.ATO.GEN.500033
            Invalid context. Period end date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^EndDate1 = RP:RP.Prev
            */
            assertion = ((report.RPPrevExists == true) && (report.RPPrevEndDate != null && report.RPPrevEndDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500033

            #region VR.ATO.GEN.500033

            /*  VR.ATO.GEN.500033
            Invalid context. Period end date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^EndDate1 = RP:RP.StapledEntity
            */
            assertion = ((report.RPStapledEntityExists == true) && (report.RPStapledEntityEndDate != null && report.RPStapledEntityEndDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500033

            #region VR.ATO.GEN.500033

            /*  VR.ATO.GEN.500033
            Invalid context. Period end date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^EndDate1 = RP:RP.TrueAndCorrect
            */
            assertion = ((report.RPTrueAndCorrectExists == true) && (report.RPTrueAndCorrectEndDate != null && report.RPTrueAndCorrectEndDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500033

            #region VR.ATO.GEN.500033

            /*  VR.ATO.GEN.500033
            Invalid context. Period end date must be the same across all duration contexts.
    
            Legacy Rule Format:
            IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^EndDate1 = RP:RP.Trustee
            */
            assertion = ((report.RPTrusteeExists == true) && (report.RPTrusteeEndDate != null && report.RPTrusteeEndDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500033

            #region VR.ATO.GEN.500037

            /*  VR.ATO.GEN.500037
            A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS SET("{","}","|","_","\","#"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^TRTAMI83, '"{","}","|","_","\","#"')
    
            Data Elements:
    
            ^TRTAMI83 = INT.TrueAndCorrect:PersonUnstructuredName.FullName.Text
            */
            assertion = IsMatch(report.TRTAMI83, @"\{|\}|\||_|\\|#", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500037",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer Contact Name contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName.FullName.Text[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500037" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI83", Value = report.TRTAMI83 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500037

            #region VR.ATO.GEN.500037

            /*  VR.ATO.GEN.500037
            A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS SET("{","}","|","_","\","#"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^TRTAMI94, '"{","}","|","_","\","#"')
    
            Data Elements:
    
            ^TRTAMI94 = RP.TrueAndCorrect:PersonUnstructuredName.FullName.Text
            */
            assertion = IsMatch(report.TRTAMI94, @"\{|\}|\||_|\\|#", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500037",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer Contact Name contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName.FullName.Text[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500037" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI94", Value = report.TRTAMI94 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500037

            #region VR.ATO.GEN.500046

            /*  VR.ATO.GEN.500046
            Invalid context. The number of RP.TrueAndCorrect contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(RP.TrueAndCorrect) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.TrueAndCorrect
            */
            assertion = (Count(report.RPTrueAndCorrectCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500046",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of RP.TrueAndCorrect contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTrueAndCorrectLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500046" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500046

            #region VR.ATO.GEN.500047

            /*  VR.ATO.GEN.500047
            Invalid context. The number of Intermediary.TrueAndCorrect contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(INT.TrueAndCorrect) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (Count(report.INTTrueAndCorrectCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500047",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Intermediary.TrueAndCorrect contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTTrueAndCorrectLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500047" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500047

            #region VR.ATO.GEN.500051

            /*  VR.ATO.GEN.500051
            Scheme for Intermediary.TrueAndCorrect must be set to http://www.abr.gov.au/abn
    
            Legacy Rule Format:
            IF (INT.TrueAndCorrect:entity.identifier.scheme <> "http://www.abr.gov.au/abn")
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IdentifierScheme <> 'http://www.abr.gov.au/abn'
    
            Data Elements:
    
            ^IdentifierScheme = INT.TrueAndCorrect
            */
            assertion = ((report.INTTrueAndCorrectExists == true) && (report.INTTrueAndCorrectIdentifierScheme != @"http://www.abr.gov.au/abn"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500051",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for Intermediary.TrueAndCorrect must be ""http://www.abr.gov.au/abn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTTrueAndCorrectOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500051" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500051

            #region VR.ATO.GEN.500054

            /*  VR.ATO.GEN.500054
            ABN in the context definition must pass the ABN algorithm check
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect)<>NULL) AND (ABNALGORITHM(INT.TrueAndCorrect:entity.identifier.ABN) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (FailsABNAlgorithm(^ABN))
    
            Data Elements:
    
            ^Context = INT.TrueAndCorrect
    
            ^ABN = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists != false && FailsABNAlgorithm(report.INTTrueAndCorrectIdentifierABN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Intermediary declarer Australian Business Number has failed the algorithm check",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTTrueAndCorrectOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500054

            #region VR.ATO.GEN.500057

            /*  VR.ATO.GEN.500057
            Name field cannot contain 'space hyphen space'.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRTAMI83, ' - ')
    
            Data Elements:
    
            ^TRTAMI83 = INT.TrueAndCorrect:PersonUnstructuredName.FullName.Text
            */
            assertion = (report.TRTAMI83 == null ? false : report.TRTAMI83.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500057",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer Contact Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName.FullName.Text[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500057" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI83", Value = report.TRTAMI83 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500057

            #region VR.ATO.GEN.500057

            /*  VR.ATO.GEN.500057
            Name field cannot contain 'space hyphen space'.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRTAMI94, ' - ')
    
            Data Elements:
    
            ^TRTAMI94 = RP.TrueAndCorrect:PersonUnstructuredName.FullName.Text
            */
            assertion = (report.TRTAMI94 == null ? false : report.TRTAMI94.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500057",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer Contact Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName.FullName.Text[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500057" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI94", Value = report.TRTAMI94 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500057

            #region VR.ATO.GEN.500058

            /*  VR.ATO.GEN.500058
            Declarer Position held of person making the declaration has a maximum field length of 50
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonNameDetails.Position.Text) > 50
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRTAMI84) > 50
    
            Data Elements:
    
            ^TRTAMI84 = INT.TrueAndCorrect:PersonNameDetails.Position.Text
            */
            assertion = (Length(report.TRTAMI84) > 50);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500058",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer position must not exceed 50 characters",
                    LongDescription = @"For Declarer position, the maximum allowable characters for this form is 50",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.Position.Text[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500058" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI84", Value = GetValueOrEmpty(report.TRTAMI84) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500058

            #region VR.ATO.GEN.500058

            /*  VR.ATO.GEN.500058
            Declarer Position held of person making the declaration has a maximum field length of 50
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonNameDetails.Position.Text) > 50
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRTAMI95) > 50
    
            Data Elements:
    
            ^TRTAMI95 = RP.TrueAndCorrect:PersonNameDetails.Position.Text
            */
            assertion = (Length(report.TRTAMI95) > 50);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500058",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer position must not exceed 50 characters",
                    LongDescription = @"For Declarer position, the maximum allowable characters for this form is 50",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.Position.Text[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500058" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI95", Value = GetValueOrEmpty(report.TRTAMI95) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500058

            #region VR.ATO.GEN.500059

            /*  VR.ATO.GEN.500059
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.Position.Text <> NULLORBLANK) AND (ANY CHARACTER OF pyde.xx.xx:PersonNameDetails.Position.Text <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI84 <> BLANK) AND (NotCharacterInSet(^TRTAMI84, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI84 = INT.TrueAndCorrect:PersonNameDetails.Position.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI84) != true && !(IsMatch(report.TRTAMI84, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500059",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer position held of person making the declaration contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.Position.Text[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500059" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI84", Value = GetValueOrEmpty(report.TRTAMI84) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500059

            #region VR.ATO.GEN.500059

            /*  VR.ATO.GEN.500059
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.Position.Text <> NULLORBLANK) AND (ANY CHARACTER OF pyde.xx.xx:PersonNameDetails.Position.Text <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI95 <> BLANK) AND (NotCharacterInSet(^TRTAMI95, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI95 = RP.TrueAndCorrect:PersonNameDetails.Position.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI95) != true && !(IsMatch(report.TRTAMI95, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500059",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer position held of person making the declaration contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.Position.Text[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500059" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI95", Value = GetValueOrEmpty(report.TRTAMI95) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500059

            #region VR.ATO.GEN.500066

            /*  VR.ATO.GEN.500066
            Signature date must not be later than today
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Declaration.Signature.Date > DATE(TODAY))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI87 > Today())
    
            Data Elements:
    
            ^TRTAMI87 = INT.TrueAndCorrect:Declaration.Signature.Date
            */
            assertion = (report.TRTAMI87.GetValueOrDefault() > DateTime.Now.Date);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430255",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Signature date must not be later than today",
                    Location = "/xbrli:xbrl/tns:Declaration.Signature.Date[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500066" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI87", Value = GetValueOrEmptyDate(report.TRTAMI87) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500066

            #region VR.ATO.GEN.500066

            /*  VR.ATO.GEN.500066
            Signature date must not be later than today
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Declaration.Signature.Date > DATE(TODAY))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI98 > Today())
    
            Data Elements:
    
            ^TRTAMI98 = RP.TrueAndCorrect:Declaration.Signature.Date
            */
            assertion = (report.TRTAMI98.GetValueOrDefault() > DateTime.Now.Date);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430255",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Signature date must not be later than today",
                    Location = "/xbrli:xbrl/tns:Declaration.Signature.Date[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500066" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI98", Value = GetValueOrEmptyDate(report.TRTAMI98) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500066

            #region VR.ATO.GEN.500069

            /*  VR.ATO.GEN.500069
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = _ [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Declaration.SignatoryIdentifier.Text <> NULLORBLANK) AND (ANY CHARACTER OF pyin.xx.xx:Declaration.SignatoryIdentifier.Text <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","_","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI89 <> BLANK) AND (NotCharacterInSet(^TRTAMI89, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","_","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI89 = INT.TrueAndCorrect:Declaration.SignatoryIdentifier.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI89) != true && !(IsMatch(report.TRTAMI89, @"^[a-zA-Z0-9!@\$%&\*\(\)\-_=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500069",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer identifier contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - _ = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:Declaration.SignatoryIdentifier.Text[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500069" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI89", Value = report.TRTAMI89 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500069

            #region VR.ATO.GEN.500069

            /*  VR.ATO.GEN.500069
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = _ [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Declaration.SignatoryIdentifier.Text <> NULLORBLANK) AND (ANY CHARACTER OF pyin.xx.xx:Declaration.SignatoryIdentifier.Text <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","_","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI100 <> BLANK) AND (NotCharacterInSet(^TRTAMI100, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","_","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI100 = RP.TrueAndCorrect:Declaration.SignatoryIdentifier.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI100) != true && !(IsMatch(report.TRTAMI100, @"^[a-zA-Z0-9!@\$%&\*\(\)\-_=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500069",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer identifier contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - _ = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:Declaration.SignatoryIdentifier.Text[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500069" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI100", Value = report.TRTAMI100 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500069

            #region VR.ATO.GEN.500088

            /*  VR.ATO.GEN.500088
            Invalid context. The number of Reporting Party (Instant) contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(RP(Instant)) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP(Instant)
            */
            assertion = (Count(report.RPInstantCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500088",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party (Instant) contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInstantLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500088" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500088

            #region VR.ATO.GEN.500089

            /*  VR.ATO.GEN.500089
            Entity identifier scheme for Reporting Party (Instant) must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RP(Instant):entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IdentifierScheme <> 'http://www.ato.gov.au/tfn'
    
            Data Elements:
    
            ^IdentifierScheme = RP(Instant)
            */
            assertion = ((report.RPInstantExists == true) && (report.RPInstantIdentifierScheme != @"http://www.ato.gov.au/tfn"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500089",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for Reporting Party (Instant) must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500089" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500089

            #region VR.ATO.GEN.500091

            /*  VR.ATO.GEN.500091
            Invalid context. The number of Reporting Party Trustee contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Trustee)) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Trustee
            */
            assertion = (Count(report.RPTrusteeCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500091",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Trustee contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTrusteeLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500091" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500091

            #region VR.ATO.GEN.500092

            /*  VR.ATO.GEN.500092
            This is an annual accounting submission. The dates provided should be for your annual accounting period, commencing on the 1st day of the month. For normal balancers, this will be the period 1 July - 30 June.
    
            Legacy Rule Format:
            IF RP:period.endDate > AddMonthsToDate(RP:period.startDate, 23)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^EndDate > AddMonthsToDate(^StartDate1, 23)
    
            Data Elements:
    
            ^EndDate = RP
    
            ^StartDate1 = RP
            */
            assertion = ((report.RPExists == true) && (report.RPEndDate > AddMonthsToDate(report.RPStartDate, 23)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500092",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The account period dates must not exceed 23 months for an approved transitional period",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500092" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500092

            #region VR.ATO.GEN.500093

            /*  VR.ATO.GEN.500093
            Accounting period start date must commence on or after 1st January in the previous year
    
            Legacy Rule Format:
            IF RP:Period.startDate <  CONVERT_TO_DATE(1,1, pyin.xx.xx:Report.TargetFinancial.Year - 1)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate1 <  ConvertToDate(1, 1, ^TRTAMI1 - 1)
    
            Data Elements:
    
            ^StartDate1 = RP
    
            ^TRTAMI1 = RP:Report.TargetFinancial.Year
            */
            assertion = ((report.RPExists == true) && (report.RPStartDate < ConvertToDate(1, 1, report.TRTAMI1.GetValueOrDefault() - 1)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500093",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Date financial year from date must commence on or after 1st January in the previous year",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500093" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI1", Value = GetValueOrEmpty(report.TRTAMI1) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500093

            #region VR.ATO.GEN.500094

            /*  VR.ATO.GEN.500094
            Intermediary Declaration Statement Accepted Indicator must be 'TRUE'
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect) <> NULL) AND (INT.TrueAndCorrect:pyin.xx.xx:Declaration.StatementAccepted.Indicator <> TRUE)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^TRTAMI88 <> TRUE)
    
            Data Elements:
    
            ^TRTAMI88 = INT.TrueAndCorrect:Declaration.StatementAccepted.Indicator
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists != false && report.TRTAMI88 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500094",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Intermediary Declaration Statement Accepted Indicator must be 'TRUE'",
                    Location = "/xbrli:xbrl/tns:Declaration.StatementAccepted.Indicator[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500094" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI88", Value = GetValueOrEmpty(report.TRTAMI88) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500094

            #region VR.ATO.GEN.500095

            /*  VR.ATO.GEN.500095
            Declaration Signatory identifier text must be present
    
            Legacy Rule Format:
            IF (CONTEXT(INT.TrueAndCorrect) <> NULL) AND (INT.TrueAndCorrect:pyin.xx.xx:Declaration.SignatoryIdentifier.Text = NULLORBLANK) 
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^TRTAMI89 = BLANK)
    
            Data Elements:
    
            ^TRTAMI89 = INT.TrueAndCorrect:Declaration.SignatoryIdentifier.Text
    
            ^Context = INT.TrueAndCorrect
            */
            assertion = (report.INTTrueAndCorrectExists != false && string.IsNullOrWhiteSpace(report.TRTAMI89) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500095",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Intermediary Declaration Signatory identifier text must be present",
                    Location = "/xbrli:xbrl/tns:Declaration.SignatoryIdentifier.Text[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500095" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI89", Value = report.TRTAMI89 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500095

            #region VR.ATO.GEN.500096

            /*  VR.ATO.GEN.500096
            Declarer Contact Name has a maximum field length of 200
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRTAMI83) > 200
    
            Data Elements:
    
            ^TRTAMI83 = INT.TrueAndCorrect:PersonUnstructuredName.FullName.Text
            */
            assertion = (Length(report.TRTAMI83) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500096",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer Contact Name must not exceed 200 characters",
                    LongDescription = @"For Declarer contact name, the maximum allowable characters for this form is 200",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName.FullName.Text[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500096" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI83", Value = report.TRTAMI83 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500096

            #region VR.ATO.GEN.500096

            /*  VR.ATO.GEN.500096
            Declarer Contact Name has a maximum field length of 200
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRTAMI94) > 200
    
            Data Elements:
    
            ^TRTAMI94 = RP.TrueAndCorrect:PersonUnstructuredName.FullName.Text
            */
            assertion = (Length(report.TRTAMI94) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500096",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer Contact Name must not exceed 200 characters",
                    LongDescription = @"For Declarer contact name, the maximum allowable characters for this form is 200",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName.FullName.Text[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500096" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI94", Value = report.TRTAMI94 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500096

            #region VR.ATO.GEN.500097

            /*  VR.ATO.GEN.500097
            Client's reference has a maximum field length of 12
    
            Legacy Rule Format:
            IF LENGTH(pyid.xx.xx:Identifiers.TaxAgentClientReference.Text) > 12
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRTAMI90) > 12
    
            Data Elements:
    
            ^TRTAMI90 = INT.TrueAndCorrect:Identifiers.TaxAgentClientReference.Text
            */
            assertion = (Length(report.TRTAMI90) > 12);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500097",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Client's reference must not exceed 12 characters",
                    LongDescription = @"For Client's reference, the maximum allowable characters for this form is 12",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxAgentClientReference.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500097" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI90", Value = GetValueOrEmpty(report.TRTAMI90) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500097

            #region VR.ATO.GEN.500098

            /*  VR.ATO.GEN.500098
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.TaxAgentClientReference.Text <> NULLORBLANK) AND (ANY CHARACTER OF pyid.xx.xx:Identifiers.TaxAgentClientReference.Text <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI90 <> BLANK) AND (NotCharacterInSet(^TRTAMI90, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI90 = INT.TrueAndCorrect:Identifiers.TaxAgentClientReference.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI90) != true && !(IsMatch(report.TRTAMI90, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500098",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Client's reference contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxAgentClientReference.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500098" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI90", Value = GetValueOrEmpty(report.TRTAMI90) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500098

            #region VR.ATO.GEN.500100

            /*  VR.ATO.GEN.500100
            Reporting Party Declaration Statement Accepted Indicator must be 'TRUE'
    
            Legacy Rule Format:
            IF (CONTEXT(RP.TrueAndCorrect) <> NULL) AND (RP.TrueAndCorrect:pyin.xx.xx:Declaration.StatementAccepted.Indicator <> TRUE)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^TRTAMI99 <> TRUE)
    
            Data Elements:
    
            ^TRTAMI99 = RP.TrueAndCorrect:Declaration.StatementAccepted.Indicator
    
            ^Context = RP:RP.TrueAndCorrect
            */
            assertion = (report.RPTrueAndCorrectExists != false && report.TRTAMI99 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500100",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Reporting Party Declaration Statement Accepted Indicator must be 'TRUE'",
                    Location = "/xbrli:xbrl/tns:Declaration.StatementAccepted.Indicator[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500100" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI99", Value = GetValueOrEmpty(report.TRTAMI99) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500100

            #region VR.ATO.GEN.500101

            /*  VR.ATO.GEN.500101
            Declaration Signatory identifier text must be present
    
            Legacy Rule Format:
            IF (CONTEXT(RP.TrueAndCorrect) <> NULL) AND (RP.TrueAndCorrect:pyin.xx.xx:Declaration.SignatoryIdentifier.Text = NULLORBLANK) 
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^TRTAMI100 = BLANK)
    
            Data Elements:
    
            ^TRTAMI100 = RP.TrueAndCorrect:Declaration.SignatoryIdentifier.Text
    
            ^Context = RP:RP.TrueAndCorrect
            */
            assertion = (report.RPTrueAndCorrectExists != false && string.IsNullOrWhiteSpace(report.TRTAMI100) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500101",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Reporting Party Declaration Signatory identifier text must be present",
                    Location = "/xbrli:xbrl/tns:Declaration.SignatoryIdentifier.Text[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500101" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI100", Value = report.TRTAMI100 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500101

            #region VR.ATO.GEN.500102

            /*  VR.ATO.GEN.500102
            The accounting period should commence from the 1st of the month.
    
            Legacy Rule Format:
            IF (RP:period.startDate.Day <> 1)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (Day(^StartDate1) <> 1)
    
            Data Elements:
    
            ^StartDate1 = RP
            */
            assertion = (Day(report.RPStartDate) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500102",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The period start date must commence from the first of the month",
                    LongDescription = @"This is an annual accounting submission. The dates provided should be for your annual accounting period, commencing on the 1st day of the month. For normal balancers, this will be the period 1 July - 30 June.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500102" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500102

            #region VR.ATO.GEN.500103

            /*  VR.ATO.GEN.500103
            Invalid context. The number of RP.POS contexts must equal 1.
    
            Legacy Rule Format:
            IF COUNT(RP.POS) <> 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) <> 1
    
            Data Elements:
    
            ^Context = RP:RP.POS
            */
            assertion = (Count(report.RPPOSCount) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500103",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of RP.POS contexts must equal 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPOSLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500103" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500103

            #region VR.ATO.GEN.500128

            /*  VR.ATO.GEN.500128
            Where either BSB number, account number or account name is provided, all must be present
    
            Legacy Rule Format:
            IF (pyid.xx.xx:FinancialInstitutionAccount.BankStateBranch.Number <> NULL OR pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccount.Number <> NULLORBLANK OR pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text <> NULLORBLANK) AND (pyid.xx.xx:FinancialInstitutionAccount.BankStateBranch.Number = NULL OR pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccount.Number = NULLORBLANK OR pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text = NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRTAMI41 <> NULL OR ^TRTAMI42 <> BLANK OR ^TRTAMI43 <> BLANK) AND (^TRTAMI41 = NULL OR ^TRTAMI42 = BLANK OR ^TRTAMI43 = BLANK)
    
            Data Elements:
    
            ^TRTAMI42 = RP:FinancialInstitutionAccount.FinancialInstitutionAccount.Number
    
            ^TRTAMI41 = RP:FinancialInstitutionAccount.BankStateBranch.Number
    
            ^TRTAMI43 = RP:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text
            */
            assertion = ((report.TRTAMI41 != null || string.IsNullOrWhiteSpace(report.TRTAMI42) != true || string.IsNullOrWhiteSpace(report.TRTAMI43) != true) && (report.TRTAMI41 == null || string.IsNullOrWhiteSpace(report.TRTAMI42) == true || string.IsNullOrWhiteSpace(report.TRTAMI43) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500128",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Where either BSB number, account number or account name is provided, all must be present",
                    LongDescription = @"If any EFT details are present, then a BSB number, account number and an account name must all be present. If EFT is not required, then all fields must be blank or not present.",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount.FinancialInstitutionAccount.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500128" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI41", Value = GetValueOrEmpty(report.TRTAMI41) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI42", Value = GetValueOrEmpty(report.TRTAMI42) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI43", Value = GetValueOrEmpty(report.TRTAMI43) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500128

            #region VR.ATO.GEN.500144

            /*  VR.ATO.GEN.500144
            Invalid context. Period date of Reporting Party (Instant) must match period end date of Reporting Party.
    
            Legacy Rule Format:
            IF (RP(Instant):period.instantDate <> RP:period.endDate)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^InstantDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^InstantDate1 = RP(Instant)
            */
            assertion = ((report.RPInstantExists == true) && (report.RPInstantInstantDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500144",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Reporting Party (Instant) period date is invalid",
                    LongDescription = @"Invalid context. Period date of Reporting Party (Instant) must match period end date of Reporting Party.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500144" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500144

            #region VR.ATO.GEN.500145

            /*  VR.ATO.GEN.500145
            Account number must contain the following: A to Z a to z or 0 to 9
    
            Legacy Rule Format:
            IF pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccount.Number <> NULLORBLANK AND ANY CHARACTER OF pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccount.Number <> SET("a-z","A-Z","0-9")
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRTAMI42 <> BLANK AND NotCharacterInSet(^TRTAMI42, '"a-z","A-Z","0-9"')
    
            Data Elements:
    
            ^TRTAMI42 = RP:FinancialInstitutionAccount.FinancialInstitutionAccount.Number
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI42) != true && !(IsMatch(report.TRTAMI42, @"^[a-zA-Z0-9]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500145",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Account number contains invalid text",
                    LongDescription = @"Account number must contain the following: A to Z a to z or 0 to 9",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount.FinancialInstitutionAccount.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500145" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI42", Value = GetValueOrEmpty(report.TRTAMI42) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500145

            #region VR.ATO.TRTAMI.000001

            /*  VR.ATO.TRTAMI.000001
            Invalid context. The number of Reporting Party Postal Previous contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.POS.Prev)) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.POS:RP.POS.Prev
            */
            assertion = (Count(report.RPPOSPrevCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Postal Previous contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPOSPrevLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000001" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000001

            #region VR.ATO.TRTAMI.000002

            /*  VR.ATO.TRTAMI.000002
            Invalid context. The number of Reporting Party Previous contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Prev)) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Prev
            */
            assertion = (Count(report.RPPrevCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Previous contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrevLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000002" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000002

            #region VR.ATO.TRTAMI.000003

            /*  VR.ATO.TRTAMI.000003
            Invalid context. The number of Reporting Party Statement of Attribution contexts must not exceed 500.
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.{SoASeqNum})) > 500
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 500
    
            Data Elements:
    
            ^Context = RP:RP.{SoASeqNum}
            */
            assertion = (Count(report.RPSoASeqNumContextCollection) > 500);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Statement of Attribution contexts must not exceed 500.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSoASeqNumContextCollection == null ? 0 : (report.RPSoASeqNumContextCollection.Count() == 0 ? 0 : report.RPSoASeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000003" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000003

            #region VR.ATO.TRTAMI.000006

            /*  VR.ATO.TRTAMI.000006
            The Trust Australian Business Number has failed the ABN algorithm check.
    
            Legacy Rule Format:
            IF ([TRTAMI5] <> NULLORBLANK) AND (ABNALGORITHM([TRTAMI5]) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI5] = TRTAMI:RP:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier

            Technical Business Rule Format:
            (^TRTAMI5 <> BLANK) AND (FailsABNAlgorithm(^TRTAMI5))
    
            Data Elements:
    
            ^TRTAMI5 = RP:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI5) != true && FailsABNAlgorithm(report.TRTAMI5));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000006",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trust Australian Business Number is invalid",
                    LongDescription = @"The Trust Australian Business Number has failed the ABN algorithm check",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000006" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI5", Value = report.TRTAMI5 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000006

            #region VR.ATO.TRTAMI.000007

            /*  VR.ATO.TRTAMI.000007
            The Trustee Australian Business Number has failed the ABN algorithm check.
    
            Legacy Rule Format:
            IF ([TRTAMI29] <> NULLORBLANK) AND (ABNALGORITHM([TRTAMI29]) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI29] = TRTAMI:RP.Trustee:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier

            Technical Business Rule Format:
            (^TRTAMI29 <> BLANK) AND (FailsABNAlgorithm(^TRTAMI29))
    
            Data Elements:
    
            ^TRTAMI29 = RP.Trustee:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI29) != true && FailsABNAlgorithm(report.TRTAMI29));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000007",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Australian Business Number is invalid",
                    LongDescription = @"The Trustee Australian Business Number has failed the ABN algorithm check",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000007" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI29", Value = GetValueOrEmpty(report.TRTAMI29) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000007

            #region VR.ATO.TRTAMI.000008

            /*  VR.ATO.TRTAMI.000008
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF ([TRTAMI10] <> NULLORBLANK) AND ([TRTAMI10] DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI10] = TRTAMI:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (^TRTAMI10 <> BLANK) AND (NotContainsSet(^TRTAMI10, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^TRTAMI10 = RP:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI10) != true && !(IsMatch(report.TRTAMI10, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of Trust must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000008" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI10", Value = report.TRTAMI10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000008

            #region VR.ATO.TRTAMI.000009

            /*  VR.ATO.TRTAMI.000009
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF ([TRTAMI10] CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI10] = TRTAMI:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI10, ' - ')
    
            Data Elements:
    
            ^TRTAMI10 = RP:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (report.TRTAMI10 == null ? false : report.TRTAMI10.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of Trust cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000009" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI10", Value = report.TRTAMI10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000009

            #region VR.ATO.TRTAMI.000010

            /*  VR.ATO.TRTAMI.000010
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF ([TRTAMI10] CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI10] = TRTAMI:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI10, 'P/L')
    
            Data Elements:
    
            ^TRTAMI10 = RP:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (report.TRTAMI10 == null ? false : report.TRTAMI10.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of Trust cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000010" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI10", Value = report.TRTAMI10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000010

            #region VR.ATO.TRTAMI.000011

            /*  VR.ATO.TRTAMI.000011
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF ([TRTAMI10] CONTAINS SET("--","''","  "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI10] = TRTAMI:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (ContainsSet(^TRTAMI10, '"--","''","  "'))
    
            Data Elements:
    
            ^TRTAMI10 = RP:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = IsMatch(report.TRTAMI10, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of Trust cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000011" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI10", Value = report.TRTAMI10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000011

            #region VR.ATO.TRTAMI.000012

            /*  VR.ATO.TRTAMI.000012
            Non Individual Name cannot start with "T/A" and end with "Pship", "P'ship" or "P/Ship"
    
            Legacy Rule Format:
            IF ([TRTAMI10] STARTSWITH "T/A ") AND ([TRTAMI10] ENDSWITH SET(" Pship"," P'ship"," P/ship"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI10] = TRTAMI:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (StartsWith(^TRTAMI10, 'T/A ')) AND (EndsWithSet(^TRTAMI10, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^TRTAMI10 = RP:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (IsMatch(report.TRTAMI10, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(report.TRTAMI10, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000012",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of Trust cannot start with ""T/A"" and end with ""Pship"", ""P'ship"" or ""P/Ship""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000012" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI10", Value = report.TRTAMI10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000012

            #region VR.ATO.TRTAMI.000014

            /*  VR.ATO.TRTAMI.000014
            Organisation name must be present
    
            Legacy Rule Format:
            IF ([TRTAMI10] = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI10] = TRTAMI:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            ^TRTAMI10 = BLANK
    
            Data Elements:
    
            ^TRTAMI10 = RP:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI10) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000014",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of Trust must be provided",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000014" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI10", Value = report.TRTAMI10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000014

            #region VR.ATO.TRTAMI.000015

            /*  VR.ATO.TRTAMI.000015
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF ([TRTAMI10] ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI10] = TRTAMI:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (EndsWithSet(^TRTAMI10, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            ^TRTAMI10 = RP:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = IsMatch(report.TRTAMI10, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000015",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of Trust cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000015" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI10", Value = report.TRTAMI10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000015

            #region VR.ATO.TRTAMI.000016

            /*  VR.ATO.TRTAMI.000016
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF ([TRTAMI11] <> NULLORBLANK) AND ([TRTAMI11] DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI11] = TRTAMI:RP.Prev:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (^TRTAMI11 <> BLANK) AND (NotContainsSet(^TRTAMI11, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^TRTAMI11 = RP.Prev:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI11) != true && !(IsMatch(report.TRTAMI11, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000016",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Name of Trust must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000016" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI11", Value = GetValueOrEmpty(report.TRTAMI11) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000016

            #region VR.ATO.TRTAMI.000017

            /*  VR.ATO.TRTAMI.000017
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF ([TRTAMI11] CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI11] = TRTAMI:RP.Prev:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI11, ' - ')
    
            Data Elements:
    
            ^TRTAMI11 = RP.Prev:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (report.TRTAMI11 == null ? false : report.TRTAMI11.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Name of Trust cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000017" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI11", Value = GetValueOrEmpty(report.TRTAMI11) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000017

            #region VR.ATO.TRTAMI.000018

            /*  VR.ATO.TRTAMI.000018
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF ([TRTAMI11] CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI11] = TRTAMI:RP.Prev:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI11, 'P/L')
    
            Data Elements:
    
            ^TRTAMI11 = RP.Prev:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (report.TRTAMI11 == null ? false : report.TRTAMI11.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000018",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Name of Trust cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000018" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI11", Value = GetValueOrEmpty(report.TRTAMI11) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000018

            #region VR.ATO.TRTAMI.000019

            /*  VR.ATO.TRTAMI.000019
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF ([TRTAMI11] CONTAINS SET("--","''","  "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI11] = TRTAMI:RP.Prev:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (ContainsSet(^TRTAMI11, '"--","''","  "'))
    
            Data Elements:
    
            ^TRTAMI11 = RP.Prev:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = IsMatch(report.TRTAMI11, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000019",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Name of Trust cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000019" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI11", Value = GetValueOrEmpty(report.TRTAMI11) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000019

            #region VR.ATO.TRTAMI.000020

            /*  VR.ATO.TRTAMI.000020
            Non Individual Name cannot start with "T/A" and end with "Pship", "P'ship" or "P/Ship"
    
            Legacy Rule Format:
            IF ([TRTAMI11] STARTSWITH "T/A ") AND ([TRTAMI11] ENDSWITH SET(" Pship"," P'ship"," P/ship"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI11] = TRTAMI:RP.Prev:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (StartsWith(^TRTAMI11, 'T/A ')) AND (EndsWithSet(^TRTAMI11, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^TRTAMI11 = RP.Prev:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (IsMatch(report.TRTAMI11, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(report.TRTAMI11, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000020",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Name of Trust cannot start with ""T/A"" and end with ""Pship"", ""P'ship"" or ""P/Ship""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000020" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI11", Value = GetValueOrEmpty(report.TRTAMI11) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000020

            #region VR.ATO.TRTAMI.000023

            /*  VR.ATO.TRTAMI.000023
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF ([TRTAMI11] ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI11] = TRTAMI:RP.Prev:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (EndsWithSet(^TRTAMI11, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            ^TRTAMI11 = RP.Prev:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = IsMatch(report.TRTAMI11, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Name of Trust cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000023" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI11", Value = GetValueOrEmpty(report.TRTAMI11) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000023

            #region VR.ATO.TRTAMI.000024

            /*  VR.ATO.TRTAMI.000024
            Trustee Daytime Telephone Number has a maximum field length of 15
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI31]) > 15
                 RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI31] = TRTAMI:RP.Trustee:pyde.02.00:ElectronicContact.Telephone.Minimal.Number

            Technical Business Rule Format:
            Length(^TRTAMI31) > 15
    
            Data Elements:
    
            ^TRTAMI31 = RP.Trustee:ElectronicContact.Telephone.Minimal.Number
            */
            assertion = (Length(report.TRTAMI31) > 15);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000024",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Daytime Telephone Number must not exceed 15 characters",
                    LongDescription = @"For Trustee Daytime Telephone Number, the maximum allowable characters for this form is 15",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Minimal.Number[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000024" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI31", Value = report.TRTAMI31 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000024

            #region VR.ATO.TRTAMI.000025

            /*  VR.ATO.TRTAMI.000025
            Trustee Daytime Telephone Number is mandatory
    
            Legacy Rule Format:
            IF [TRTAMI31] = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI31] = TRTAMI:RP.Trustee:pyde.02.00:ElectronicContact.Telephone.Minimal.Number

            Technical Business Rule Format:
            ^TRTAMI31 = BLANK
    
            Data Elements:
    
            ^TRTAMI31 = RP.Trustee:ElectronicContact.Telephone.Minimal.Number
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI31) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Daytime Telephone Number must be provided",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Minimal.Number[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000025" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI31", Value = report.TRTAMI31 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000025

            #region VR.ATO.TRTAMI.000026

            /*  VR.ATO.TRTAMI.000026
            Declarer Telephone number has a maximum field length of 15
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI86]) > 15
                 RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI86] = TRTAMI:INT.TrueAndCorrect:pyde.02.00:ElectronicContact.Telephone.Minimal.Number

            Technical Business Rule Format:
            Length(^TRTAMI86) > 15
    
            Data Elements:
    
            ^TRTAMI86 = INT.TrueAndCorrect:ElectronicContact.Telephone.Minimal.Number
            */
            assertion = (Length(report.TRTAMI86) > 15);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000026",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer Telephone number must not exceed 15 characters",
                    LongDescription = @"For Declarer Telephone number, the maximum allowable characters for this form is 15",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Minimal.Number[@contextRef='" + report.INTTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000026" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI86", Value = report.TRTAMI86 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000026

            #region VR.ATO.TRTAMI.000027

            /*  VR.ATO.TRTAMI.000027
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI12]) > 38
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI12] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            Length(^TRTAMI12) > 38
    
            Data Elements:
    
            ^TRTAMI12 = RP.POS:AddressDetails.Line1.Text
            */
            assertion = (Length(report.TRTAMI12) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000027",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 1 must not exceed 38 characters",
                    LongDescription = @"For Current Postal Address of Trust - Address Line 1, the maximum allowable characters for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000027" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI12", Value = report.TRTAMI12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000027

            #region VR.ATO.TRTAMI.000028

            /*  VR.ATO.TRTAMI.000028
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF [TRTAMI12] = FOUND("AS ABOVE")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI12] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            FoundSet(^TRTAMI12, '"AS ABOVE"')
    
            Data Elements:
    
            ^TRTAMI12 = RP.POS:AddressDetails.Line1.Text
            */
            assertion = IsMatch(report.TRTAMI12, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000028",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000028" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI12", Value = report.TRTAMI12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000028

            #region VR.ATO.TRTAMI.000029

            /*  VR.ATO.TRTAMI.000029
            Address Line 1 must be present
    
            Legacy Rule Format:
            IF [TRTAMI12] = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI12] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            ^TRTAMI12 = BLANK
    
            Data Elements:
    
            ^TRTAMI12 = RP.POS:AddressDetails.Line1.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI12) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 1 must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000029" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI12", Value = report.TRTAMI12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000029

            #region VR.ATO.TRTAMI.000030

            /*  VR.ATO.TRTAMI.000030
            Address line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-.
    
            Legacy Rule Format:
            IF [TRTAMI12] = FOUND("C/O","C/","Care Of","CO")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI12] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            FoundSet(^TRTAMI12, '"C/O","C/","Care Of","CO"')
    
            Data Elements:
    
            ^TRTAMI12 = RP.POS:AddressDetails.Line1.Text
            */
            assertion = IsMatch(report.TRTAMI12, @"(^(C/O|C/|Care Of|CO)$)|(^(.*)( C/O | C/ | Care Of | CO )(.*)$)|(^(C/O |C/ |Care Of |CO )(.*)$)|(^(.*)( C/O| C/| Care Of| CO)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 1 is invalid",
                    LongDescription = @"Current Postal Address of Trust - Address Line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000030" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI12", Value = report.TRTAMI12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000030

            #region VR.ATO.TRTAMI.000031

            /*  VR.ATO.TRTAMI.000031
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF [TRTAMI12] CONTAINS "UNKNOWN"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI12] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            Contains(^TRTAMI12, 'UNKNOWN')
    
            Data Elements:
    
            ^TRTAMI12 = RP.POS:AddressDetails.Line1.Text
            */
            assertion = (report.TRTAMI12 == null ? false : report.TRTAMI12.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000031",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000031" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI12", Value = report.TRTAMI12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000031

            #region VR.ATO.TRTAMI.000032

            /*  VR.ATO.TRTAMI.000032
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI13]) > 38
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI13] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            Length(^TRTAMI13) > 38
    
            Data Elements:
    
            ^TRTAMI13 = RP.POS:AddressDetails.Line2.Text
            */
            assertion = (Length(report.TRTAMI13) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000032",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 2 must not exceed 38 characters",
                    LongDescription = @"For Current Postal Address of Trust - Address Line 2, the maximum allowable characters for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000032" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI13", Value = GetValueOrEmpty(report.TRTAMI13) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000032

            #region VR.ATO.TRTAMI.000033

            /*  VR.ATO.TRTAMI.000033
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF [TRTAMI13] = FOUND("C/-","C/O","C/","Care Of","CO")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI13] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            FoundSet(^TRTAMI13, '"C/-","C/O","C/","Care Of","CO"')
    
            Data Elements:
    
            ^TRTAMI13 = RP.POS:AddressDetails.Line2.Text
            */
            assertion = IsMatch(report.TRTAMI13, @"(^(C/-|C/O|C/|Care Of|CO)$)|(^(.*)( C/- | C/O | C/ | Care Of | CO )(.*)$)|(^(C/- |C/O |C/ |Care Of |CO )(.*)$)|(^(.*)( C/-| C/O| C/| Care Of| CO)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for Current Postal Address of Trust - Address Line 2",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000033" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI13", Value = GetValueOrEmpty(report.TRTAMI13) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000033

            #region VR.ATO.TRTAMI.000034

            /*  VR.ATO.TRTAMI.000034
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF [TRTAMI13] CONTAINS "UNKNOWN"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI13] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            Contains(^TRTAMI13, 'UNKNOWN')
    
            Data Elements:
    
            ^TRTAMI13 = RP.POS:AddressDetails.Line2.Text
            */
            assertion = (report.TRTAMI13 == null ? false : report.TRTAMI13.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000034",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000034" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI13", Value = GetValueOrEmpty(report.TRTAMI13) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000034

            #region VR.ATO.TRTAMI.000035

            /*  VR.ATO.TRTAMI.000035
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            IF ([TRTAMI12] CONTAINS "C/-") AND ([TRTAMI13] = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI12] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line1.Text
            [TRTAMI13] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            (Contains(^TRTAMI12, 'C/-')) AND (^TRTAMI13 = BLANK)
    
            Data Elements:
    
            ^TRTAMI13 = RP.POS:AddressDetails.Line2.Text
    
            ^TRTAMI12 = RP.POS:AddressDetails.Line1.Text
            */
            assertion = ((report.TRTAMI12 == null ? false : report.TRTAMI12.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.TRTAMI13) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000035",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 2 must be provided",
                    LongDescription = @"When Current Postal Address of Trust - Address Line 1 contains 'C/-', Current Postal Address of Trust - Address Line 2 must be provided",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000035" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI12", Value = report.TRTAMI12 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI13", Value = GetValueOrEmpty(report.TRTAMI13) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000035

            #region VR.ATO.TRTAMI.000036

            /*  VR.ATO.TRTAMI.000036
            Suburb/Town has a maximum field length of 46
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI14]) > 46
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI14] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.LocalityName.Text

            Technical Business Rule Format:
            Length(^TRTAMI14) > 46
    
            Data Elements:
    
            ^TRTAMI14 = RP.POS:AddressDetails.LocalityName.Text
            */
            assertion = (Length(report.TRTAMI14) > 46);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000036",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Suburb/Town must not exceed 46 characters",
                    LongDescription = @"For Current Postal Address of Trust - Suburb/Town, the maximum allowable characters for this form is 46",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000036" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI14", Value = report.TRTAMI14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000036

            #region VR.ATO.TRTAMI.000037

            /*  VR.ATO.TRTAMI.000037
            State code [QLD,NSW,VIC,SA,WA,NT,ACT,TAS] or a leading space or combination of spaces with a State code is present in the suburb/town field.
    
            Legacy Rule Format:
            IF ([TRTAMI17] = NULL OR [TRTAMI17] = "au") AND ([TRTAMI14] = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI14] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.LocalityName.Text
            [TRTAMI17] = TRTAMI:RP.POS:pyde.02.08:AddressDetails.Country.Code

            Technical Business Rule Format:
            (^TRTAMI17 = NULL OR ^TRTAMI17 = 'au') AND (FoundSet(^TRTAMI14, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^TRTAMI14 = RP.POS:AddressDetails.LocalityName.Text
    
            ^TRTAMI17 = RP.POS:AddressDetails.Country.Code
            */
            assertion = ((report.TRTAMI17 == null || report.TRTAMI17 == @"au") && IsMatch(report.TRTAMI14, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000037",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Suburb/Town cannot contain state code",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000037" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI17", Value = GetValueOrEmpty(report.TRTAMI17) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI14", Value = report.TRTAMI14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000037

            #region VR.ATO.TRTAMI.000038

            /*  VR.ATO.TRTAMI.000038
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF [TRTAMI14] CONTAINS "UNKNOWN"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI14] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.LocalityName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI14, 'UNKNOWN')
    
            Data Elements:
    
            ^TRTAMI14 = RP.POS:AddressDetails.LocalityName.Text
            */
            assertion = (report.TRTAMI14 == null ? false : report.TRTAMI14.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000038",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Suburb/Town cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000038" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI14", Value = report.TRTAMI14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000038

            #region VR.ATO.TRTAMI.000039

            /*  VR.ATO.TRTAMI.000039
            Suburb/Town/City is mandatory
    
            Legacy Rule Format:
            IF [TRTAMI14] = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI14] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.LocalityName.Text

            Technical Business Rule Format:
            ^TRTAMI14 = BLANK
    
            Data Elements:
    
            ^TRTAMI14 = RP.POS:AddressDetails.LocalityName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI14) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000039",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Suburb/Town must be provided",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000039" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI14", Value = report.TRTAMI14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000039

            #region VR.ATO.TRTAMI.000040

            /*  VR.ATO.TRTAMI.000040
            Numerics [0 to 9] are not allowed in the suburb/town field.
    
            Legacy Rule Format:
            IF ([TRTAMI17] = NULL OR [TRTAMI17] = "au") AND ([TRTAMI14] CONTAINS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI14] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.LocalityName.Text
            [TRTAMI17] = TRTAMI:RP.POS:pyde.02.08:AddressDetails.Country.Code

            Technical Business Rule Format:
            (^TRTAMI17 = NULL OR ^TRTAMI17 = 'au') AND (ContainsSet(^TRTAMI14, '0-9'))
    
            Data Elements:
    
            ^TRTAMI14 = RP.POS:AddressDetails.LocalityName.Text
    
            ^TRTAMI17 = RP.POS:AddressDetails.Country.Code
            */
            assertion = ((report.TRTAMI17 == null || report.TRTAMI17 == @"au") && IsMatch(report.TRTAMI14, @"[0-9]", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000040",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Suburb/Town is invalid",
                    LongDescription = @"When an Australian address is provided, Current Postal Address of Trust - Suburb/Town must not contain numeric characters",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000040" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI17", Value = GetValueOrEmpty(report.TRTAMI17) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI14", Value = report.TRTAMI14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000040

            #region VR.ATO.TRTAMI.000041

            /*  VR.ATO.TRTAMI.000041
            Value entered is not a valid code/value for that field.
    
            Legacy Rule Format:
            IF ([TRTAMI16] <> NULL) AND ([TRTAMI16] <> SET(0200-9999))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI16] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Postcode.Text

            Technical Business Rule Format:
            (^TRTAMI16 <> BLANK) AND (IsNumeric(^TRTAMI16) = 'FALSE' OR AsNumeric(^TRTAMI16) < 200 OR AsNumeric(^TRTAMI16) > 9999)
    
            Data Elements:
    
            ^TRTAMI16 = RP.POS:AddressDetails.Postcode.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI16) != true && (IsNumeric(report.TRTAMI16) == false || AsNumeric(report.TRTAMI16) < 200 || AsNumeric(report.TRTAMI16) > 9999));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000041",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Postcode is invalid",
                    LongDescription = @"For an Australian address, Current Postal Address of Trust - Postcode is invalid. Valid range is 0200 - 9999.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Postcode.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000041" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI16", Value = GetValueOrEmpty(report.TRTAMI16) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000041

            #region VR.ATO.TRTAMI.000042

            /*  VR.ATO.TRTAMI.000042
            State is present in the address field without a postcode.
    
            Legacy Rule Format:
            IF ([TRTAMI15] <> NULL) AND ([TRTAMI16] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI15] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.StateOrTerritory.Code
            [TRTAMI16] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Postcode.Text

            Technical Business Rule Format:
            (^TRTAMI15 <> NULL) AND (^TRTAMI16 = NULL)
    
            Data Elements:
    
            ^TRTAMI16 = RP.POS:AddressDetails.Postcode.Text
    
            ^TRTAMI15 = RP.POS:AddressDetails.StateOrTerritory.Code
            */
            assertion = (report.TRTAMI15 != null && report.TRTAMI16 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000042",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Postcode must be present",
                    LongDescription = @"If the Current Postal Address of Trust is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Postcode.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000042" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI15", Value = GetValueOrEmpty(report.TRTAMI15) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI16", Value = GetValueOrEmpty(report.TRTAMI16) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000042

            #region VR.ATO.TRTAMI.000043

            /*  VR.ATO.TRTAMI.000043
            State, postcode, or country are missing from the address field. Determine whether the address is Australian or overseas and correct accordingly.
    
            Legacy Rule Format:
            IF ([TRTAMI15] = NULL) AND ([TRTAMI16] = NULL) AND ([TRTAMI17] = NULL OR [TRTAMI17] = "au")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI15] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.StateOrTerritory.Code
            [TRTAMI16] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Postcode.Text
            [TRTAMI17] = TRTAMI:RP.POS:pyde.02.08:AddressDetails.Country.Code

            Technical Business Rule Format:
            (^TRTAMI15 = NULL) AND (^TRTAMI16 = NULL) AND (^TRTAMI17 = NULL OR ^TRTAMI17 = 'au')
    
            Data Elements:
    
            ^TRTAMI15 = RP.POS:AddressDetails.StateOrTerritory.Code
    
            ^TRTAMI16 = RP.POS:AddressDetails.Postcode.Text
    
            ^TRTAMI17 = RP.POS:AddressDetails.Country.Code
            */
            assertion = (report.TRTAMI15 == null && report.TRTAMI16 == null && (report.TRTAMI17 == null || report.TRTAMI17 == @"au"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000043",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - State and postcode are required for Australian addresses",
                    LongDescription = @"Current Postal Address of Trust - State and postcode, or country are required. Determine whether the address is Australian or overseas and correct accordingly.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.StateOrTerritory.Code[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000043" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI15", Value = GetValueOrEmpty(report.TRTAMI15) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI16", Value = GetValueOrEmpty(report.TRTAMI16) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI17", Value = GetValueOrEmpty(report.TRTAMI17) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000043

            #region VR.ATO.TRTAMI.000044

            /*  VR.ATO.TRTAMI.000044
            Postcode is present in the address field without a State code.
    
            Legacy Rule Format:
            IF ([TRTAMI16] <> NULL) AND ([TRTAMI15] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI15] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.StateOrTerritory.Code
            [TRTAMI16] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Postcode.Text

            Technical Business Rule Format:
            (^TRTAMI16 <> NULL) AND (^TRTAMI15 = NULL)
    
            Data Elements:
    
            ^TRTAMI15 = RP.POS:AddressDetails.StateOrTerritory.Code
    
            ^TRTAMI16 = RP.POS:AddressDetails.Postcode.Text
            */
            assertion = (report.TRTAMI16 != null && report.TRTAMI15 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000044",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - State must be present",
                    LongDescription = @"If the Current Postal Address of Trust is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.StateOrTerritory.Code[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000044" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI16", Value = GetValueOrEmpty(report.TRTAMI16) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI15", Value = GetValueOrEmpty(report.TRTAMI15) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000044

            #region VR.ATO.TRTAMI.000045

            /*  VR.ATO.TRTAMI.000045
            Country name is present with State or postcode in the address field.  Determine whether the address is Australian or overseas and correct accordingly.
    
            Legacy Rule Format:
            IF ([TRTAMI17] <> NULL AND [TRTAMI17] <> "au") AND ([TRTAMI15] <> NULL OR [TRTAMI16] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI15] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.StateOrTerritory.Code
            [TRTAMI16] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Postcode.Text
            [TRTAMI17] = TRTAMI:RP.POS:pyde.02.08:AddressDetails.Country.Code

            Technical Business Rule Format:
            (^TRTAMI17 <> NULL AND ^TRTAMI17 <> 'au') AND (^TRTAMI15 <> NULL OR ^TRTAMI16 <> NULL)
    
            Data Elements:
    
            ^TRTAMI17 = RP.POS:AddressDetails.Country.Code
    
            ^TRTAMI15 = RP.POS:AddressDetails.StateOrTerritory.Code
    
            ^TRTAMI16 = RP.POS:AddressDetails.Postcode.Text
            */
            assertion = (report.TRTAMI17 != null && report.TRTAMI17 != @"au" && (report.TRTAMI15 != null || report.TRTAMI16 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000045",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Australia must be entered as the country where an Australian state or territory has been entered",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Country.Code[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000045" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI17", Value = GetValueOrEmpty(report.TRTAMI17) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI15", Value = GetValueOrEmpty(report.TRTAMI15) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI16", Value = GetValueOrEmpty(report.TRTAMI16) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000045

            #region VR.ATO.TRTAMI.000046

            /*  VR.ATO.TRTAMI.000046
            Previous postal address is only required when an address change has occurred since the last return.  Delete previous postal address or input the old address if a change of address has occurred.
    
            Legacy Rule Format:
            IF ([TRTAMI18] = [TRTAMI12]) AND ([TRTAMI19] = [TRTAMI13]) AND ([TRTAMI20] = [TRTAMI14]) AND ([TRTAMI21] = [TRTAMI15]) AND ([TRTAMI22] = [TRTAMI16])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI12] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line1.Text
            [TRTAMI13] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line2.Text
            [TRTAMI14] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.LocalityName.Text
            [TRTAMI15] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.StateOrTerritory.Code
            [TRTAMI16] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Postcode.Text
            [TRTAMI18] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line1.Text
            [TRTAMI19] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line2.Text
            [TRTAMI20] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.LocalityName.Text
            [TRTAMI21] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.StateOrTerritory.Code
            [TRTAMI22] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Postcode.Text

            Technical Business Rule Format:
            (^TRTAMI18 = ^TRTAMI12) AND (^TRTAMI19 = ^TRTAMI13) AND (^TRTAMI20 = ^TRTAMI14) AND (^TRTAMI21 = ^TRTAMI15) AND (^TRTAMI22 = ^TRTAMI16)
    
            Data Elements:
    
            ^TRTAMI18 = RP.POS.Prev:AddressDetails.Line1.Text
    
            ^TRTAMI12 = RP.POS:AddressDetails.Line1.Text
    
            ^TRTAMI13 = RP.POS:AddressDetails.Line2.Text
    
            ^TRTAMI14 = RP.POS:AddressDetails.LocalityName.Text
    
            ^TRTAMI15 = RP.POS:AddressDetails.StateOrTerritory.Code
    
            ^TRTAMI16 = RP.POS:AddressDetails.Postcode.Text
    
            ^TRTAMI19 = RP.POS.Prev:AddressDetails.Line2.Text
    
            ^TRTAMI20 = RP.POS.Prev:AddressDetails.LocalityName.Text
    
            ^TRTAMI21 = RP.POS.Prev:AddressDetails.StateOrTerritory.Code
    
            ^TRTAMI22 = RP.POS.Prev:AddressDetails.Postcode.Text
            */
            assertion = (report.TRTAMI18 == report.TRTAMI12 && report.TRTAMI19 == report.TRTAMI13 && report.TRTAMI20 == report.TRTAMI14 && report.TRTAMI21 == report.TRTAMI15 && report.TRTAMI22 == report.TRTAMI16);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous postal address must not be the same as current postal address",
                    LongDescription = @"Previous postal address is only required when an address change has occurred since the last return.  Delete previous postal address or input the old address if a change of address has occurred.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000046" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI18", Value = GetValueOrEmpty(report.TRTAMI18) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI12", Value = report.TRTAMI12 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI19", Value = GetValueOrEmpty(report.TRTAMI19) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI13", Value = GetValueOrEmpty(report.TRTAMI13) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI20", Value = GetValueOrEmpty(report.TRTAMI20) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI14", Value = report.TRTAMI14 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI21", Value = GetValueOrEmpty(report.TRTAMI21) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI15", Value = GetValueOrEmpty(report.TRTAMI15) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI22", Value = GetValueOrEmpty(report.TRTAMI22) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI16", Value = GetValueOrEmpty(report.TRTAMI16) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000046

            #region VR.ATO.TRTAMI.000047

            /*  VR.ATO.TRTAMI.000047
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI18]) > 38
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI18] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            Length(^TRTAMI18) > 38
    
            Data Elements:
    
            ^TRTAMI18 = RP.POS.Prev:AddressDetails.Line1.Text
            */
            assertion = (Length(report.TRTAMI18) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000047",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Address Line 1 must not exceed 38 characters",
                    LongDescription = @"For Previous Postal Address of Trust - Address Line 1, the maximum allowable characters for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000047" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI18", Value = GetValueOrEmpty(report.TRTAMI18) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000047

            #region VR.ATO.TRTAMI.000048

            /*  VR.ATO.TRTAMI.000048
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF [TRTAMI18] = FOUND("AS ABOVE")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI18] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            FoundSet(^TRTAMI18, '"AS ABOVE"')
    
            Data Elements:
    
            ^TRTAMI18 = RP.POS.Prev:AddressDetails.Line1.Text
            */
            assertion = IsMatch(report.TRTAMI18, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000048",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000048" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI18", Value = GetValueOrEmpty(report.TRTAMI18) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000048

            #region VR.ATO.TRTAMI.000049

            /*  VR.ATO.TRTAMI.000049
            Address line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-.
    
            Legacy Rule Format:
            IF [TRTAMI18] = FOUND("C/O","C/","Care Of","CO")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI18] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            FoundSet(^TRTAMI18, '"C/O","C/","Care Of","CO"')
    
            Data Elements:
    
            ^TRTAMI18 = RP.POS.Prev:AddressDetails.Line1.Text
            */
            assertion = IsMatch(report.TRTAMI18, @"(^(C/O|C/|Care Of|CO)$)|(^(.*)( C/O | C/ | Care Of | CO )(.*)$)|(^(C/O |C/ |Care Of |CO )(.*)$)|(^(.*)( C/O| C/| Care Of| CO)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000049",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Address Line 1 is invalid",
                    LongDescription = @"Previous Postal Address of Trust - Address Line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000049" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI18", Value = GetValueOrEmpty(report.TRTAMI18) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000049

            #region VR.ATO.TRTAMI.000050

            /*  VR.ATO.TRTAMI.000050
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF [TRTAMI18] CONTAINS "UNKNOWN"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI18] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            Contains(^TRTAMI18, 'UNKNOWN')
    
            Data Elements:
    
            ^TRTAMI18 = RP.POS.Prev:AddressDetails.Line1.Text
            */
            assertion = (report.TRTAMI18 == null ? false : report.TRTAMI18.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000050",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Address Line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000050" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI18", Value = GetValueOrEmpty(report.TRTAMI18) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000050

            #region VR.ATO.TRTAMI.000051

            /*  VR.ATO.TRTAMI.000051
            When any Previous Postal Address for Trust details are provided, Address Line 1 and Suburb/Town must be provided, including State and Postcode if address is Australian
    
            Legacy Rule Format:
            IF ([TRTAMI18] <> NULLORBLANK OR [TRTAMI19] <> NULLORBLANK OR [TRTAMI20] <> NULLORBLANK OR [TRTAMI21] <> NULL OR [TRTAMI22] <> NULL OR ([TRTAMI23] <> NULL AND [TRTAMI23] <> "au")) AND ([TRTAMI18] = NULLORBLANK OR [TRTAMI20] = NULLORBLANK OR ([TRTAMI21] = NULL AND [TRTAMI22] = NULL AND ([TRTAMI23] = NULL OR [TRTAMI23] = "au")))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI18] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line1.Text
            [TRTAMI19] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line2.Text
            [TRTAMI20] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.LocalityName.Text
            [TRTAMI21] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.StateOrTerritory.Code
            [TRTAMI22] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Postcode.Text
            [TRTAMI23] = TRTAMI:RP.POS.Prev:pyde.02.08:AddressDetails.Country.Code

            Technical Business Rule Format:
            (^TRTAMI18 <> BLANK OR ^TRTAMI19 <> BLANK OR ^TRTAMI20 <> BLANK OR ^TRTAMI21 <> NULL OR ^TRTAMI22 <> NULL OR (^TRTAMI23 <> NULL AND ^TRTAMI23 <> 'au')) AND (^TRTAMI18 = BLANK OR ^TRTAMI20 = BLANK OR (^TRTAMI21 = NULL AND ^TRTAMI22 = NULL AND (^TRTAMI23 = NULL OR ^TRTAMI23 = 'au')))
    
            Data Elements:
    
            ^TRTAMI18 = RP.POS.Prev:AddressDetails.Line1.Text
    
            ^TRTAMI19 = RP.POS.Prev:AddressDetails.Line2.Text
    
            ^TRTAMI20 = RP.POS.Prev:AddressDetails.LocalityName.Text
    
            ^TRTAMI21 = RP.POS.Prev:AddressDetails.StateOrTerritory.Code
    
            ^TRTAMI22 = RP.POS.Prev:AddressDetails.Postcode.Text
    
            ^TRTAMI23 = RP.POS.Prev:AddressDetails.Country.Code
            */
            assertion = ((string.IsNullOrWhiteSpace(report.TRTAMI18) != true || string.IsNullOrWhiteSpace(report.TRTAMI19) != true || string.IsNullOrWhiteSpace(report.TRTAMI20) != true || report.TRTAMI21 != null || report.TRTAMI22 != null || report.TRTAMI23 != null && report.TRTAMI23 != @"au") && (string.IsNullOrWhiteSpace(report.TRTAMI18) == true || string.IsNullOrWhiteSpace(report.TRTAMI20) == true || report.TRTAMI21 == null && report.TRTAMI22 == null && (report.TRTAMI23 == null || report.TRTAMI23 == @"au")));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000051",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Full Previous Postal Address of Trust details must be provided",
                    LongDescription = @"When any Previous Postal Address for Trust details are provided, Address Line 1 and Suburb/Town must be provided, including State and Postcode if address is Australian",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000051" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI18", Value = GetValueOrEmpty(report.TRTAMI18) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI19", Value = GetValueOrEmpty(report.TRTAMI19) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI20", Value = GetValueOrEmpty(report.TRTAMI20) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI21", Value = GetValueOrEmpty(report.TRTAMI21) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI22", Value = GetValueOrEmpty(report.TRTAMI22) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI23", Value = GetValueOrEmpty(report.TRTAMI23) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000051

            #region VR.ATO.TRTAMI.000052

            /*  VR.ATO.TRTAMI.000052
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI19]) > 38
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI19] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            Length(^TRTAMI19) > 38
    
            Data Elements:
    
            ^TRTAMI19 = RP.POS.Prev:AddressDetails.Line2.Text
            */
            assertion = (Length(report.TRTAMI19) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000052",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Address Line 2 must not exceed 38 characters",
                    LongDescription = @"For Previous Postal Address of Trust - Address Line 2, the maximum allowable characters for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000052" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI19", Value = GetValueOrEmpty(report.TRTAMI19) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000052

            #region VR.ATO.TRTAMI.000053

            /*  VR.ATO.TRTAMI.000053
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF [TRTAMI19] = FOUND("C/-","C/O","C/","Care Of","CO")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI19] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            FoundSet(^TRTAMI19, '"C/-","C/O","C/","Care Of","CO"')
    
            Data Elements:
    
            ^TRTAMI19 = RP.POS.Prev:AddressDetails.Line2.Text
            */
            assertion = IsMatch(report.TRTAMI19, @"(^(C/-|C/O|C/|Care Of|CO)$)|(^(.*)( C/- | C/O | C/ | Care Of | CO )(.*)$)|(^(C/- |C/O |C/ |Care Of |CO )(.*)$)|(^(.*)( C/-| C/O| C/| Care Of| CO)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for Previous Postal Address of Trust - Address Line 2",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000053" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI19", Value = GetValueOrEmpty(report.TRTAMI19) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000053

            #region VR.ATO.TRTAMI.000054

            /*  VR.ATO.TRTAMI.000054
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF [TRTAMI19] CONTAINS "UNKNOWN"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI19] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            Contains(^TRTAMI19, 'UNKNOWN')
    
            Data Elements:
    
            ^TRTAMI19 = RP.POS.Prev:AddressDetails.Line2.Text
            */
            assertion = (report.TRTAMI19 == null ? false : report.TRTAMI19.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Address Line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000054" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI19", Value = GetValueOrEmpty(report.TRTAMI19) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000054

            #region VR.ATO.TRTAMI.000055

            /*  VR.ATO.TRTAMI.000055
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            IF ([TRTAMI18] CONTAINS "C/-") AND ([TRTAMI19] = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI18] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line1.Text
            [TRTAMI19] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            (Contains(^TRTAMI18, 'C/-')) AND (^TRTAMI19 = BLANK)
    
            Data Elements:
    
            ^TRTAMI19 = RP.POS.Prev:AddressDetails.Line2.Text
    
            ^TRTAMI18 = RP.POS.Prev:AddressDetails.Line1.Text
            */
            assertion = ((report.TRTAMI18 == null ? false : report.TRTAMI18.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.TRTAMI19) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000055",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Address Line 2 must be provided",
                    LongDescription = @"When Previous Postal Address of Trust - Address Line 1 contains 'C/-', Current Previous Address of Trust - Address Line 2 must be provided",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000055" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI18", Value = GetValueOrEmpty(report.TRTAMI18) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI19", Value = GetValueOrEmpty(report.TRTAMI19) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000055

            #region VR.ATO.TRTAMI.000056

            /*  VR.ATO.TRTAMI.000056
            Suburb/Town has a maximum field length of 46
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI20]) > 46
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI20] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.LocalityName.Text

            Technical Business Rule Format:
            Length(^TRTAMI20) > 46
    
            Data Elements:
    
            ^TRTAMI20 = RP.POS.Prev:AddressDetails.LocalityName.Text
            */
            assertion = (Length(report.TRTAMI20) > 46);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000056",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Suburb/Town must not exceed 46 characters",
                    LongDescription = @"For Previous Postal Address of Trust - Suburb/Town, the maximum allowable characters for this form is 46",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000056" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI20", Value = GetValueOrEmpty(report.TRTAMI20) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000056

            #region VR.ATO.TRTAMI.000057

            /*  VR.ATO.TRTAMI.000057
            State code [QLD,NSW,VIC,SA,WA,NT,ACT,TAS] or a leading space or combination of spaces with a State code is present in the suburb/town field.
    
            Legacy Rule Format:
            IF ([TRTAMI23] = NULL OR [TRTAMI23] = "au") AND ([TRTAMI20] = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI20] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.LocalityName.Text
            [TRTAMI23] = TRTAMI:RP.POS.Prev:pyde.02.08:AddressDetails.Country.Code

            Technical Business Rule Format:
            (^TRTAMI23 = NULL OR ^TRTAMI23 = 'au') AND (FoundSet(^TRTAMI20, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^TRTAMI20 = RP.POS.Prev:AddressDetails.LocalityName.Text
    
            ^TRTAMI23 = RP.POS.Prev:AddressDetails.Country.Code
            */
            assertion = ((report.TRTAMI23 == null || report.TRTAMI23 == @"au") && IsMatch(report.TRTAMI20, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000057",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Suburb/Town cannot contain state code",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000057" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI23", Value = GetValueOrEmpty(report.TRTAMI23) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI20", Value = GetValueOrEmpty(report.TRTAMI20) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000057

            #region VR.ATO.TRTAMI.000058

            /*  VR.ATO.TRTAMI.000058
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF [TRTAMI20] CONTAINS "UNKNOWN"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI20] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.LocalityName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI20, 'UNKNOWN')
    
            Data Elements:
    
            ^TRTAMI20 = RP.POS.Prev:AddressDetails.LocalityName.Text
            */
            assertion = (report.TRTAMI20 == null ? false : report.TRTAMI20.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000058",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Suburb/Town cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000058" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI20", Value = GetValueOrEmpty(report.TRTAMI20) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000058

            #region VR.ATO.TRTAMI.000059

            /*  VR.ATO.TRTAMI.000059
            Numerics [0 to 9] are not allowed in the suburb/town field.
    
            Legacy Rule Format:
            IF ([TRTAMI23] = NULL OR [TRTAMI23] = "au") AND ([TRTAMI20] CONTAINS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI20] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.LocalityName.Text
            [TRTAMI23] = TRTAMI:RP.POS.Prev:pyde.02.08:AddressDetails.Country.Code

            Technical Business Rule Format:
            (^TRTAMI23 = NULL OR ^TRTAMI23 = 'au') AND (ContainsSet(^TRTAMI20, '0-9'))
    
            Data Elements:
    
            ^TRTAMI20 = RP.POS.Prev:AddressDetails.LocalityName.Text
    
            ^TRTAMI23 = RP.POS.Prev:AddressDetails.Country.Code
            */
            assertion = ((report.TRTAMI23 == null || report.TRTAMI23 == @"au") && IsMatch(report.TRTAMI20, @"[0-9]", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000059",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Suburb/Town is invalid",
                    LongDescription = @"When an Australian address is provided, Previous Postal Address of Trust - Suburb/Town must not contain numeric characters",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000059" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI23", Value = GetValueOrEmpty(report.TRTAMI23) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI20", Value = GetValueOrEmpty(report.TRTAMI20) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000059

            #region VR.ATO.TRTAMI.000060

            /*  VR.ATO.TRTAMI.000060
            Value entered is not a valid code/value for that field.
    
            Legacy Rule Format:
            IF ([TRTAMI22] <> NULL) AND ([TRTAMI22]  <> SET(0200-9999))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI22] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Postcode.Text

            Technical Business Rule Format:
            (^TRTAMI22 <> BLANK) AND (IsNumeric(^TRTAMI22) = 'FALSE' OR AsNumeric(^TRTAMI22) < 200 OR AsNumeric(^TRTAMI22) > 9999)
    
            Data Elements:
    
            ^TRTAMI22 = RP.POS.Prev:AddressDetails.Postcode.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI22) != true && (IsNumeric(report.TRTAMI22) == false || AsNumeric(report.TRTAMI22) < 200 || AsNumeric(report.TRTAMI22) > 9999));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000060",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Postcode is invalid",
                    LongDescription = @"For an Australian address, Previous Postal Address of Trust - Postcode is invalid. Valid range is 0200 - 9999.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Postcode.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000060" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI22", Value = GetValueOrEmpty(report.TRTAMI22) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000060

            #region VR.ATO.TRTAMI.000061

            /*  VR.ATO.TRTAMI.000061
            State is present in the address field without a postcode.
    
            Legacy Rule Format:
            IF ([TRTAMI21] <> NULL) AND ([TRTAMI22] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI21] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.StateOrTerritory.Code
            [TRTAMI22] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Postcode.Text

            Technical Business Rule Format:
            (^TRTAMI21 <> NULL) AND (^TRTAMI22 = NULL)
    
            Data Elements:
    
            ^TRTAMI22 = RP.POS.Prev:AddressDetails.Postcode.Text
    
            ^TRTAMI21 = RP.POS.Prev:AddressDetails.StateOrTerritory.Code
            */
            assertion = (report.TRTAMI21 != null && report.TRTAMI22 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000061",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Postcode must be present",
                    LongDescription = @"If the Previous Postal Address of Trust is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Postcode.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000061" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI21", Value = GetValueOrEmpty(report.TRTAMI21) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI22", Value = GetValueOrEmpty(report.TRTAMI22) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000061

            #region VR.ATO.TRTAMI.000062

            /*  VR.ATO.TRTAMI.000062
            Postcode is present in the address field without a State code.
    
            Legacy Rule Format:
            IF ([TRTAMI22] <> NULL) AND ([TRTAMI21] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI21] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.StateOrTerritory.Code
            [TRTAMI22] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Postcode.Text

            Technical Business Rule Format:
            (^TRTAMI22 <> NULL) AND (^TRTAMI21 = NULL)
    
            Data Elements:
    
            ^TRTAMI21 = RP.POS.Prev:AddressDetails.StateOrTerritory.Code
    
            ^TRTAMI22 = RP.POS.Prev:AddressDetails.Postcode.Text
            */
            assertion = (report.TRTAMI22 != null && report.TRTAMI21 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000062",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - State must be present",
                    LongDescription = @"If the Previous Postal Address of Trust is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.StateOrTerritory.Code[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000062" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI22", Value = GetValueOrEmpty(report.TRTAMI22) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI21", Value = GetValueOrEmpty(report.TRTAMI21) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000062

            #region VR.ATO.TRTAMI.000063

            /*  VR.ATO.TRTAMI.000063
            Title is invalid
    
            Legacy Rule Format:
            IF ([TRTAMI24] <> NULLORBLANK) AND ([TRTAMI24] <> SET(DOMAIN(TITLE CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI24] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.Title.Text

            Technical Business Rule Format:
            (^TRTAMI24 <> BLANK) AND (NotInDomain(^TRTAMI24, 'TITLE CODES'))
    
            Data Elements:
    
            ^TRTAMI24 = RP.Trustee:PersonNameDetails.Title.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI24) != true && !(IsMatch(report.TRTAMI24, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000063",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Title is invalid",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.Title.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000063" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI24", Value = GetValueOrEmpty(report.TRTAMI24) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000063

            #region VR.ATO.TRTAMI.000064

            /*  VR.ATO.TRTAMI.000064
            Family name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF ([TRTAMI25] <> NULLORBLANK) AND ([TRTAMI25] DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSSAGE
            ENDIF
            
            [TRTAMI25] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.FamilyName.Text

            Technical Business Rule Format:
            (^TRTAMI25 <> BLANK) AND (NotContainsSet(^TRTAMI25, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^TRTAMI25 = RP.Trustee:PersonNameDetails.FamilyName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI25) != true && !(IsMatch(report.TRTAMI25, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000064",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Family Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000064" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI25", Value = GetValueOrEmpty(report.TRTAMI25) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000064

            #region VR.ATO.TRTAMI.000065

            /*  VR.ATO.TRTAMI.000065
            Family name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF ([TRTAMI25] = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI25] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.FamilyName.Text

            Technical Business Rule Format:
            (FoundSet(^TRTAMI25, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^TRTAMI25 = RP.Trustee:PersonNameDetails.FamilyName.Text
            */
            assertion = IsMatch(report.TRTAMI25, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000065",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Family Name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000065" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI25", Value = GetValueOrEmpty(report.TRTAMI25) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000065

            #region VR.ATO.TRTAMI.000066

            /*  VR.ATO.TRTAMI.000066
            Family name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF ([TRTAMI25] = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI25] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.FamilyName.Text

            Technical Business Rule Format:
            (FoundSet(^TRTAMI25, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^TRTAMI25 = RP.Trustee:PersonNameDetails.FamilyName.Text
            */
            assertion = IsMatch(report.TRTAMI25, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000066",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Family Name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000066" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI25", Value = GetValueOrEmpty(report.TRTAMI25) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000066

            #region VR.ATO.TRTAMI.000067

            /*  VR.ATO.TRTAMI.000067
            Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF ([TRTAMI25] = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI25] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.FamilyName.Text

            Technical Business Rule Format:
            (FoundSet(^TRTAMI25, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^TRTAMI25 = RP.Trustee:PersonNameDetails.FamilyName.Text
            */
            assertion = IsMatch(report.TRTAMI25, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000319",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Family Name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000067" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI25", Value = GetValueOrEmpty(report.TRTAMI25) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000067

            #region VR.ATO.TRTAMI.000068

            /*  VR.ATO.TRTAMI.000068
            Family Name cannot contain "space hyphen space"
    
            Legacy Rule Format:
            IF ([TRTAMI25] CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI25] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.FamilyName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI25, ' - ')
    
            Data Elements:
    
            ^TRTAMI25 = RP.Trustee:PersonNameDetails.FamilyName.Text
            */
            assertion = (report.TRTAMI25 == null ? false : report.TRTAMI25.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000068",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Family Name cannot contain ""space hyphen space""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000068" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI25", Value = GetValueOrEmpty(report.TRTAMI25) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000068

            #region VR.ATO.TRTAMI.000069

            /*  VR.ATO.TRTAMI.000069
            Family name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF ([TRTAMI25] CONTAINS SET("--","''","  "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI25] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.FamilyName.Text

            Technical Business Rule Format:
            (ContainsSet(^TRTAMI25, '"--","''","  "'))
    
            Data Elements:
    
            ^TRTAMI25 = RP.Trustee:PersonNameDetails.FamilyName.Text
            */
            assertion = IsMatch(report.TRTAMI25, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000069",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Family Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000069" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI25", Value = GetValueOrEmpty(report.TRTAMI25) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000069

            #region VR.ATO.TRTAMI.000070

            /*  VR.ATO.TRTAMI.000070
            Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF ([TRTAMI26] <> NULLORBLANK) AND ([TRTAMI26] DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSSAGE
            ENDIF
            
            [TRTAMI26] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.GivenName.Text

            Technical Business Rule Format:
            (^TRTAMI26 <> BLANK) AND (NotContainsSet(^TRTAMI26, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^TRTAMI26 = RP.Trustee:PersonNameDetails.GivenName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI26) != true && !(IsMatch(report.TRTAMI26, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000070",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee First Given Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000070" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI26", Value = GetValueOrEmpty(report.TRTAMI26) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000070

            #region VR.ATO.TRTAMI.000071

            /*  VR.ATO.TRTAMI.000071
            First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF ([TRTAMI26] = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI26] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.GivenName.Text

            Technical Business Rule Format:
            (FoundSet(^TRTAMI26, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^TRTAMI26 = RP.Trustee:PersonNameDetails.GivenName.Text
            */
            assertion = IsMatch(report.TRTAMI26, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000320",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee First Name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000071" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI26", Value = GetValueOrEmpty(report.TRTAMI26) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000071

            #region VR.ATO.TRTAMI.000072

            /*  VR.ATO.TRTAMI.000072
            First name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF ([TRTAMI26] = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI26] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.GivenName.Text

            Technical Business Rule Format:
            (FoundSet(^TRTAMI26, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^TRTAMI26 = RP.Trustee:PersonNameDetails.GivenName.Text
            */
            assertion = IsMatch(report.TRTAMI26, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000072",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee First Given Name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000072" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI26", Value = GetValueOrEmpty(report.TRTAMI26) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000072

            #region VR.ATO.TRTAMI.000073

            /*  VR.ATO.TRTAMI.000073
            First name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF ([TRTAMI26] = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI26] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.GivenName.Text

            Technical Business Rule Format:
            (FoundSet(^TRTAMI26, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^TRTAMI26 = RP.Trustee:PersonNameDetails.GivenName.Text
            */
            assertion = IsMatch(report.TRTAMI26, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000073",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee First Given Name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000073" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI26", Value = GetValueOrEmpty(report.TRTAMI26) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000073

            #region VR.ATO.TRTAMI.000074

            /*  VR.ATO.TRTAMI.000074
            FirstName must be entered if OtherGivenNames is entered
    
            Legacy Rule Format:
            IF ([TRTAMI27] <> NULLORBLANK) AND ([TRTAMI26] = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI26] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.GivenName.Text
            [TRTAMI27] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.OtherGivenName.Text

            Technical Business Rule Format:
            (^TRTAMI27 <> BLANK) AND (^TRTAMI26 = BLANK)
    
            Data Elements:
    
            ^TRTAMI26 = RP.Trustee:PersonNameDetails.GivenName.Text
    
            ^TRTAMI27 = RP.Trustee:PersonNameDetails.OtherGivenName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI27) != true && string.IsNullOrWhiteSpace(report.TRTAMI26) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000074",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee First Given Name must be entered if Trustee Other Given Names is entered",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000074" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI27", Value = GetValueOrEmpty(report.TRTAMI27) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI26", Value = GetValueOrEmpty(report.TRTAMI26) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000074

            #region VR.ATO.TRTAMI.000075

            /*  VR.ATO.TRTAMI.000075
            First name cannot contain space hyphen space
    
            Legacy Rule Format:
            IF ([TRTAMI26] CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI26] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.GivenName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI26, ' - ')
    
            Data Elements:
    
            ^TRTAMI26 = RP.Trustee:PersonNameDetails.GivenName.Text
            */
            assertion = (report.TRTAMI26 == null ? false : report.TRTAMI26.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000075",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee First Given Name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000075" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI26", Value = GetValueOrEmpty(report.TRTAMI26) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000075

            #region VR.ATO.TRTAMI.000076

            /*  VR.ATO.TRTAMI.000076
            First name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF ([TRTAMI26] CONTAINS SET("--","''","  "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI26] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.GivenName.Text

            Technical Business Rule Format:
            (ContainsSet(^TRTAMI26, '"--","''","  "'))
    
            Data Elements:
    
            ^TRTAMI26 = RP.Trustee:PersonNameDetails.GivenName.Text
            */
            assertion = IsMatch(report.TRTAMI26, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000076",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee First Given Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000076" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI26", Value = GetValueOrEmpty(report.TRTAMI26) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000076

            #region VR.ATO.TRTAMI.000077

            /*  VR.ATO.TRTAMI.000077
            Other Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF ([TRTAMI27] <> NULLORBLANK) AND ([TRTAMI27] DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI27] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.OtherGivenName.Text

            Technical Business Rule Format:
            (^TRTAMI27 <> BLANK) AND (NotContainsSet(^TRTAMI27, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^TRTAMI27 = RP.Trustee:PersonNameDetails.OtherGivenName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI27) != true && !(IsMatch(report.TRTAMI27, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000077",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Other Given Names must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000077" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI27", Value = GetValueOrEmpty(report.TRTAMI27) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000077

            #region VR.ATO.TRTAMI.000078

            /*  VR.ATO.TRTAMI.000078
            Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF ([TRTAMI27] = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI27] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.OtherGivenName.Text

            Technical Business Rule Format:
            (FoundSet(^TRTAMI27, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^TRTAMI27 = RP.Trustee:PersonNameDetails.OtherGivenName.Text
            */
            assertion = IsMatch(report.TRTAMI27, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000321",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Other Given Name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000078" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI27", Value = GetValueOrEmpty(report.TRTAMI27) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000078

            #region VR.ATO.TRTAMI.000079

            /*  VR.ATO.TRTAMI.000079
            Other given name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF ([TRTAMI27] = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI27] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.OtherGivenName.Text

            Technical Business Rule Format:
            (FoundSet(^TRTAMI27, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^TRTAMI27 = RP.Trustee:PersonNameDetails.OtherGivenName.Text
            */
            assertion = IsMatch(report.TRTAMI27, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000079",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Other Given Names cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000079" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI27", Value = GetValueOrEmpty(report.TRTAMI27) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000079

            #region VR.ATO.TRTAMI.000080

            /*  VR.ATO.TRTAMI.000080
            Other given name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF ([TRTAMI27] = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI27] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.OtherGivenName.Text

            Technical Business Rule Format:
            (FoundSet(^TRTAMI27, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^TRTAMI27 = RP.Trustee:PersonNameDetails.OtherGivenName.Text
            */
            assertion = IsMatch(report.TRTAMI27, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000080",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Other Given Names cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000080" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI27", Value = GetValueOrEmpty(report.TRTAMI27) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000080

            #region VR.ATO.TRTAMI.000081

            /*  VR.ATO.TRTAMI.000081
            Other given name cannot contain space hyphen space
    
            Legacy Rule Format:
            IF ([TRTAMI27] CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI27] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.OtherGivenName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI27, ' - ')
    
            Data Elements:
    
            ^TRTAMI27 = RP.Trustee:PersonNameDetails.OtherGivenName.Text
            */
            assertion = (report.TRTAMI27 == null ? false : report.TRTAMI27.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000081",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Other Given Names cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000081" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI27", Value = GetValueOrEmpty(report.TRTAMI27) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000081

            #region VR.ATO.TRTAMI.000082

            /*  VR.ATO.TRTAMI.000082
            Other given name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF ([TRTAMI27] CONTAINS SET("--","''","  "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI27] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.OtherGivenName.Text

            Technical Business Rule Format:
            (ContainsSet(^TRTAMI27, '"--","''","  "'))
    
            Data Elements:
    
            ^TRTAMI27 = RP.Trustee:PersonNameDetails.OtherGivenName.Text
            */
            assertion = IsMatch(report.TRTAMI27, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000082",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Other Given Names cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000082" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI27", Value = GetValueOrEmpty(report.TRTAMI27) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000082

            #region VR.ATO.TRTAMI.000083

            /*  VR.ATO.TRTAMI.000083
            Family name or non-individual name must be present.
    
            Legacy Rule Format:
            IF [TRTAMI25] = NULLORBLANK AND [TRTAMI28] = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI25] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.FamilyName.Text
            [TRTAMI28] = TRTAMI:RP.Trustee:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            ^TRTAMI25 = BLANK AND ^TRTAMI28 = BLANK
    
            Data Elements:
    
            ^TRTAMI28 = RP.Trustee:OrganisationNameDetails.OrganisationalName.Text
    
            ^TRTAMI25 = RP.Trustee:PersonNameDetails.FamilyName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI25) == true && string.IsNullOrWhiteSpace(report.TRTAMI28) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000083",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of the trustee to whom notices should be sent must be present",
                    LongDescription = @"If the trustee is an individual, provide the name of the individual. If the trustee is a company, provide the name of the company.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000083" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI25", Value = GetValueOrEmpty(report.TRTAMI25) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI28", Value = GetValueOrEmpty(report.TRTAMI28) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000083

            #region VR.ATO.TRTAMI.000084

            /*  VR.ATO.TRTAMI.000084
            Non-individual name and Family name details are both present. Only one name can be present.
    
            Legacy Rule Format:
            IF ([TRTAMI28] <> NULLORBLANK OR [TRTAMI29] <> NULLORBLANK) AND ([TRTAMI24] <> NULL OR [TRTAMI25] <> NULLORBLANK OR [TRTAMI26] <> NULLORBLANK OR [TRTAMI27] <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI24] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.Title.Text
            [TRTAMI25] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.FamilyName.Text
            [TRTAMI26] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.GivenName.Text
            [TRTAMI27] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.OtherGivenName.Text
            [TRTAMI28] = TRTAMI:RP.Trustee:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
            [TRTAMI29] = TRTAMI:RP.Trustee:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier

            Technical Business Rule Format:
            (^TRTAMI28 <> BLANK OR ^TRTAMI29 <> BLANK) AND (^TRTAMI24 <> NULL OR ^TRTAMI25 <> BLANK OR ^TRTAMI26 <> BLANK OR ^TRTAMI27 <> BLANK)
    
            Data Elements:
    
            ^TRTAMI28 = RP.Trustee:OrganisationNameDetails.OrganisationalName.Text
    
            ^TRTAMI24 = RP.Trustee:PersonNameDetails.Title.Text
    
            ^TRTAMI25 = RP.Trustee:PersonNameDetails.FamilyName.Text
    
            ^TRTAMI26 = RP.Trustee:PersonNameDetails.GivenName.Text
    
            ^TRTAMI27 = RP.Trustee:PersonNameDetails.OtherGivenName.Text
    
            ^TRTAMI29 = RP.Trustee:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = ((string.IsNullOrWhiteSpace(report.TRTAMI28) != true || string.IsNullOrWhiteSpace(report.TRTAMI29) != true) && (report.TRTAMI24 != null || string.IsNullOrWhiteSpace(report.TRTAMI25) != true || string.IsNullOrWhiteSpace(report.TRTAMI26) != true || string.IsNullOrWhiteSpace(report.TRTAMI27) != true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000084",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Non-Individual Name and Trustee Family Name details must not both be present",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000084" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI28", Value = GetValueOrEmpty(report.TRTAMI28) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI29", Value = GetValueOrEmpty(report.TRTAMI29) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI24", Value = GetValueOrEmpty(report.TRTAMI24) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI25", Value = GetValueOrEmpty(report.TRTAMI25) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI26", Value = GetValueOrEmpty(report.TRTAMI26) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI27", Value = GetValueOrEmpty(report.TRTAMI27) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000084

            #region VR.ATO.TRTAMI.000085

            /*  VR.ATO.TRTAMI.000085
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF ([TRTAMI28] <> NULLORBLANK) AND ([TRTAMI28] DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI28] = TRTAMI:RP.Trustee:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (^TRTAMI28 <> BLANK) AND (NotContainsSet(^TRTAMI28, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^TRTAMI28 = RP.Trustee:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI28) != true && !(IsMatch(report.TRTAMI28, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000085",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Non-Individual Name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000085" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI28", Value = GetValueOrEmpty(report.TRTAMI28) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000085

            #region VR.ATO.TRTAMI.000086

            /*  VR.ATO.TRTAMI.000086
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF ([TRTAMI28] CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI28] = TRTAMI:RP.Trustee:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI28, ' - ')
    
            Data Elements:
    
            ^TRTAMI28 = RP.Trustee:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (report.TRTAMI28 == null ? false : report.TRTAMI28.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Non-Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000086" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI28", Value = GetValueOrEmpty(report.TRTAMI28) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000086

            #region VR.ATO.TRTAMI.000087

            /*  VR.ATO.TRTAMI.000087
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF ([TRTAMI28] CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI28] = TRTAMI:RP.Trustee:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            Contains(^TRTAMI28, 'P/L')
    
            Data Elements:
    
            ^TRTAMI28 = RP.Trustee:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (report.TRTAMI28 == null ? false : report.TRTAMI28.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000087",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Non-Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000087" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI28", Value = GetValueOrEmpty(report.TRTAMI28) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000087

            #region VR.ATO.TRTAMI.000088

            /*  VR.ATO.TRTAMI.000088
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF ([TRTAMI28] CONTAINS SET("--","''","  "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI28] = TRTAMI:RP.Trustee:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (ContainsSet(^TRTAMI28, '"--","''","  "'))
    
            Data Elements:
    
            ^TRTAMI28 = RP.Trustee:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = IsMatch(report.TRTAMI28, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000088",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Non-Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000088" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI28", Value = GetValueOrEmpty(report.TRTAMI28) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000088

            #region VR.ATO.TRTAMI.000089

            /*  VR.ATO.TRTAMI.000089
            Non Individual Name cannot start with "T/A" and end with "Pship", "P'ship" or "P/Ship"
    
            Legacy Rule Format:
            IF ([TRTAMI28] STARTSWITH "T/A ") AND ([TRTAMI28] ENDSWITH SET(" Pship"," P'ship"," P/ship"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI28] = TRTAMI:RP.Trustee:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (StartsWith(^TRTAMI28, 'T/A ')) AND (EndsWithSet(^TRTAMI28, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^TRTAMI28 = RP.Trustee:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (IsMatch(report.TRTAMI28, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(report.TRTAMI28, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000089",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Non-Individual Name cannot start with ""T/A"" and end with ""Pship"", ""P'ship"" or ""P/Ship""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000089" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI28", Value = GetValueOrEmpty(report.TRTAMI28) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000089

            #region VR.ATO.TRTAMI.000091

            /*  VR.ATO.TRTAMI.000091
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF ([TRTAMI28] ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI28] = TRTAMI:RP.Trustee:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            (EndsWithSet(^TRTAMI28, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            ^TRTAMI28 = RP.Trustee:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = IsMatch(report.TRTAMI28, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000091",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Non-Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000091" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI28", Value = GetValueOrEmpty(report.TRTAMI28) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000091

            #region VR.ATO.TRTAMI.000092

            /*  VR.ATO.TRTAMI.000092
            Is any tax payable by the Trustee? must be present
    
            Legacy Rule Format:
            IF [TRTAMI33] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI33] = TRTAMI:RP:pyin.02.00:Report.Statement.Summary.PaymentRefundOrNil.Code

            Technical Business Rule Format:
            ^TRTAMI33 = NULL
    
            Data Elements:
    
            ^TRTAMI33 = RP:Report.Statement.Summary.PaymentRefundOrNil.Code
            */
            assertion = (report.TRTAMI33 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000092",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Is any tax payable by the Trustee?' must be provided",
                    Location = "/xbrli:xbrl/tns:Report.Statement.Summary.PaymentRefundOrNil.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000092" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI33", Value = report.TRTAMI33 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000092

            #region VR.ATO.TRTAMI.000093

            /*  VR.ATO.TRTAMI.000093
            Final Tax Return must be present
    
            Legacy Rule Format:
            IF [TRTAMI34] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI34] = TRTAMI:RP:pyin.02.28:Lodgment.FinalReturn.Indicator

            Technical Business Rule Format:
            ^TRTAMI34 = NULL
    
            Data Elements:
    
            ^TRTAMI34 = RP:Lodgment.FinalReturn.Indicator
            */
            assertion = (report.TRTAMI34 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000093",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Final Tax Return must be provided",
                    Location = "/xbrli:xbrl/tns:Lodgment.FinalReturn.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000093" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI34", Value = GetValueOrEmpty(report.TRTAMI34) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000093

            #region VR.ATO.TRTAMI.000094

            /*  VR.ATO.TRTAMI.000094
            Declarer Telephone number has a maximum field length of 15
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI97]) > 15
                 RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI97] = TRTAMI:RP.TrueAndCorrect:pyde.02.00:ElectronicContact.Telephone.Minimal.Number

            Technical Business Rule Format:
            Length(^TRTAMI97) > 15
    
            Data Elements:
    
            ^TRTAMI97 = RP.TrueAndCorrect:ElectronicContact.Telephone.Minimal.Number
            */
            assertion = (Length(report.TRTAMI97) > 15);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000094",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declarer Telephone number must not exceed 15 characters",
                    LongDescription = @"For Declarer Telephone number, the maximum allowable characters for this form is 15",
                    Location = "/xbrli:xbrl/tns:ElectronicContact.Telephone.Minimal.Number[@contextRef='" + report.RPTrueAndCorrectId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000094" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI97", Value = report.TRTAMI97 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000094

            #region VR.ATO.TRTAMI.000095

            /*  VR.ATO.TRTAMI.000095
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI35] <> NULL AND [TRTAMI35] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI35] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustComponentDeficitOffset.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI35, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI35 = RP:IncomeTax.TrustComponentDeficitOffset.Amount
            */
            assertion = NotMonetary(report.TRTAMI35, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000095",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trust Component Deficit of Character Relating to Tax Offset is not in a valid monetary format",
                    LongDescription = @"Trust Component Deficit of Character Relating to Tax Offset must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TrustComponentDeficitOffset.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000095" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI35", Value = GetValueOrEmpty(report.TRTAMI35) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000095

            #region VR.ATO.TRTAMI.000096

            /*  VR.ATO.TRTAMI.000096
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI36] <> NULL AND [TRTAMI36] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI36] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustShortfallIncome.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI36, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI36 = RP:IncomeTax.TrustShortfallIncome.Amount
            */
            assertion = NotMonetary(report.TRTAMI36, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000096",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Shortfall in Determined Member Components of Character Relating to Assessable Income is not in a valid monetary format",
                    LongDescription = @"Shortfall in Determined Member Components of Character Relating to Assessable Income must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TrustShortfallIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000096" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI36", Value = GetValueOrEmpty(report.TRTAMI36) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000096

            #region VR.ATO.TRTAMI.000099

            /*  VR.ATO.TRTAMI.000099
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI39] <> NULL AND [TRTAMI39] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI39] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustExcessOffset.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI39, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI39 = RP:IncomeTax.TrustExcessOffset.Amount
            */
            assertion = NotMonetary(report.TRTAMI39, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000099",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Excess in Determined Member Components of Character Relating to Tax Offset is not in a valid monetary format",
                    LongDescription = @"Excess in Determined Member Components of Character Relating to Tax Offset must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TrustExcessOffset.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000099" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI39", Value = GetValueOrEmpty(report.TRTAMI39) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000099

            #region VR.ATO.TRTAMI.000100

            /*  VR.ATO.TRTAMI.000100
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI40] <> NULL AND [TRTAMI40] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI40] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustComponentNotReflected.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI40, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI40 = RP:IncomeTax.TrustComponentNotReflected.Amount
            */
            assertion = NotMonetary(report.TRTAMI40, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000100",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Determined Trust Component Amounts that are not reflected in Member Components is not in a valid monetary format",
                    LongDescription = @"Determined Trust Component Amounts that are not reflected in Member Components must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TrustComponentNotReflected.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000100" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI40", Value = GetValueOrEmpty(report.TRTAMI40) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000100

            #region VR.ATO.TRTAMI.000101

            /*  VR.ATO.TRTAMI.000101
            A text character must be one of the following: A to Z a to z 0 to 9 & * ( ) - ' . / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI43] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI43] <> SET("a-z","A-Z","0-9","&","*","(",")","-","'",".","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI43] = TRTAMI:RP:pyid.02.00:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text

            Technical Business Rule Format:
            (^TRTAMI43 <> BLANK) AND (NotCharacterInSet(^TRTAMI43, '"a-z","A-Z","0-9","&","*","(",")","-","'",".","/"," "'))
    
            Data Elements:
    
            ^TRTAMI43 = RP:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI43) != true && !(IsMatch(report.TRTAMI43, @"^[a-zA-Z0-9&\*\(\)\-'\./ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000101",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Account name contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 & * ( ) - ' . / or a space character",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000101" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI43", Value = GetValueOrEmpty(report.TRTAMI43) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000101

            #region VR.ATO.TRTAMI.000102

            /*  VR.ATO.TRTAMI.000102
            When Industry Code has been provided, Description of Main Business or Professional Activity must be present
    
            Legacy Rule Format:
            IF ([TRTAMI45] <> NULL) AND ([TRTAMI44] = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI44] = TRTAMI:RP:pyde.02.05:OrganisationDetails.MainIncomeActivity.Description
            [TRTAMI45] = TRTAMI:RP:pyde.02.48:OrganisationDetails.OrganisationIndustry2006Extended.Code

            Technical Business Rule Format:
            (^TRTAMI45 <> NULL) AND (^TRTAMI44 = BLANK)
    
            Data Elements:
    
            ^TRTAMI44 = RP:OrganisationDetails.MainIncomeActivity.Description
    
            ^TRTAMI45 = RP:OrganisationDetails.OrganisationIndustry2006Extended.Code
            */
            assertion = (report.TRTAMI45 != null && string.IsNullOrWhiteSpace(report.TRTAMI44) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000102",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Description of Main Business or Professional Activity must be present",
                    LongDescription = @"When Industry Code has been provided, Description of Main Business or Professional Activity must be present",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000102" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI45", Value = GetValueOrEmpty(report.TRTAMI45) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI44", Value = GetValueOrEmpty(report.TRTAMI44) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000102

            #region VR.ATO.TRTAMI.000103

            /*  VR.ATO.TRTAMI.000103
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI44] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI44] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI44] = TRTAMI:RP:pyde.02.05:OrganisationDetails.MainIncomeActivity.Description

            Technical Business Rule Format:
            (^TRTAMI44 <> BLANK) AND (NotCharacterInSet(^TRTAMI44, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI44 = RP:OrganisationDetails.MainIncomeActivity.Description
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI44) != true && !(IsMatch(report.TRTAMI44, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000103",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Description of Main Business or Professional Activity contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000103" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI44", Value = GetValueOrEmpty(report.TRTAMI44) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000103

            #region VR.ATO.TRTAMI.000104

            /*  VR.ATO.TRTAMI.000104
            Credit: Interest on early payments must not be less than 50 cents
    
            Legacy Rule Format:
            IF ([TRTAMI46] <> NULL) AND ([TRTAMI46] < 0.50)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI46] = TRTAMI:RP:bafpr2.02.09:Income.Interest.EarlyPaymentCredit.Amount

            Technical Business Rule Format:
            (^TRTAMI46 <> NULL) AND (^TRTAMI46 < 0.50)
    
            Data Elements:
    
            ^TRTAMI46 = RP:Income.Interest.EarlyPaymentCredit.Amount
            */
            assertion = (report.TRTAMI46 != null && report.TRTAMI46.GetValueOrDefault() < 0.50M);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434079",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit: Interest on early payments must not be less than 50 cents",
                    Location = "/xbrli:xbrl/tns:Income.Interest.EarlyPaymentCredit.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000104" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI46", Value = GetValueOrEmpty(report.TRTAMI46) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000104

            #region VR.ATO.TRTAMI.000105

            /*  VR.ATO.TRTAMI.000105
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRTAMI46] <> NULL AND [TRTAMI46] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI46] = TRTAMI:RP:bafpr2.02.09:Income.Interest.EarlyPaymentCredit.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI46, 'U', 13, 2)
    
            Data Elements:
    
            ^TRTAMI46 = RP:Income.Interest.EarlyPaymentCredit.Amount
            */
            assertion = NotMonetary(report.TRTAMI46, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000105",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit for Interest on Early Payments - Amount of Interest is not in a valid monetary format",
                    LongDescription = @"Credit for Interest on Early Payments - Amount of Interest must be unsigned, contain 2 decimals and not exceed 11 significant digits; no other characters are allowed except for the decimal point",
                    Location = "/xbrli:xbrl/tns:Income.Interest.EarlyPaymentCredit.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000105" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI46", Value = GetValueOrEmpty(report.TRTAMI46) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000105

            #region VR.ATO.TRTAMI.000106

            /*  VR.ATO.TRTAMI.000106
            Was the aggregate amount of your transactions or dealings with international related parties (including the value of any property or service transferred or the balance of any loans) greater than $2 million? must be present
    
            Legacy Rule Format:
            IF [TRTAMI47] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI47] = TRTAMI:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator

            Technical Business Rule Format:
            ^TRTAMI47 = NULL
    
            Data Elements:
    
            ^TRTAMI47 = RP:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
            */
            assertion = (report.TRTAMI47 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000106",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Was the aggregate amount of your transactions or dealings with international related parties...' must be provided",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000106" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI47", Value = GetValueOrEmpty(report.TRTAMI47) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000106

            #region VR.ATO.TRTAMI.000107

            /*  VR.ATO.TRTAMI.000107
            Did the Thin Capitalisation provisions affect you? must be present
    
            Legacy Rule Format:
            IF [TRTAMI48] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI48] = TRTAMI:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator

            Technical Business Rule Format:
            ^TRTAMI48 = NULL
    
            Data Elements:
    
            ^TRTAMI48 = RP:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
            */
            assertion = (report.TRTAMI48 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000107",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Did the Thin Capitalisation provisions affect you? must be provided",
                    Location = "/xbrli:xbrl/tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000107" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI48", Value = GetValueOrEmpty(report.TRTAMI48) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000107

            #region VR.ATO.TRTAMI.000108

            /*  VR.ATO.TRTAMI.000108
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI49] <> NULL AND [TRTAMI49] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI49] = TRTAMI:RP:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI49, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI49 = RP:Expense.Interest.Amount
            */
            assertion = NotMonetary(report.TRTAMI49, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000108",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Interest Expenses Overseas is not in a valid monetary format",
                    LongDescription = @"Interest Expenses Overseas must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Expense.Interest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000108" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI49", Value = GetValueOrEmpty(report.TRTAMI49) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000108

            #region VR.ATO.TRTAMI.000109

            /*  VR.ATO.TRTAMI.000109
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI50] <> NULL AND [TRTAMI50] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI50] = TRTAMI:RP:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI50, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI50 = RP:Expense.Royalties.Amount
            */
            assertion = NotMonetary(report.TRTAMI50, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000109",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalty Expenses Overseas is not in a valid monetary format",
                    LongDescription = @"Royalty Expenses Overseas must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000109" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI50", Value = GetValueOrEmpty(report.TRTAMI50) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000109

            #region VR.ATO.TRTAMI.000113

            /*  VR.ATO.TRTAMI.000113
            All current assets is greater than total assets.
    
            Legacy Rule Format:
            IF [TRTAMI57] <> NULL AND  [TRTAMI58] < [TRTAMI57]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI57] = TRTAMI:RP(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount
            [TRTAMI58] = TRTAMI:RP(Instant):bafpo1.02.13:Assets.Total.Amount

            Technical Business Rule Format:
            ^TRTAMI57 <> NULL AND  ^TRTAMI58 < ^TRTAMI57
    
            Data Elements:
    
            ^TRTAMI57 = RP(Instant):Assets.CurrentTotal.Amount
    
            ^TRTAMI58 = RP(Instant):Assets.Total.Amount
            */
            assertion = (report.TRTAMI57 != null && report.TRTAMI58.GetValueOrDefault() < report.TRTAMI57.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430194",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"All current assets must not exceed Total assets",
                    Location = "/xbrli:xbrl/tns:Assets.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000113" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI57", Value = GetValueOrEmpty(report.TRTAMI57) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI58", Value = GetValueOrEmpty(report.TRTAMI58) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000113

            #region VR.ATO.TRTAMI.000114

            /*  VR.ATO.TRTAMI.000114
            Monetary field, must be unsigned, maximum of 15 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI57] <> NULL AND [TRTAMI57] <> MONETARY(U,15,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI57] = TRTAMI:RP(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI57, 'U', 15, 0)
    
            Data Elements:
    
            ^TRTAMI57 = RP(Instant):Assets.CurrentTotal.Amount
            */
            assertion = NotMonetary(report.TRTAMI57, @"U", 15, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000114",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"All Current Assets is not in a valid monetary format",
                    LongDescription = @"All Current Assets must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Assets.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000114" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI57", Value = GetValueOrEmpty(report.TRTAMI57) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000114

            #region VR.ATO.TRTAMI.000115

            /*  VR.ATO.TRTAMI.000115
            Monetary field, must be unsigned, maximum of 15 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI58] <> NULL AND [TRTAMI58] <> MONETARY(U,15,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI58] = TRTAMI:RP(Instant):bafpo1.02.13:Assets.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI58, 'U', 15, 0)
    
            Data Elements:
    
            ^TRTAMI58 = RP(Instant):Assets.Total.Amount
            */
            assertion = NotMonetary(report.TRTAMI58, @"U", 15, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000115",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Assets is not in a valid monetary format",
                    LongDescription = @"Total Assets must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Assets.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000115" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI58", Value = GetValueOrEmpty(report.TRTAMI58) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000115

            #region VR.ATO.TRTAMI.000116

            /*  VR.ATO.TRTAMI.000116
            All current liabilities is greater than total liabilities.
    
            Legacy Rule Format:
            IF [TRTAMI59] <> NULL AND [TRTAMI60] < [TRTAMI59]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI59] = TRTAMI:RP(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount
            [TRTAMI60] = TRTAMI:RP(Instant):bafpo1.02.13:Liabilities.Total.Amount

            Technical Business Rule Format:
            ^TRTAMI59 <> NULL AND ^TRTAMI60 < ^TRTAMI59
    
            Data Elements:
    
            ^TRTAMI59 = RP(Instant):Liabilities.CurrentTotal.Amount
    
            ^TRTAMI60 = RP(Instant):Liabilities.Total.Amount
            */
            assertion = (report.TRTAMI59 != null && report.TRTAMI60.GetValueOrDefault() < report.TRTAMI59.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430195",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"All current liabilities must not exceed Total liabilities",
                    Location = "/xbrli:xbrl/tns:Liabilities.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000116" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI59", Value = GetValueOrEmpty(report.TRTAMI59) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI60", Value = GetValueOrEmpty(report.TRTAMI60) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000116

            #region VR.ATO.TRTAMI.000117

            /*  VR.ATO.TRTAMI.000117
            Monetary field, must be unsigned, maximum of 15 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI59] <> NULL AND [TRTAMI59] <> MONETARY(U,15,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI59] = TRTAMI:RP(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI59, 'U', 15, 0)
    
            Data Elements:
    
            ^TRTAMI59 = RP(Instant):Liabilities.CurrentTotal.Amount
            */
            assertion = NotMonetary(report.TRTAMI59, @"U", 15, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000117",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"All Current Liabilities is not in a valid monetary format",
                    LongDescription = @"All Current Liabilities must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Liabilities.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000117" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI59", Value = GetValueOrEmpty(report.TRTAMI59) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000117

            #region VR.ATO.TRTAMI.000118

            /*  VR.ATO.TRTAMI.000118
            Monetary field, must be unsigned, maximum of 15 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI60] <> NULL AND [TRTAMI60] <> MONETARY(U,15,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI60] = TRTAMI:RP(Instant):bafpo1.02.13:Liabilities.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI60, 'U', 15, 0)
    
            Data Elements:
    
            ^TRTAMI60 = RP(Instant):Liabilities.Total.Amount
            */
            assertion = NotMonetary(report.TRTAMI60, @"U", 15, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000118",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Liabilities is not in a valid monetary format",
                    LongDescription = @"Total Liabilities must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Liabilities.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000118" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI60", Value = GetValueOrEmpty(report.TRTAMI60) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000118

            #region VR.ATO.TRTAMI.000150

            /*  VR.ATO.TRTAMI.000150
            Tax File Number has failed the algorithm check.
    
            Legacy Rule Format:
            IF (RP:entity.identifier.TFN <> NULLORBLANK) AND (TFNALGORITHM (RP:entity.identifier.TFN) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TFN <> BLANK) AND (FailsTFNAlgorithm(^TFN))
    
            Data Elements:
    
            ^TFN = RP
            */
            assertion = (string.IsNullOrWhiteSpace(report.RPIdentifierTFN) != true && FailsTFNAlgorithm(report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000150",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trust Tax File Number has failed the algorithm check",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000150" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000150

            #region VR.ATO.TRTAMI.000162

            /*  VR.ATO.TRTAMI.000162
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI71] <> NULL AND [TRTAMI71] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI71] = TRTAMI:RP:bafpr1.02.02:Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI71, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI71 = RP:Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount
            */
            assertion = NotMonetary(report.TRTAMI71, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000162",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Section 40-880 Deductions is not in a valid monetary format",
                    LongDescription = @"Total Section 40-880 Deductions must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000162" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI71", Value = GetValueOrEmpty(report.TRTAMI71) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000162

            #region VR.ATO.TRTAMI.000164

            /*  VR.ATO.TRTAMI.000164
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRTAMI73] <> NULL AND [TRTAMI73] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI73] = TRTAMI:RP:bafpr1.02.10:Income.TrustEstateIncome.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI73, 'S', 11, 0)
    
            Data Elements:
    
            ^TRTAMI73 = RP:Income.TrustEstateIncome.Amount
            */
            assertion = NotMonetary(report.TRTAMI73, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000164",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Accounting Profit/(Loss) of AMIT is not in a valid monetary format",
                    LongDescription = @"Total Accounting Profit/(Loss) of Attribution Managed Investment Trust (AMIT) may be signed but must not exceed 11 digits; no other characters are allowed except for the preceding sign",
                    Location = "/xbrli:xbrl/tns:Income.TrustEstateIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000164" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI73", Value = GetValueOrEmpty(report.TRTAMI73) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000164

            #region VR.ATO.TRTAMI.000165

            /*  VR.ATO.TRTAMI.000165
            Total Accounting Profit/(Loss) of Attribution Managed Investment Trust (AMIT) must be present
    
            Legacy Rule Format:
            IF [TRTAMI73] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI73] = TRTAMI:RP:bafpr1.02.10:Income.TrustEstateIncome.Amount

            Technical Business Rule Format:
            ^TRTAMI73 = NULL
    
            Data Elements:
    
            ^TRTAMI73 = RP:Income.TrustEstateIncome.Amount
            */
            assertion = (report.TRTAMI73 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000165",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Accounting Profit/(Loss) of Attribution Managed Investment Trust (AMIT) must be provided",
                    Location = "/xbrli:xbrl/tns:Income.TrustEstateIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000165" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI73", Value = GetValueOrEmpty(report.TRTAMI73) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000165

            #region VR.ATO.TRTAMI.000167

            /*  VR.ATO.TRTAMI.000167
            The TFN provided for the Trust member must not be the same as the TFN of the Trust.
    
            Legacy Rule Format:
            IF ANY OCCURRENCE OF([TRTAMI75]) = RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI75] = TRTAMI:RP.{SoASeqNum}:pyid.02.00:Identifiers.TaxFileNumber.Identifier

            Technical Business Rule Format:
            AnyOccurrence(^TRTAMI75, ^TRTAMI75 = ^TFN)
    
            Data Elements:
    
            ^TRTAMI75 = RP.{SoASeqNum}:Identifiers.TaxFileNumber.Identifier
    
            ^TFN = RP
            */
            assertion = (report.RPSoASeqNumContextCollection == null ? false : report.RPSoASeqNumContextCollection.Any(TRTAMI75Repeat => TRTAMI75Repeat.TRTAMI75 == report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000167",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Member Tax File Number must not be the same as the Trust Tax File Number",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000167" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI75", Value = "TRTAMI75" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000167

            #region VR.ATO.TRTAMI.000173

            /*  VR.ATO.TRTAMI.000173
            Trust Australian Business Number is mandatory
    
            Legacy Rule Format:
            IF [TRTAMI5] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI5] = TRTAMI:RP:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier

            Technical Business Rule Format:
            ^TRTAMI5 = NULL
    
            Data Elements:
    
            ^TRTAMI5 = RP:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (report.TRTAMI5 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000173",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trust Australian Business Number must be present",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000173" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI5", Value = report.TRTAMI5 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000173

            #region VR.ATO.TRTAMI.000176

            /*  VR.ATO.TRTAMI.000176
            Date Financial Year From must not be earlier than July 2015 as AMIT legislation only came into effect from 1 July 2015
    
            Legacy Rule Format:
            IF RP:period.startDate < "2015-07-01"
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate1 <  AsDate('2015-07-01')
    
            Data Elements:
    
            ^StartDate1 = RP
            */
            assertion = ((report.RPExists == true) && (report.RPStartDate < AsDate(@"2015-07-01")));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000176",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Date Financial Year From must not be earlier than July 2015",
                    LongDescription = @"Date Financial Year From must not be earlier than July 2015 as AMIT legislation only came into effect from 1 July 2015",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000176" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000176

            #region VR.ATO.TRTAMI.000178

            /*  VR.ATO.TRTAMI.000178
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character
    
            Legacy Rule Format:
            IF ([TRTAMI8] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI8] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI8] = TRTAMI:RP:pyin.02.08:Report.AmendmentReason.Text

            Technical Business Rule Format:
            (^TRTAMI8 <> BLANK) AND (NotCharacterInSet(^TRTAMI8, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI8 = RP:Report.AmendmentReason.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI8) != true && !(IsMatch(report.TRTAMI8, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000178",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Amendment Reason contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character",
                    Location = "/xbrli:xbrl/tns:Report.AmendmentReason.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000178" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI8", Value = GetValueOrEmpty(report.TRTAMI8) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000178

            #region VR.ATO.TRTAMI.000179

            /*  VR.ATO.TRTAMI.000179
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ # $ % & * ( ) - _ = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI10] <> NULLORBLANK) AND ([TRTAMI10] CONTAINS SET("{","}","|","\"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI10] = TRTAMI:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            ContainsSet(^TRTAMI10, '"{","}","|","\"')
    
            Data Elements:
    
            ^TRTAMI10 = RP:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = IsMatch(report.TRTAMI10, @"\{|\}|\||\\", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000179",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of Trust contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000179" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI10", Value = report.TRTAMI10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000179

            #region VR.ATO.TRTAMI.000180

            /*  VR.ATO.TRTAMI.000180
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ # $ % & * ( ) - _ = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI11] <> NULLORBLANK) AND ([TRTAMI11] CONTAINS SET("{","}","|","\"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI11] = TRTAMI:RP.Prev:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            ContainsSet(^TRTAMI11, '"{","}","|","\"')
    
            Data Elements:
    
            ^TRTAMI11 = RP.Prev:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = IsMatch(report.TRTAMI11, @"\{|\}|\||\\", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000180",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Name of Trust contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000180" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI11", Value = GetValueOrEmpty(report.TRTAMI11) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000180

            #region VR.ATO.TRTAMI.000181

            /*  VR.ATO.TRTAMI.000181
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI12] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI12] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI12] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            (^TRTAMI12 <> BLANK) AND (NotCharacterInSet(^TRTAMI12, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI12 = RP.POS:AddressDetails.Line1.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI12) != true && !(IsMatch(report.TRTAMI12, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000181",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 1 contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000181" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI12", Value = report.TRTAMI12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000181

            #region VR.ATO.TRTAMI.000182

            /*  VR.ATO.TRTAMI.000182
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI13] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI13] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI13] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            (^TRTAMI13 <> BLANK) AND (NotCharacterInSet(^TRTAMI13, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI13 = RP.POS:AddressDetails.Line2.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI13) != true && !(IsMatch(report.TRTAMI13, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000182",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Address Line 2 contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000182" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI13", Value = GetValueOrEmpty(report.TRTAMI13) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000182

            #region VR.ATO.TRTAMI.000183

            /*  VR.ATO.TRTAMI.000183
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI18] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI18] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI18] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line1.Text

            Technical Business Rule Format:
            (^TRTAMI18 <> BLANK) AND (NotCharacterInSet(^TRTAMI18, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI18 = RP.POS.Prev:AddressDetails.Line1.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI18) != true && !(IsMatch(report.TRTAMI18, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000183",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Address Line 1 contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000183" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI18", Value = GetValueOrEmpty(report.TRTAMI18) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000183

            #region VR.ATO.TRTAMI.000184

            /*  VR.ATO.TRTAMI.000184
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI19] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI19] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI19] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.Line2.Text

            Technical Business Rule Format:
            (^TRTAMI19 <> BLANK) AND (NotCharacterInSet(^TRTAMI19, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI19 = RP.POS.Prev:AddressDetails.Line2.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI19) != true && !(IsMatch(report.TRTAMI19, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000184",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Address Line 2 contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000184" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI19", Value = GetValueOrEmpty(report.TRTAMI19) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000184

            #region VR.ATO.TRTAMI.000185

            /*  VR.ATO.TRTAMI.000185
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI25] <> NULLORBLANK) AND ([TRTAMI25] CONTAINS SET("{","}","|","_","\","#"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI25] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.FamilyName.Text

            Technical Business Rule Format:
            ContainsSet(^TRTAMI25, '"{","}","|","_","\","#"')
    
            Data Elements:
    
            ^TRTAMI25 = RP.Trustee:PersonNameDetails.FamilyName.Text
            */
            assertion = IsMatch(report.TRTAMI25, @"\{|\}|\||_|\\|#", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000185",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Family Name contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000185" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI25", Value = GetValueOrEmpty(report.TRTAMI25) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000185

            #region VR.ATO.TRTAMI.000186

            /*  VR.ATO.TRTAMI.000186
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI26] <> NULLORBLANK) AND ([TRTAMI26] CONTAINS SET("{","}","|","_","\","#"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI26] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.GivenName.Text

            Technical Business Rule Format:
            ContainsSet(^TRTAMI26, '"{","}","|","_","\","#"')
    
            Data Elements:
    
            ^TRTAMI26 = RP.Trustee:PersonNameDetails.GivenName.Text
            */
            assertion = IsMatch(report.TRTAMI26, @"\{|\}|\||_|\\|#", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000186",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee First Given Name contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000186" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI26", Value = GetValueOrEmpty(report.TRTAMI26) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000186

            #region VR.ATO.TRTAMI.000187

            /*  VR.ATO.TRTAMI.000187
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI27] <> NULLORBLANK) AND ([TRTAMI27] CONTAINS SET("{","}","|","_","\","#"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI27] = TRTAMI:RP.Trustee:pyde.02.00:PersonNameDetails.OtherGivenName.Text

            Technical Business Rule Format:
            ContainsSet(^TRTAMI27, '"{","}","|","_","\","#"')
    
            Data Elements:
    
            ^TRTAMI27 = RP.Trustee:PersonNameDetails.OtherGivenName.Text
            */
            assertion = IsMatch(report.TRTAMI27, @"\{|\}|\||_|\\|#", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000187",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Other Given Names contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000187" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI27", Value = GetValueOrEmpty(report.TRTAMI27) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000187

            #region VR.ATO.TRTAMI.000188

            /*  VR.ATO.TRTAMI.000188
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ # $ % & * ( ) - _ = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI28] <> NULLORBLANK) AND ([TRTAMI28] CONTAINS SET("{","}","|","\"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI28] = TRTAMI:RP.Trustee:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
            ContainsSet(^TRTAMI28, '"{","}","|","\"')
    
            Data Elements:
    
            ^TRTAMI28 = RP.Trustee:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = IsMatch(report.TRTAMI28, @"\{|\}|\||\\", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000188",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee Non-Individual Name contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + report.RPTrusteeId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000188" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI28", Value = GetValueOrEmpty(report.TRTAMI28) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000188

            #region VR.ATO.TRTAMI.000189

            /*  VR.ATO.TRTAMI.000189
            If any Trustee Liabilities amounts, or any Statement of Attribution (SA) Taxable Determined Member Component amounts have a positive value, then 'Is any tax payable by the trustee?' must be 'yes'.
    
            Legacy Rule Format:
            IF [TRTAMI33] <> "1" AND ([TRTAMI35] > 0 OR [TRTAMI36] > 0 OR [TRTAMI39] > 0 OR [TRTAMI40] > 0 OR ANY OCCURRENCE OF([TRTAMI82]) > 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI33] = TRTAMI:RP:pyin.02.00:Report.Statement.Summary.PaymentRefundOrNil.Code
            [TRTAMI35] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustComponentDeficitOffset.Amount
            [TRTAMI36] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustShortfallIncome.Amount
            [TRTAMI39] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustExcessOffset.Amount
            [TRTAMI40] = TRTAMI:RP:rvctc1.02.24:IncomeTax.TrustComponentNotReflected.Amount
            [TRTAMI82] = TRTAMI:RP.{SoASeqNum}:rvctc4.02.10:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount

            Technical Business Rule Format:
            ^TRTAMI33 <> '1' AND (^TRTAMI35 > 0 OR ^TRTAMI36 > 0 OR ^TRTAMI39 > 0 OR ^TRTAMI40 > 0 OR AnyOccurrence(^TRTAMI82, ^TRTAMI82 > 0))
    
            Data Elements:
    
            ^TRTAMI33 = RP:Report.Statement.Summary.PaymentRefundOrNil.Code
    
            ^TRTAMI35 = RP:IncomeTax.TrustComponentDeficitOffset.Amount
    
            ^TRTAMI36 = RP:IncomeTax.TrustShortfallIncome.Amount
    
            ^TRTAMI39 = RP:IncomeTax.TrustExcessOffset.Amount
    
            ^TRTAMI40 = RP:IncomeTax.TrustComponentNotReflected.Amount
    
            ^TRTAMI82 = RP.{SoASeqNum}:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount
            */
            assertion = (report.TRTAMI33 != @"1" && (report.TRTAMI35.GetValueOrDefault() > 0 || report.TRTAMI36.GetValueOrDefault() > 0 || report.TRTAMI39.GetValueOrDefault() > 0 || report.TRTAMI40.GetValueOrDefault() > 0 || (report.RPSoASeqNumContextCollection == null ? false : report.RPSoASeqNumContextCollection.Any(TRTAMI82Repeat => TRTAMI82Repeat.TRTAMI82.GetValueOrDefault() > 0))));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000189",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Is any tax payable by the Trustee?' must be set to ""1"" (payment)",
                    LongDescription = @"When a positive value has been provided for any Trustee Liabilities amounts or any Statement of Attribution (SA) Taxable Determined Member Component amounts, the 'Is any tax payable by the trustee?' must be set to ""1"" (Payment)",
                    Location = "/xbrli:xbrl/tns:Report.Statement.Summary.PaymentRefundOrNil.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000189" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI33", Value = report.TRTAMI33 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI35", Value = GetValueOrEmpty(report.TRTAMI35) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI36", Value = GetValueOrEmpty(report.TRTAMI36) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI39", Value = GetValueOrEmpty(report.TRTAMI39) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI40", Value = GetValueOrEmpty(report.TRTAMI40) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI82", Value = "TRTAMI82" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000189

            #region VR.ATO.TRTAMI.000191

            /*  VR.ATO.TRTAMI.000191
            Account Number has a maximum field length of 9
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI42]) > 9
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI42] = TRTAMI:RP:pyid.02.20:FinancialInstitutionAccount.FinancialInstitutionAccount.Number

            Technical Business Rule Format:
            Length(^TRTAMI42) > 9
    
            Data Elements:
    
            ^TRTAMI42 = RP:FinancialInstitutionAccount.FinancialInstitutionAccount.Number
            */
            assertion = (Length(report.TRTAMI42) > 9);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000191",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Account Number must not exceed 9 characters",
                    LongDescription = @"For Account Number, the maximum allowable characters for this form is 9",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount.FinancialInstitutionAccount.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000191" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI42", Value = GetValueOrEmpty(report.TRTAMI42) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000191

            #region VR.ATO.TRTAMI.000192

            /*  VR.ATO.TRTAMI.000192
            Description of Main Business or Professional Activity must not exceed 200 characters
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI44]) > 200
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI44] = TRTAMI:RP:pyde.02.05:OrganisationDetails.MainIncomeActivity.Description

            Technical Business Rule Format:
            Length(^TRTAMI44) > 200
    
            Data Elements:
    
            ^TRTAMI44 = RP:OrganisationDetails.MainIncomeActivity.Description
            */
            assertion = (Length(report.TRTAMI44) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000192",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Description of Main Business or Professional Activity must not exceed 200 characters",
                    LongDescription = @"For Description of Main Business or Professional Activity, the maximum allowable characters for this form is 200",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000192" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI44", Value = GetValueOrEmpty(report.TRTAMI44) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000192

            #region VR.ATO.TRTAMI.000204

            /*  VR.ATO.TRTAMI.000204
            Funds or property, whether directly or indirectly, have been sent to, or received from, any of the specified countries must be present
    
            Legacy Rule Format:
            IF [TRTAMI137] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI137] = TRTAMI:RP:rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator

            Technical Business Rule Format:
            ^TRTAMI137 = NULL
    
            Data Elements:
    
            ^TRTAMI137 = RP:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
            */
            assertion = (report.TRTAMI137 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000204",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Funds or property, whether directly or indirectly, have been sent to, ...' must be provided",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000204" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI137", Value = GetValueOrEmpty(report.TRTAMI137) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000204

            #region VR.ATO.TRTAMI.000206

            /*  VR.ATO.TRTAMI.000206
            Funds or property, whether directly or indirectly, have been sent to, or received from, any of the specified countries must be present
    
            Legacy Rule Format:
            IF [TRTAMI138] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI138] = TRTAMI:RP:rvctc4.02.11:InternationalDealings.TransactionControlTaxHavenCountries.Indicator

            Technical Business Rule Format:
            ^TRTAMI138 = NULL
    
            Data Elements:
    
            ^TRTAMI138 = RP:InternationalDealings.TransactionControlTaxHavenCountries.Indicator
            */
            assertion = (report.TRTAMI138 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000206",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trustee has the ability or expectation to control, whether directly or indirectly, ...' must be provided",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.TransactionControlTaxHavenCountries.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000206" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI138", Value = GetValueOrEmpty(report.TRTAMI138) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000206

            #region VR.ATO.TRTAMI.000207

            /*  VR.ATO.TRTAMI.000207
            Have you chosen to treat separate classes in the trust as a separate AMIT? must be present
    
            Legacy Rule Format:
            IF [TRTAMI102] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI102] = TRTAMI:RP:rvctc1.02.20:IncomeTax.MultiClassTreatment.Indicator

            Technical Business Rule Format:
            ^TRTAMI102 = NULL
    
            Data Elements:
    
            ^TRTAMI102 = RP:IncomeTax.MultiClassTreatment.Indicator
            */
            assertion = (report.TRTAMI102 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000207",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Have you chosen to treat separate classes in the trust as a separate AMIT?' must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MultiClassTreatment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000207" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI102", Value = GetValueOrEmpty(report.TRTAMI102) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000207

            #region VR.ATO.TRTAMI.000208

            /*  VR.ATO.TRTAMI.000208
            Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters.
    
            Legacy Rule Format:
            IF [TRTAMI103] <> NULL AND [TRTAMI103] <> NUMERIC(U,7,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI103] = TRTAMI:RP:rvctc1.02.20:IncomeTax.MultiClassTreatment.Count

            Technical Business Rule Format:
            NotNumeric(^TRTAMI103, 'U', 7, 0)
    
            Data Elements:
    
            ^TRTAMI103 = RP:IncomeTax.MultiClassTreatment.Count
            */
            assertion = NotNumeric(report.TRTAMI103, @"U", 7, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000208",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of Separate Classes for Trust is not in a valid numeric format",
                    LongDescription = @"Number of Separate Classes for Trust must be unsigned, not exceed 7 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MultiClassTreatment.Count",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000208" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI103", Value = report.TRTAMI103.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000208

            #region VR.ATO.TRTAMI.000209

            /*  VR.ATO.TRTAMI.000209
            When the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' is 'True' (yes), the minimum Number of Separate Classes for Trust must be at least 1.
    
            Legacy Rule Format:
            IF [TRTAMI102] = TRUE AND ([TRTAMI103] = NULL OR [TRTAMI103] = 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI102] = TRTAMI:RP:rvctc1.02.20:IncomeTax.MultiClassTreatment.Indicator
            [TRTAMI103] = TRTAMI:RP:rvctc1.02.20:IncomeTax.MultiClassTreatment.Count

            Technical Business Rule Format:
            ^TRTAMI102 = TRUE AND (^TRTAMI103 = NULL OR ^TRTAMI103 = 0)
    
            Data Elements:
    
            ^TRTAMI103 = RP:IncomeTax.MultiClassTreatment.Count
    
            ^TRTAMI102 = RP:IncomeTax.MultiClassTreatment.Indicator
            */
            assertion = (report.TRTAMI102 == true && (report.TRTAMI103 == null || report.TRTAMI103 == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000209",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of Separate Classes for Trust is invalid",
                    LongDescription = @"When the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' is 'True' (yes), the minimum Number of Separate Classes for Trust must be at least 1",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MultiClassTreatment.Count",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000209" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI102", Value = GetValueOrEmpty(report.TRTAMI102) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI103", Value = report.TRTAMI103.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000209

            #region VR.ATO.TRTAMI.000210

            /*  VR.ATO.TRTAMI.000210
            When the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' is 'True' (yes), a response to 'Have you transferred assets between classes during this income year?' must be provided
    
            Legacy Rule Format:
            IF [TRTAMI102] = TRUE AND [TRTAMI104] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI102] = TRTAMI:RP:rvctc1.02.20:IncomeTax.MultiClassTreatment.Indicator
            [TRTAMI104] = TRTAMI:RP:rvctc1.02.22:IncomeTax.MultiClassTransfer.Indicator

            Technical Business Rule Format:
            ^TRTAMI102 = TRUE AND ^TRTAMI104 = NULL
    
            Data Elements:
    
            ^TRTAMI104 = RP:IncomeTax.MultiClassTransfer.Indicator
    
            ^TRTAMI102 = RP:IncomeTax.MultiClassTreatment.Indicator
            */
            assertion = (report.TRTAMI102 == true && report.TRTAMI104 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000210",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Have you transferred assets between classes during this income year?' must be provided",
                    LongDescription = @"When the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' is 'True' (yes), a response to 'Have you transferred assets between classes during this income year?' must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MultiClassTransfer.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000210" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI102", Value = GetValueOrEmpty(report.TRTAMI102) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI104", Value = GetValueOrEmpty(report.TRTAMI104) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000210

            #region VR.ATO.TRTAMI.000211

            /*  VR.ATO.TRTAMI.000211
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF  [TRTAMI118] <> NULL AND [TRTAMI118] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI118] = TRTAMI:RP:bafpr1.02.15:Expense.DepreciationAllowableDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI118, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI118 = RP:Expense.DepreciationAllowableDeduction.Amount
            */
            assertion = NotMonetary(report.TRTAMI118, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000211",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Depreciation Deducted for Income Year is not in a valid monetary format",
                    LongDescription = @"Total Depreciation Deducted for Income Year must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Expense.DepreciationAllowableDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000211" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI118", Value = GetValueOrEmpty(report.TRTAMI118) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000211

            #region VR.ATO.TRTAMI.000216

            /*  VR.ATO.TRTAMI.000216
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF  [TRTAMI121] <> NULL AND [TRTAMI121] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI121] = TRTAMI:RP:bafpr1.02.29:Expense.DividendInterestRoyalty.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI121, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI121 = RP:Expense.DividendInterestRoyalty.Amount
            */
            assertion = NotMonetary(report.TRTAMI121, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000216",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Deemed AMIT DIR Payments is not in a valid monetary format",
                    LongDescription = @"Total Deemed Attribution Managed Investment Trust (AMIT) Dividend, Interest or Royalty (DIR) Payments must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Expense.DividendInterestRoyalty.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000216" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI121", Value = GetValueOrEmpty(report.TRTAMI121) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000216

            #region VR.ATO.TRTAMI.000217

            /*  VR.ATO.TRTAMI.000217
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF  [TRTAMI108] <> NULL AND [TRTAMI108] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI108] = TRTAMI:RP:bafpr1.02.13:Income.TrustDistributions.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI108, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI108 = RP:Income.TrustDistributions.Amount
            */
            assertion = NotMonetary(report.TRTAMI108, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000217",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Deemed Fund Payments is not in a valid monetary format",
                    LongDescription = @"Total Deemed Fund Payments must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Income.TrustDistributions.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000217" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI108", Value = GetValueOrEmpty(report.TRTAMI108) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000217

            #region VR.ATO.TRTAMI.000218

            /*  VR.ATO.TRTAMI.000218
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF  [TRTAMI111] <> NULL AND [TRTAMI111] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI111] = TRTAMI:RP:bafpr1.02.29:Expense.ReturnsPaidDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI111, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI111 = RP:Expense.ReturnsPaidDeduction.Amount
            */
            assertion = NotMonetary(report.TRTAMI111, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000218",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Deductions Claimed for Returns Paid is not in a valid monetary format",
                    LongDescription = @"Total Deductions Claimed for Returns Paid must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Expense.ReturnsPaidDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000218" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI111", Value = GetValueOrEmpty(report.TRTAMI111) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000218

            #region VR.ATO.TRTAMI.000219

            /*  VR.ATO.TRTAMI.000219
            Is the Attribution Managed Investment Trust (AMIT) Stapled to another Entity? must be present
    
            Legacy Rule Format:
            IF [TRTAMI112] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI112] = TRTAMI:RP:rvctc1.02.20:IncomeTax.AttributionManagedInvestmentTrustStapledStructure.Indicator

            Technical Business Rule Format:
            ^TRTAMI112 = NULL
    
            Data Elements:
    
            ^TRTAMI112 = RP:IncomeTax.AttributionManagedInvestmentTrustStapledStructure.Indicator
            */
            assertion = (report.TRTAMI112 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000219",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Is the Attribution Managed Investment Trust (AMIT) Stapled to another Entity?' must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.AttributionManagedInvestmentTrustStapledStructure.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000219" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI112", Value = GetValueOrEmpty(report.TRTAMI112) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000219

            #region VR.ATO.TRTAMI.000220

            /*  VR.ATO.TRTAMI.000220
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF  [TRTAMI114] <> NULL AND [TRTAMI114] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI114] = TRTAMI:RP:rvctc2.02.06:IncomeTax.Deduction.CreditsEligibleTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI114, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI114 = RP:IncomeTax.Deduction.CreditsEligibleTotal.Amount
            */
            assertion = NotMonetary(report.TRTAMI114, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000220",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Amount of Eligible Investment Business Income is not in a valid monetary format",
                    LongDescription = @"Total Amount of Eligible Investment Business Income must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CreditsEligibleTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000220" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI114", Value = GetValueOrEmpty(report.TRTAMI114) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000220

            #region VR.ATO.TRTAMI.000221

            /*  VR.ATO.TRTAMI.000221
            Did the Attribution Managed Investment Trust (AMIT) rely on the 2% Non Eligible Investment Business (EIB) Safe Harbour in 102MC during the Income year? must be present
    
            Legacy Rule Format:
            IF [TRTAMI115] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI115] = TRTAMI:RP:rvctc1.02.20:IncomeTax.EligibleInvestmentBusinessSafeHarbour.Indicator

            Technical Business Rule Format:
            ^TRTAMI115 = NULL
    
            Data Elements:
    
            ^TRTAMI115 = RP:IncomeTax.EligibleInvestmentBusinessSafeHarbour.Indicator
            */
            assertion = (report.TRTAMI115 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000221",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Did the AMIT rely on the 2% Non-EIB Safe Harbour in 102MC during the Income year?' must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.EligibleInvestmentBusinessSafeHarbour.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000221" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI115", Value = GetValueOrEmpty(report.TRTAMI115) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000221

            #region VR.ATO.TRTAMI.000222

            /*  VR.ATO.TRTAMI.000222
            Did the Attribution Managed Investment Trust (AMIT) rely on the Rental Safe Harbour rule in 102MB during the Income year? must be present
    
            Legacy Rule Format:
            IF [TRTAMI116] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI116] = TRTAMI:RP:rvctc1.02.20:IncomeTax.RentalSafeHarbour.Indicator

            Technical Business Rule Format:
            ^TRTAMI116 = NULL
    
            Data Elements:
    
            ^TRTAMI116 = RP:IncomeTax.RentalSafeHarbour.Indicator
            */
            assertion = (report.TRTAMI116 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000222",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Did the AMIT rely on the Rental Safe Harbour rule in 102MB during the Income year?' must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.RentalSafeHarbour.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000222" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI116", Value = GetValueOrEmpty(report.TRTAMI116) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000222

            #region VR.ATO.TRTAMI.000223

            /*  VR.ATO.TRTAMI.000223
            Did the Attribution Managed Investment Trust (AMIT) receive Payments from Related Entities during the Income Year? must be present
    
            Legacy Rule Format:
            IF [TRTAMI117] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI117] = TRTAMI:RP:rvctc1.02.22:IncomeTax.AttributionManagedInvestmentTrustRelatedEntitiesPayment.Indicator

            Technical Business Rule Format:
            ^TRTAMI117 = NULL
    
            Data Elements:
    
            ^TRTAMI117 = RP:IncomeTax.AttributionManagedInvestmentTrustRelatedEntitiesPayment.Indicator
            */
            assertion = (report.TRTAMI117 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000223",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Did the AMIT receive Payments from Related Entities during the Income Year?' must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.AttributionManagedInvestmentTrustRelatedEntitiesPayment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000223" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI117", Value = GetValueOrEmpty(report.TRTAMI117) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000223

            #region VR.ATO.TRTAMI.000224

            /*  VR.ATO.TRTAMI.000224
            Additional Information has a maximum field length of 3000
    
            Legacy Rule Format:
            IF LENGTH([TRTAMI101]) > 3000
                 RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI101] = TRTAMI:RP:bafot.02.00:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text

            Technical Business Rule Format:
            Length(^TRTAMI101) > 3000
    
            Data Elements:
    
            ^TRTAMI101 = RP:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text
            */
            assertion = (Length(report.TRTAMI101) > 3000);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000224",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Additional Information must not exceed 3000 characters",
                    LongDescription = @"Additional Information, the maximum allowable characters for this form is 3000",
                    Location = "/xbrli:xbrl/tns:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000224" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI101", Value = GetValueOrEmpty(report.TRTAMI101) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000224

            #region VR.ATO.TRTAMI.000238

            /*  VR.ATO.TRTAMI.000238
            When the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' is 'False' (no), the Number of Separate Classes for Trust must be left blank or set to zero.
    
            Legacy Rule Format:
            IF [TRTAMI102] = FALSE AND ([TRTAMI103] <> NULL AND [TRTAMI103] <> 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI102] = TRTAMI:RP:rvctc1.02.20:IncomeTax.MultiClassTreatment.Indicator
            [TRTAMI103] = TRTAMI:RP:rvctc1.02.20:IncomeTax.MultiClassTreatment.Count

            Technical Business Rule Format:
            ^TRTAMI102 = FALSE AND (^TRTAMI103 <> NULL AND ^TRTAMI103 <> 0)
    
            Data Elements:
    
            ^TRTAMI103 = RP:IncomeTax.MultiClassTreatment.Count
    
            ^TRTAMI102 = RP:IncomeTax.MultiClassTreatment.Indicator
            */
            assertion = (report.TRTAMI102 == false && (report.TRTAMI103 != null && report.TRTAMI103 != 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000238",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of Separate Classes for Trust is incorrect",
                    LongDescription = @"When the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' is 'False' (no), the Number of Separate Classes for Trust must be left blank or set to zero",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MultiClassTreatment.Count",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000238" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI102", Value = GetValueOrEmpty(report.TRTAMI102) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI103", Value = report.TRTAMI103.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000238

            #region VR.ATO.TRTAMI.000283

            /*  VR.ATO.TRTAMI.000283
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI14] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI14] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI14] = TRTAMI:RP.POS:pyde.02.00:AddressDetails.LocalityName.Text

            Technical Business Rule Format:
            (^TRTAMI14 <> BLANK) AND (NotCharacterInSet(^TRTAMI14, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI14 = RP.POS:AddressDetails.LocalityName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI14) != true && !(IsMatch(report.TRTAMI14, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000283",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current Postal Address of Trust - Suburb/Town contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000283" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI14", Value = report.TRTAMI14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000283

            #region VR.ATO.TRTAMI.000284

            /*  VR.ATO.TRTAMI.000284
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI20] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI20] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI20] = TRTAMI:RP.POS.Prev:pyde.02.00:AddressDetails.LocalityName.Text

            Technical Business Rule Format:
            (^TRTAMI20 <> BLANK) AND (NotCharacterInSet(^TRTAMI20, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI20 = RP.POS.Prev:AddressDetails.LocalityName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI20) != true && !(IsMatch(report.TRTAMI20, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000284",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous Postal Address of Trust - Suburb/Town contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + report.RPPOSPrevId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000284" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI20", Value = GetValueOrEmpty(report.TRTAMI20) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000284

            #region VR.ATO.TRTAMI.000287

            /*  VR.ATO.TRTAMI.000287
            A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
            Legacy Rule Format:
            IF ([TRTAMI101] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI101] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI101] = TRTAMI:RP:bafot.02.00:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text

            Technical Business Rule Format:
            (^TRTAMI101 <> BLANK) AND (NotCharacterInSet(^TRTAMI101, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
    
            Data Elements:
    
            ^TRTAMI101 = RP:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRTAMI101) != true && !(IsMatch(report.TRTAMI101, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000287",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Additional Information contains invalid text",
                    LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                    Location = "/xbrli:xbrl/tns:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000287" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI101", Value = GetValueOrEmpty(report.TRTAMI101) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000287

            #region VR.ATO.TRTAMI.000289

            /*  VR.ATO.TRTAMI.000289
            Has the Trustee made an election into Managed Investment Trust Capital Account Treatment? must be present
    
            Legacy Rule Format:
            IF [TRTAMI32] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI32] = TRTAMI:RP:rvctc3.02.24:Elections.ManagedInvestmentTrusts.Indicator

            Technical Business Rule Format:
            ^TRTAMI32 = NULL
    
            Data Elements:
    
            ^TRTAMI32 = RP:Elections.ManagedInvestmentTrusts.Indicator
            */
            assertion = (report.TRTAMI32 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000289",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Has the Trustee made an election into Managed Investment Trust Capital Account Treatment?' must be provided",
                    Location = "/xbrli:xbrl/tns:Elections.ManagedInvestmentTrusts.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000289" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI32", Value = GetValueOrEmpty(report.TRTAMI32) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000289

            #region VR.ATO.TRTAMI.000290

            /*  VR.ATO.TRTAMI.000290
            When any Stapled Entity Australian Business Numbers are provided, the response to 'Is the Attribution Managed Investment Trust (AMIT) Stapled to another Entity?' must be set to 'True' (yes).
    
            Legacy Rule Format:
            IF [TRTAMI112] <> TRUE AND ANY OCCURRENCE OF([TRTAMI139]) <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI112] = TRTAMI:RP:rvctc1.02.20:IncomeTax.AttributionManagedInvestmentTrustStapledStructure.Indicator
            [TRTAMI139] = TRTAMI:RP.StapledEntity:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier

            Technical Business Rule Format:
            ^TRTAMI112 <> TRUE AND AnyOccurrence(^TRTAMI139, ^TRTAMI139 <> NULL)
    
            Data Elements:
    
            ^TRTAMI112 = RP:IncomeTax.AttributionManagedInvestmentTrustStapledStructure.Indicator
    
            ^TRTAMI139 = RP.StapledEntity:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (report.TRTAMI112 != true && (report.TRTAMI139Collection == null ? false : report.TRTAMI139Collection.Any(TRTAMI139Repeat => TRTAMI139Repeat != null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000290",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Is the Attribution Managed Investment Trust (AMIT) Stapled to another Entity?' is incorrect",
                    LongDescription = @"When any Stapled Entity Australian Business Numbers are provided, the response to 'Is the Attribution Managed Investment Trust (AMIT) Stapled to another Entity?' must be set to 'True' (yes)",
                    Location = "/xbrli:xbrl/tns:IncomeTax.AttributionManagedInvestmentTrustStapledStructure.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000290" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI112", Value = GetValueOrEmpty(report.TRTAMI112) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI139", Value = "TRTAMI139" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000290

            #region VR.ATO.TRTAMI.000291

            /*  VR.ATO.TRTAMI.000291
            When the response to 'Is the Attribution Managed Investment Trust (AMIT) Stapled to another Entity?' is 'True' (yes), at least one Stapled Entity Australian Business Number must be provided.
    
            Legacy Rule Format:
            IF [TRTAMI112] = TRUE AND ALL OCCURRENCES OF([TRTAMI139]) = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI112] = TRTAMI:RP:rvctc1.02.20:IncomeTax.AttributionManagedInvestmentTrustStapledStructure.Indicator
            [TRTAMI139] = TRTAMI:RP.StapledEntity:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier

            Technical Business Rule Format:
            ^TRTAMI112 = TRUE AND AllOccurrences(^TRTAMI139, ^TRTAMI139 = NULL)
    
            Data Elements:
    
            ^TRTAMI139 = RP.StapledEntity:Identifiers.AustralianBusinessNumber.Identifier
    
            ^TRTAMI112 = RP:IncomeTax.AttributionManagedInvestmentTrustStapledStructure.Indicator
            */
            assertion = (report.TRTAMI112 == true && (report.TRTAMI139Collection == null ? false : report.TRTAMI139Collection.All(TRTAMI139Repeat => TRTAMI139Repeat == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000291",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of Separate Classes for Trust is invalid",
                    LongDescription = @"When the response to 'Is the Attribution Managed Investment Trust (AMIT) Stapled to another Entity?' is 'True' (yes), at least one Stapled Entity Australian Business Number must be provided",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000291" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI112", Value = GetValueOrEmpty(report.TRTAMI112) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI139", Value = "TRTAMI139" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000291

            #region VR.ATO.TRTAMI.000292

            /*  VR.ATO.TRTAMI.000292
            The maximum number of Stapled Entity Australian Business Numbers that may be provided is 10.
    
            Legacy Rule Format:
            IF COUNT([TRTAMI139]) > 10
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI139] = TRTAMI:RP.StapledEntity:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier

            Technical Business Rule Format:
            Count(^TRTAMI139) > 10
    
            Data Elements:
    
            ^TRTAMI139 = RP.StapledEntity:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (Count(report.TRTAMI139Collection) > 10);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000292",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The maximum number of Stapled Entity Australian Business Numbers that may be provided is 10",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier[@contextRef='" + report.RPStapledEntityId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000292" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000292

            #region VR.ATO.TRTAMI.000296

            /*  VR.ATO.TRTAMI.000296
            Have you self-assessed the effective life of any depreciating assets acquired in the income year? must be present
    
            Legacy Rule Format:
            IF [TRTAMI63] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI63] = TRTAMI:RP:bafpo7.02.22:Assets.PropertyPlantAndEquipment.DepreciatingAssets.FirstDeductedSelfAssessedEffectiveLife.Indicator

            Technical Business Rule Format:
            ^TRTAMI63 = NULL
    
            Data Elements:
    
            ^TRTAMI63 = RP:Assets.PropertyPlantAndEquipment.DepreciatingAssets.FirstDeductedSelfAssessedEffectiveLife.Indicator
            */
            assertion = (report.TRTAMI63 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000296",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Have you self-assessed the effective life of any depreciating assets acquired in the income year?' must be provided",
                    Location = "/xbrli:xbrl/tns:Assets.PropertyPlantAndEquipment.DepreciatingAssets.FirstDeductedSelfAssessedEffectiveLife.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000296" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI63", Value = GetValueOrEmpty(report.TRTAMI63) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000296

            #region VR.ATO.TRTAMI.000297

            /*  VR.ATO.TRTAMI.000297
            Have you self-assessed the effective life of any depreciating assets acquired in the income year? must be present
    
            Legacy Rule Format:
            IF [TRTAMI64] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI64] = TRTAMI:RP:bafpo7.02.22:Assets.PropertyPlantAndEquipment.DepreciatingAssets.EffectiveLifeRecalculation.Indicator

            Technical Business Rule Format:
            ^TRTAMI64 = NULL
    
            Data Elements:
    
            ^TRTAMI64 = RP:Assets.PropertyPlantAndEquipment.DepreciatingAssets.EffectiveLifeRecalculation.Indicator
            */
            assertion = (report.TRTAMI64 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000297",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Did you recalculate the effective life for any of your depreciating assets this income year?' must be provided",
                    Location = "/xbrli:xbrl/tns:Assets.PropertyPlantAndEquipment.DepreciatingAssets.EffectiveLifeRecalculation.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000297" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI64", Value = GetValueOrEmpty(report.TRTAMI64) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000297

            #region VR.ATO.TRTAMI.000298

            /*  VR.ATO.TRTAMI.000298
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF  [TRTAMI140] <> NULL AND [TRTAMI140] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI140] = TRTAMI:RP:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI140, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI140 = RP:Expense.CapitalWorksDeduction.Amount
            */
            assertion = NotMonetary(report.TRTAMI140, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000298",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Division 43 Capital Works Deductions (special building write-off) is not in a valid monetary format",
                    LongDescription = @"Total Division 43 Capital Works Deductions (special building write-off) must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:Expense.CapitalWorksDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000298" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI140", Value = GetValueOrEmpty(report.TRTAMI140) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000298

            #region VR.ATO.TRTAMI.000299

            /*  VR.ATO.TRTAMI.000299
            Invalid context. The number of Reporting Party Stapled Entity contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(RP.StapledEntity) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.StapledEntity
            */
            assertion = (Count(report.RPStapledEntityCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000299",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Stapled Entity contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPStapledEntityLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000299" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000299

            #region VR.ATO.TRTAMI.000300

            /*  VR.ATO.TRTAMI.000300
            For Percentage of Income from things other than an Eligible Investment Business, the valid values are; "0% to 0.5%", "Over 0.5% to 1.0%", "Over 1.0% to 1.5%", "Over 1.5% to 2.0%".
    
            Legacy Rule Format:
            IF [TRTAMI141] <> NULL AND [TRTAMI141] = 'Over 2.0%'
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI141] = TRTAMI:RP:rvctc2.02.19:IncomeTax.Deduction.NonEligibleInvestmentBusinessPercentage.Code

            Technical Business Rule Format:
            ^TRTAMI141 <> NULL AND ^TRTAMI141 = 'Over 2.0%'
    
            Data Elements:
    
            ^TRTAMI141 = RP:IncomeTax.Deduction.NonEligibleInvestmentBusinessPercentage.Code
            */
            assertion = (report.TRTAMI141 != null && report.TRTAMI141 == @"Over 2.0%");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000300",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Percentage of Income from things other than an Eligible Investment Business is invalid",
                    LongDescription = @"For Percentage of Income from things other than an Eligible Investment Business, the valid values are; ""0% to 0.5%"", ""Over 0.5% to 1.0%"", ""Over 1.0% to 1.5%"", ""Over 1.5% to 2.0%"".",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.NonEligibleInvestmentBusinessPercentage.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000300" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI141", Value = GetValueOrEmpty(report.TRTAMI141) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000300

            #region VR.ATO.TRTAMI.000301

            /*  VR.ATO.TRTAMI.000301
            When a positive amount is provided for Total Payments Received from Related Entities during the Income Year, the response to 'Did the Attribution Managed Investment Trust (AMIT) receive Payments from Related Entities during the Income Year?' must be set to 'True' (yes).
    
            Legacy Rule Format:
            IF [TRTAMI117] <> TRUE AND [TRTAMI142] > 0
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI117] = TRTAMI:RP:rvctc1.02.22:IncomeTax.AttributionManagedInvestmentTrustRelatedEntitiesPayment.Indicator
            [TRTAMI142] = TRTAMI:RP:rvctc1.02.24:IncomeTax.RelatedEntitiesPaymentsReceived.Amount

            Technical Business Rule Format:
            ^TRTAMI117 <> TRUE AND ^TRTAMI142 > 0
    
            Data Elements:
    
            ^TRTAMI117 = RP:IncomeTax.AttributionManagedInvestmentTrustRelatedEntitiesPayment.Indicator
    
            ^TRTAMI142 = RP:IncomeTax.RelatedEntitiesPaymentsReceived.Amount
            */
            assertion = (report.TRTAMI117 != true && report.TRTAMI142.GetValueOrDefault() > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000301",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Did the AMIT receive Payments from Related Entities during the Income Year?' is incorrect",
                    LongDescription = @"When a positive amount is provided for Total Payments Received from Related Entities during the Income Year, the response to 'Did the Attribution Managed Investment Trust (AMIT) receive Payments from Related Entities during the Income Year?' must be set to 'True' (yes)",
                    Location = "/xbrli:xbrl/tns:IncomeTax.AttributionManagedInvestmentTrustRelatedEntitiesPayment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000301" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI117", Value = GetValueOrEmpty(report.TRTAMI117) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI142", Value = GetValueOrEmpty(report.TRTAMI142) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000301

            #region VR.ATO.TRTAMI.000302

            /*  VR.ATO.TRTAMI.000302
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF  [TRTAMI142] <> NULL AND [TRTAMI142] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI142] = TRTAMI:RP:rvctc1.02.24:IncomeTax.RelatedEntitiesPaymentsReceived.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI142, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI142 = RP:IncomeTax.RelatedEntitiesPaymentsReceived.Amount
            */
            assertion = NotMonetary(report.TRTAMI142, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000302",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Payments Received from Related Entities during the Income Year is not in a valid monetary format",
                    LongDescription = @"Total Payments Received from Related Entities during the Income Year must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                    Location = "/xbrli:xbrl/tns:IncomeTax.RelatedEntitiesPaymentsReceived.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000302" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI142", Value = GetValueOrEmpty(report.TRTAMI142) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000302

            #region VR.ATO.TRTAMI.000303

            /*  VR.ATO.TRTAMI.000303
            When the response to 'Did the Attribution Managed Investment Trust (AMIT) receive Payments from Related Entities during the Income Year?' is 'True' (yes), a positive amount for Total Payments Received from Related Entities during the Income Year must be provided.
    
            Legacy Rule Format:
            IF [TRTAMI117] = TRUE AND ([TRTAMI142] = NULL OR [TRTAMI142] = 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI117] = TRTAMI:RP:rvctc1.02.22:IncomeTax.AttributionManagedInvestmentTrustRelatedEntitiesPayment.Indicator
            [TRTAMI142] = TRTAMI:RP:rvctc1.02.24:IncomeTax.RelatedEntitiesPaymentsReceived.Amount

            Technical Business Rule Format:
            ^TRTAMI117 = TRUE AND (^TRTAMI142 = NULL OR ^TRTAMI142 = 0)
    
            Data Elements:
    
            ^TRTAMI142 = RP:IncomeTax.RelatedEntitiesPaymentsReceived.Amount
    
            ^TRTAMI117 = RP:IncomeTax.AttributionManagedInvestmentTrustRelatedEntitiesPayment.Indicator
            */
            assertion = (report.TRTAMI117 == true && (report.TRTAMI142 == null || report.TRTAMI142 == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000303",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Payments Received from Related Entities during the Income Year is invalid",
                    LongDescription = @"When the response to 'Did the Attribution Managed Investment Trust (AMIT) receive Payments from Related Entities during the Income Year?' is 'True' (yes), a positive amount for Total Payments Received from Related Entities during the Income Year must be provided.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.RelatedEntitiesPaymentsReceived.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000303" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI117", Value = GetValueOrEmpty(report.TRTAMI117) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI142", Value = GetValueOrEmpty(report.TRTAMI142) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000303

            #region VR.ATO.TRTAMI.000312

            /*  VR.ATO.TRTAMI.000312
            Year of return must be either the current or next (future) lodgment year associated with this service. If a future year lodgment is required, then it must be lodged no later than the 15th of June of the current year, unless it is an amendment. After the 15th of June, a return for that lodgment year must be submitted from the 1st of July, using the new service associated to that specific lodgment year.
    
            Legacy Rule Format:
            IF ([TRTAMI1] <> SET(2022,2023))
            OR ([TRTAMI1] = 2023 AND DATE(Today) > 2023-06-15 
            AND [TRTAMI6] <> TRUE)
             RETURN VALIDATION MESSAGE 
            ENDIF
            
            [TRTAMI1] = TRTAMI:RP:pyin.02.37:Report.TargetFinancial.Year
            [TRTAMI6] = TRTAMI:RP:pyin.02.28:Report.Amendment.Indicator

            Technical Business Rule Format:
            (NotInSet(^TRTAMI1, '"2022","2023"')) OR (^TRTAMI1 = 2023 AND Today() > AsDate('2023-06-15') AND ^TRTAMI6 <> TRUE)
    
            Data Elements:
    
            ^TRTAMI1 = RP:Report.TargetFinancial.Year
    
            ^TRTAMI6 = RP:Report.Amendment.Indicator
            */
            assertion = (!(IsMatch(report.TRTAMI1.GetValueOrDefault().ToString(), @"^(2022|2023)$", RegexOptions.IgnoreCase)) || report.TRTAMI1.GetValueOrDefault() == 2023 && DateTime.Now.Date > AsDate(@"2023-06-15") && report.TRTAMI6 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500127",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of return is invalid",
                    LongDescription = @"Year of return must be either the current or next (future) lodgment year associated with this service. If a future year lodgment is required, then it must be lodged no later than the 15th of June of the current year, unless it is an amendment. After the 15th of June, a return for that lodgment year must be submitted from the 1st of July, using the new service associated to that specific lodgment year.",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000312" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI1", Value = GetValueOrEmpty(report.TRTAMI1) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI6", Value = GetValueOrEmpty(report.TRTAMI6) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000312

            #region VR.ATO.TRTAMI.000331

            /*  VR.ATO.TRTAMI.000331
            If the trust has self-assessed that they are a Country by Country Reporting Entity (CbCRE), then the Significant Global Entity (SGE) Indicator must be true.
    
            Legacy Rule Format:
            IF [TRTAMI163] = TRUE AND ([TRTAMI162] = NULL OR [TRTAMI162] = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI162] = TRTAMI:RP:pyde.02.43:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
            [TRTAMI163] = TRTAMI:RP:rvctc4.02.17:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator

            Technical Business Rule Format:
            ^TRTAMI163 = TRUE AND (^TRTAMI162 = NULL OR ^TRTAMI162 = FALSE)
    
            Data Elements:
    
            ^TRTAMI163 = RP:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator
    
            ^TRTAMI162 = RP:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
            */
            assertion = (report.TRTAMI163 == true && (report.TRTAMI162 == null || report.TRTAMI162 == false));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000331",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Significant Global Entity (SGE)  must be 'TRUE'.",
                    LongDescription = @"If the trust has self-assessed that they are a Country by Country Reporting Entity (CbCRE), then the Significant Global Entity (SGE) Indicator must be true.",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000331" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI163", Value = GetValueOrEmpty(report.TRTAMI163) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI162", Value = GetValueOrEmpty(report.TRTAMI162) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000331

            #region VR.ATO.TRTAMI.000332

            /*  VR.ATO.TRTAMI.000332
            Number of assets and value of assets you are opting out for must be provided if opting out for some or all of your eligible assets.
    
            Legacy Rule Format:
            IF [TRTAMI164] <> NULL AND ([TRTAMI153] = NULL OR ([TRTAMI154] = NULL OR [TRTAMI154] =0))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI164] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            [TRTAMI153] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
            [TRTAMI154] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount

            Technical Business Rule Format:
            ^TRTAMI164 <> NULL AND (^TRTAMI153 = NULL OR (^TRTAMI154 = NULL OR ^TRTAMI154 =0))
    
            Data Elements:
    
            ^TRTAMI164 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
    
            ^TRTAMI153 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
    
            ^TRTAMI154 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount
            */
            assertion = (report.TRTAMI164 != null && (report.TRTAMI153 == null || (report.TRTAMI154 == null || report.TRTAMI154 == 0)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000332",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of assets and value of assets you are opting out for must be provided if opting out for some or all of your eligible assets.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000332" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI164", Value = GetValueOrEmpty(report.TRTAMI164) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI153", Value = report.TRTAMI153.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI154", Value = GetValueOrEmpty(report.TRTAMI154) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000332

            #region VR.ATO.TRTAMI.000333

            /*  VR.ATO.TRTAMI.000333
            Number of assets and value of assets you are opting out for must not be provided if not opting out for some or all of your eligible assets.
    
            Legacy Rule Format:
            IF [TRTAMI164] = NULL AND ([TRTAMI153] <> NULL OR [TRTAMI154] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI164] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            [TRTAMI153] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
            [TRTAMI154] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount

            Technical Business Rule Format:
            ^TRTAMI164 = NULL AND (^TRTAMI153 <> NULL OR ^TRTAMI154 <> NULL)
    
            Data Elements:
    
            ^TRTAMI164 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
    
            ^TRTAMI153 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
    
            ^TRTAMI154 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount
            */
            assertion = (report.TRTAMI164 == null && (report.TRTAMI153 != null || report.TRTAMI154 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000333",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of assets and value of assets you are opting out for must not be provided if not opting out for some or all of your eligible assets.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000333" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI164", Value = GetValueOrEmpty(report.TRTAMI164) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI153", Value = report.TRTAMI153.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI154", Value = GetValueOrEmpty(report.TRTAMI154) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000333

            #region VR.ATO.TRTAMI.000334

            /*  VR.ATO.TRTAMI.000334
            Temporary full expensing deductions must be provided if opting out of temporary full expensing for some of your eligible assets.
    
            Legacy Rule Format:
            IF [TRTAMI164] = "Some eligible assets" AND ([TRTAMI155] = NULL OR [TRTAMI155] = 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI164] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            [TRTAMI155] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount

            Technical Business Rule Format:
            ^TRTAMI164 = 'Some eligible assets' AND (^TRTAMI155 = NULL OR ^TRTAMI155 = 0)
    
            Data Elements:
    
            ^TRTAMI164 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
    
            ^TRTAMI155 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = (report.TRTAMI164 == @"Some eligible assets" && (report.TRTAMI155 == null || report.TRTAMI155 == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000334",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Temporary full expensing deductions must be provided if opting out of temporary full expensing for some of your eligible assets.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000334" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI164", Value = GetValueOrEmpty(report.TRTAMI164) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI155", Value = GetValueOrEmpty(report.TRTAMI155) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000334

            #region VR.ATO.TRTAMI.000335

            /*  VR.ATO.TRTAMI.000335
            Temporary full expensing deductions must not be provided if opting out of temporary full expensing for all of your eligible assets.
    
            Legacy Rule Format:
            IF [TRTAMI164] = "All eligible assets" AND [TRTAMI155] <> NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI164] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            [TRTAMI155] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount

            Technical Business Rule Format:
            ^TRTAMI164 = 'All eligible assets' AND ^TRTAMI155 <> NULL
    
            Data Elements:
    
            ^TRTAMI164 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
    
            ^TRTAMI155 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = (report.TRTAMI164 == @"All eligible assets" && report.TRTAMI155 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000335",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Temporary full expensing deductions must not be provided if opting out of temporary full expensing for all of your eligible assets.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000335" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI164", Value = GetValueOrEmpty(report.TRTAMI164) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI155", Value = GetValueOrEmpty(report.TRTAMI155) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000335

            #region VR.ATO.TRTAMI.000336

            /*  VR.ATO.TRTAMI.000336
            Number of assets you are opting out for is not in a valid format.
    
            Legacy Rule Format:
            IF [TRTAMI153] <> NULL AND  [TRTAMI153] <> NUMBER (U,5,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI153] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number

            Technical Business Rule Format:
            NotNumeric(^TRTAMI153, 'U', 5, 0)
    
            Data Elements:
    
            ^TRTAMI153 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
            */
            assertion = NotNumeric(report.TRTAMI153, @"U", 5, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000336",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of assets you are opting out for is not in a valid format.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000336" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI153", Value = report.TRTAMI153.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000336

            #region VR.ATO.TRTAMI.000337

            /*  VR.ATO.TRTAMI.000337
            Value of assets you are opting out for is not in a valid monetary format.
    
            Legacy Rule Format:
            IF [TRTAMI154] <> NULL AND [TRTAMI154] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI154] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI154, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI154 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount
            */
            assertion = NotMonetary(report.TRTAMI154, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000337",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Value of assets you are opting out for is not in a valid monetary format.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000337" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI154", Value = GetValueOrEmpty(report.TRTAMI154) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000337

            #region VR.ATO.TRTAMI.000338

            /*  VR.ATO.TRTAMI.000338
            Temporary full expensing deductions is not in a valid monetary format.
    
            Legacy Rule Format:
            IF [TRTAMI155] <> NULL AND [TRTAMI155] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI155] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI155, 'U', 11, 0)
    
            Data Elements:
    
            ^TRTAMI155 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = NotMonetary(report.TRTAMI155, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000338",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Temporary full expensing deductions is not in a valid monetary format.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000338" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI155", Value = GetValueOrEmpty(report.TRTAMI155) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000338

            #region VR.ATO.TRTAMI.000339

            /*  VR.ATO.TRTAMI.000339
            Number of assets you are claiming for is not in a valid format.
    
            Legacy Rule Format:
            IF [TRTAMI156] <> NULL AND  [TRTAMI156] <> NUMBER (U,5,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI156] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number

            Technical Business Rule Format:
            NotNumeric(^TRTAMI156, 'U', 5, 0)
    
            Data Elements:
    
            ^TRTAMI156 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number
            */
            assertion = NotNumeric(report.TRTAMI156, @"U", 5, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000339",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of assets you are claiming for is not in a valid format.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000339" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI156", Value = report.TRTAMI156.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000339

            #region VR.ATO.TRTAMI.000340

            /*  VR.ATO.TRTAMI.000340
            Number of assets you are claiming for must be provided if temporary full expensing deductions is supplied.
    
            Legacy Rule Format:
            IF [TRTAMI155] <> NULL AND [TRTAMI156] = NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI155] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            [TRTAMI156] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number

            Technical Business Rule Format:
            ^TRTAMI155 <> NULL AND ^TRTAMI156 = NULL
    
            Data Elements:
    
            ^TRTAMI156 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number
    
            ^TRTAMI155 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = (report.TRTAMI155 != null && report.TRTAMI156 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000340",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of assets you are claiming for must be provided if temporary full expensing deductions is supplied.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000340" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI155", Value = GetValueOrEmpty(report.TRTAMI155) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI156", Value = report.TRTAMI156.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000340

            #region VR.ATO.TRTAMI.000341

            /*  VR.ATO.TRTAMI.000341
            Number of assets you are claiming for must not be provided if temporary full expensing deductions is not supplied.
    
            Legacy Rule Format:
            IF [TRTAMI155] = NULL AND [TRTAMI156] <> NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI155] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            [TRTAMI156] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number

            Technical Business Rule Format:
            ^TRTAMI155 = NULL AND ^TRTAMI156 <> NULL
    
            Data Elements:
    
            ^TRTAMI156 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number
    
            ^TRTAMI155 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = (report.TRTAMI155 == null && report.TRTAMI156 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000341",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of assets you are claiming for must not be provided if temporary full expensing deductions is not supplied.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000341" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI155", Value = GetValueOrEmpty(report.TRTAMI155) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI156", Value = report.TRTAMI156.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000341

            #region VR.ATO.TRTAMI.000346

            /*  VR.ATO.TRTAMI.000346
            Aggregated turnover range must be provided if temporary full expensing deductions is supplied.
    
            Legacy Rule Format:
            IF [TRTAMI155] > 0 AND [TRTAMI160] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI155] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            [TRTAMI160] = TRTAMI:RP:bafpr1.02.44:Income.AggregatedTurnoverRange.Code

            Technical Business Rule Format:
            ^TRTAMI155 > 0 AND ^TRTAMI160 = NULL
    
            Data Elements:
    
            ^TRTAMI160 = RP:Income.AggregatedTurnoverRange.Code
    
            ^TRTAMI155 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = (report.TRTAMI155.GetValueOrDefault() > 0 && report.TRTAMI160 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000346",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Aggregated turnover range must be provided if temporary full expensing deductions is supplied.",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnoverRange.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000346" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI155", Value = GetValueOrEmpty(report.TRTAMI155) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI160", Value = GetValueOrEmpty(report.TRTAMI160) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000346

            #region VR.ATO.TRTAMI.000347

            /*  VR.ATO.TRTAMI.000347
            Aggregated turnover must be provided if aggregated turnover range of 'P' is supplied.
    
            Legacy Rule Format:
            IF [TRTAMI160] = "P" AND ([TRTAMI161] = NULL OR [TRTAMI161] = 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI160] = TRTAMI:RP:bafpr1.02.44:Income.AggregatedTurnoverRange.Code
            [TRTAMI161] = TRTAMI:RP:bafpr1.02.44:Income.AggregatedTurnover.Amount

            Technical Business Rule Format:
            ^TRTAMI160 = 'P' AND (^TRTAMI161 = NULL OR ^TRTAMI161 = 0)
    
            Data Elements:
    
            ^TRTAMI160 = RP:Income.AggregatedTurnoverRange.Code
    
            ^TRTAMI161 = RP:Income.AggregatedTurnover.Amount
            */
            assertion = (report.TRTAMI160 == @"P" && (report.TRTAMI161 == null || report.TRTAMI161 == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000347",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Aggregated turnover must be provided if aggregated turnover range of 'P' is supplied.",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnoverRange.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000347" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI160", Value = GetValueOrEmpty(report.TRTAMI160) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI161", Value = GetValueOrEmpty(report.TRTAMI161) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000347

            #region VR.ATO.TRTAMI.000348

            /*  VR.ATO.TRTAMI.000348
            Aggregated turnover is not in a valid monetary format.
    
            Legacy Rule Format:
            IF [TRTAMI161] <> NULL AND [TRTAMI161] <> MONETARY(U,12,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI161] = TRTAMI:RP:bafpr1.02.44:Income.AggregatedTurnover.Amount

            Technical Business Rule Format:
            NotMonetary(^TRTAMI161, 'U', 12, 0)
    
            Data Elements:
    
            ^TRTAMI161 = RP:Income.AggregatedTurnover.Amount
            */
            assertion = NotMonetary(report.TRTAMI161, @"U", 12, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000348",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Aggregated turnover is not in a valid monetary format.",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnover.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000348" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI161", Value = GetValueOrEmpty(report.TRTAMI161) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000348

            #region VR.ATO.TRTAMI.000349

            /*  VR.ATO.TRTAMI.000349
            Aggregated turnover exceeds the threshold to claim temporary full expensing deductions.
    
            Legacy Rule Format:
            IF [TRTAMI161] >= 5000000000 AND [TRTAMI155] <> NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI161] = TRTAMI:RP:bafpr1.02.44:Income.AggregatedTurnover.Amount
            [TRTAMI155] = TRTAMI:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount

            Technical Business Rule Format:
            ^TRTAMI161 >= 5000000000 AND ^TRTAMI155 <> NULL
    
            Data Elements:
    
            ^TRTAMI161 = RP:Income.AggregatedTurnover.Amount
    
            ^TRTAMI155 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = (report.TRTAMI161.GetValueOrDefault() >= 5000000000 && report.TRTAMI155 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000349",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Aggregated turnover exceeds the threshold to claim temporary full expensing deductions.",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnover.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000349" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI161", Value = GetValueOrEmpty(report.TRTAMI161) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI155", Value = GetValueOrEmpty(report.TRTAMI155) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.000349

            #region VR.ATO.TRTAMI.W00001

            /*  VR.ATO.TRTAMI.W00001
            If the trust has self-assessed that they are a Significant Global Entity (SGE) then please confirm if they are a Country by Country Reporting Entity (CbCRE).
    
            Legacy Rule Format:
            IF [TRTAMI163] = NULL AND [TRTAMI162] = TRUE
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI162] = TRTAMI:RP:pyde.02.43:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
            [TRTAMI163] = TRTAMI:RP:rvctc4.02.17:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator

            Technical Business Rule Format:
            ^TRTAMI163 = NULL AND ^TRTAMI162 = TRUE
    
            Data Elements:
    
            ^TRTAMI163 = RP:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator
    
            ^TRTAMI162 = RP:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
            */
            assertion = (report.TRTAMI163 == null && report.TRTAMI162 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.W00001",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Country by Country Reporting Entity (CbCRE) indicator not supplied.",
                    LongDescription = @"If the trust has self-assessed that they are a Significant Global Entity (SGE) then please confirm if they are a Country by Country Reporting Entity (CbCRE).",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.W00001" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI163", Value = GetValueOrEmpty(report.TRTAMI163) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI162", Value = GetValueOrEmpty(report.TRTAMI162) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.W00001

            #region VR.ATO.TRTAMI.W00003

            /*  VR.ATO.TRTAMI.W00003
            If 'P' is selected at 'Aggregated turnover range', then the aggregated turnover amount should be $1 billion or over.
    
            Legacy Rule Format:
            IF [TRTAMI160] = "P" AND [TRTAMI161] < 1000000000
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRTAMI160] = TRTAMI:RP:bafpr1.02.44:Income.AggregatedTurnoverRange.Code
            [TRTAMI161] = TRTAMI:RP:bafpr1.02.44:Income.AggregatedTurnover.Amount

            Technical Business Rule Format:
            ^TRTAMI160 = 'P' AND ^TRTAMI161 < 1000000000
    
            Data Elements:
    
            ^TRTAMI160 = RP:Income.AggregatedTurnoverRange.Code
    
            ^TRTAMI161 = RP:Income.AggregatedTurnover.Amount
            */
            assertion = (report.TRTAMI160 == @"P" && report.TRTAMI161.GetValueOrDefault() < 1000000000);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.W00003",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Aggregated turnover amount is incorrect.",
                    LongDescription = @"If 'P' is selected at 'Aggregated turnover range', then the aggregated turnover amount should be $1 billion or over.",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnoverRange.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.W00003" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI160", Value = GetValueOrEmpty(report.TRTAMI160) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRTAMI161", Value = GetValueOrEmpty(report.TRTAMI161) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.TRTAMI.W00003

            #region Repeating RPSoASeqNumContext
            if (report.RPSoASeqNumContextCollection != null)
            {
                int itemIndexContext2 = 0;
                foreach (TRTAMI2022.RPSoASeqNumContext rpSoASeqNumContext in report.RPSoASeqNumContextCollection)
                {
                    itemIndexContext2++;

                    #region VR.ATO.GEN.001023

                    /*  VR.ATO.GEN.001023
                    TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
                    Legacy Rule Format:
                    IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^Context <> NULL AND ^OtherTFN <> ^TFN
            
                    Data Elements:
            
                    ^Context = RP:RP.{SoASeqNum}
            
                    ^OtherTFN = RP:RP.{SoASeqNum}
            
                    ^TFN = RP
                    */
                    assertion = (rpSoASeqNumContext.Exists != false && rpSoASeqNumContext.IdentifierTFN != report.RPIdentifierTFN);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.001023",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Unexpected TFN mismatch with Reporting Party",
                            LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpSoASeqNumContext.OccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.001023

                    #region VR.ATO.GEN.438029

                    /*  VR.ATO.GEN.438029
                    Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
                    Legacy Rule Format:
                    IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
            
                    Data Elements:
            
                    ^IdentifierScheme = RP:RP.{SoASeqNum}
            
                    ^Context = RP:RP.{SoASeqNum}
                    */
                    assertion = (rpSoASeqNumContext.Exists != false && rpSoASeqNumContext.IdentifierScheme != @"http://www.ato.gov.au/tfn");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438029",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpSoASeqNumContext.OccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.438029

                    #region VR.ATO.GEN.500012

                    /*  VR.ATO.GEN.500012
                    The date entered is a future date. Date of birth cannot be any day in the future.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonDemographicDetails.Birth.Date <> NULL) AND (pyde.xx.xx:PersonDemographicDetails.Birth.Date > RP:period.startDate)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRTAMI124 <> NULL) AND (^TRTAMI124 > ^StartDate1)
            
                    Data Elements:
            
                    ^TRTAMI124 = RP.{SoASeqNum}:PersonDemographicDetails.Birth.Date
            
                    ^StartDate1 = RP
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI124 != null && rpSoASeqNumContext.TRTAMI124.GetValueOrDefault() > report.RPStartDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500012",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Date of birth must be prior to the form submission start date",
                            LongDescription = @"The date entered is a future date. Date of birth cannot be any day in advance of the form submission start date.",
                            Location = "/xbrli:xbrl/tns:PersonDemographicDetails.Birth.Date[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500012" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI124", Value = GetValueOrEmptyDate(rpSoASeqNumContext.TRTAMI124) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500012

                    #region VR.ATO.GEN.500030

                    /*  VR.ATO.GEN.500030
                    Invalid context. Period start date must be the same across all duration contexts.
    
                    Legacy Rule Format:
                    IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
            
                    Data Elements:
            
                    ^StartDate = RP
            
                    ^StartDate1 = RP:RP.{SoASeqNum}
                    */
                    assertion = (rpSoASeqNumContext.StartDate != null && rpSoASeqNumContext.StartDate != report.RPStartDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500030",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500030

                    #region VR.ATO.GEN.500033

                    /*  VR.ATO.GEN.500033
                    Invalid context. Period end date must be the same across all duration contexts.
    
                    Legacy Rule Format:
                    IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
            
                    Data Elements:
            
                    ^EndDate = RP
            
                    ^EndDate1 = RP:RP.{SoASeqNum}
                    */
                    assertion = (rpSoASeqNumContext.EndDate != null && rpSoASeqNumContext.EndDate != report.RPEndDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500033",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500033

                    #region VR.ATO.TRTAMI.000121

                    /*  VR.ATO.TRTAMI.000121
                    Title is invalid
    
                    Legacy Rule Format:
                    IF ([TRTAMI76] <> NULLORBLANK) AND ANY OCCURRENCE OF([TRTAMI76]) <> SET(DOMAIN(TITLE CODES))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI76] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.Title.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI76 <> BLANK AND (NotInDomain(^TRTAMI76, 'TITLE CODES')))
            
                    Data Elements:
            
                    ^TRTAMI76 = RP.{SoASeqNum}:PersonNameDetails.Title.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI76) != true && !(IsMatch(rpSoASeqNumContext.TRTAMI76, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000121",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Title is invalid",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.Title.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000121" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI76", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI76) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000121

                    #region VR.ATO.TRTAMI.000122

                    /*  VR.ATO.TRTAMI.000122
                    Family name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF ([TRTAMI77] <> NULLORBLANK) AND ([TRTAMI77] DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI77 <> BLANK) AND (NotContainsSet(^TRTAMI77, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI77) != true && !(IsMatch(rpSoASeqNumContext.TRTAMI77, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000122",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Family Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000122" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000122

                    #region VR.ATO.TRTAMI.000123

                    /*  VR.ATO.TRTAMI.000123
                    Family name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF ([TRTAMI77] = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
        
                    Technical Business Rule Format:
                    (FoundSet(^TRTAMI77, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI77, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000123",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Family Name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000123" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000123

                    #region VR.ATO.TRTAMI.000124

                    /*  VR.ATO.TRTAMI.000124
                    Family name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF ([TRTAMI77] = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
        
                    Technical Business Rule Format:
                    (FoundSet(^TRTAMI77, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI77, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000124",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Family Name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000124" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000124

                    #region VR.ATO.TRTAMI.000125

                    /*  VR.ATO.TRTAMI.000125
                    Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF ([TRTAMI77] = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
        
                    Technical Business Rule Format:
                    (FoundSet(^TRTAMI77, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI77, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000316",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Family Name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000125" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000125

                    #region VR.ATO.TRTAMI.000126

                    /*  VR.ATO.TRTAMI.000126
                    Family Name cannot contain "space hyphen space"
    
                    Legacy Rule Format:
                    IF ([TRTAMI77] CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI77, ' - ')
            
                    Data Elements:
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI77 == null ? false : rpSoASeqNumContext.TRTAMI77.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000126",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Family Name cannot contain ""space hyphen space""",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000126" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000126

                    #region VR.ATO.TRTAMI.000127

                    /*  VR.ATO.TRTAMI.000127
                    Family name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF ([TRTAMI77] CONTAINS SET("--","''","  "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
        
                    Technical Business Rule Format:
                    (ContainsSet(^TRTAMI77, '"--","''","  "'))
            
                    Data Elements:
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI77, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000127",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Family Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000127" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000127

                    #region VR.ATO.TRTAMI.000128

                    /*  VR.ATO.TRTAMI.000128
                    Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF ([TRTAMI78] <> NULLORBLANK) AND ([TRTAMI78] DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSSAGE
                    ENDIF
                    
                    [TRTAMI78] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.GivenName.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI78 <> BLANK) AND (NotContainsSet(^TRTAMI78, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^TRTAMI78 = RP.{SoASeqNum}:PersonNameDetails.GivenName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI78) != true && !(IsMatch(rpSoASeqNumContext.TRTAMI78, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000128",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member First Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000128" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI78", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI78) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000128

                    #region VR.ATO.TRTAMI.000129

                    /*  VR.ATO.TRTAMI.000129
                    First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF ([TRTAMI78] = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI78] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.GivenName.Text
        
                    Technical Business Rule Format:
                    (FoundSet(^TRTAMI78, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^TRTAMI78 = RP.{SoASeqNum}:PersonNameDetails.GivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI78, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000317",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member First Name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000129" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI78", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI78) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000129

                    #region VR.ATO.TRTAMI.000130

                    /*  VR.ATO.TRTAMI.000130
                    First name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF ([TRTAMI78] = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI78] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.GivenName.Text
        
                    Technical Business Rule Format:
                    (FoundSet(^TRTAMI78, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^TRTAMI78 = RP.{SoASeqNum}:PersonNameDetails.GivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI78, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000130",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member First Given Name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000130" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI78", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI78) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000130

                    #region VR.ATO.TRTAMI.000131

                    /*  VR.ATO.TRTAMI.000131
                    First name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF ([TRTAMI78] = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI78] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.GivenName.Text
        
                    Technical Business Rule Format:
                    (FoundSet(^TRTAMI78, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^TRTAMI78 = RP.{SoASeqNum}:PersonNameDetails.GivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI78, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000131",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member First Given Name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000131" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI78", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI78) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000131

                    #region VR.ATO.TRTAMI.000132

                    /*  VR.ATO.TRTAMI.000132
                    FirstName must be entered if OtherGivenNames is entered
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum})
                    IF ([TRTAMI79] <> NULLORBLANK) AND ([TRTAMI78] = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI78] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.GivenName.Text
                    [TRTAMI79] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.OtherGivenName.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI79 <> BLANK) AND (^TRTAMI78 = BLANK)
            
                    Data Elements:
            
                    ^TRTAMI78 = RP.{SoASeqNum}:PersonNameDetails.GivenName.Text
            
                    ^TRTAMI79 = RP.{SoASeqNum}:PersonNameDetails.OtherGivenName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI79) != true && string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI78) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000132",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member First Given Name must be entered if Member Other Given Names is entered",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000132" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI79", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI79) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI78", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI78) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000132

                    #region VR.ATO.TRTAMI.000133

                    /*  VR.ATO.TRTAMI.000133
                    First name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF ([TRTAMI78] CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI78] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.GivenName.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI78, ' - ')
            
                    Data Elements:
            
                    ^TRTAMI78 = RP.{SoASeqNum}:PersonNameDetails.GivenName.Text
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI78 == null ? false : rpSoASeqNumContext.TRTAMI78.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000133",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member First Given Name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000133" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI78", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI78) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000133

                    #region VR.ATO.TRTAMI.000134

                    /*  VR.ATO.TRTAMI.000134
                    First name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF ([TRTAMI78] CONTAINS SET("--","''","  "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI78] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.GivenName.Text
        
                    Technical Business Rule Format:
                    (ContainsSet(^TRTAMI78, '"--","''","  "'))
            
                    Data Elements:
            
                    ^TRTAMI78 = RP.{SoASeqNum}:PersonNameDetails.GivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI78, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000134",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member First Given Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000134" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI78", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI78) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000134

                    #region VR.ATO.TRTAMI.000135

                    /*  VR.ATO.TRTAMI.000135
                    Other Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF ([TRTAMI79] <> NULLORBLANK) AND ([TRTAMI79] DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI79] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.OtherGivenName.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI79 <> BLANK) AND (NotContainsSet(^TRTAMI79, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^TRTAMI79 = RP.{SoASeqNum}:PersonNameDetails.OtherGivenName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI79) != true && !(IsMatch(rpSoASeqNumContext.TRTAMI79, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000135",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Other Given Names must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000135" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI79", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI79) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000135

                    #region VR.ATO.TRTAMI.000136

                    /*  VR.ATO.TRTAMI.000136
                    Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF ([TRTAMI79] = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI79] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.OtherGivenName.Text
        
                    Technical Business Rule Format:
                    (FoundSet(^TRTAMI79, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^TRTAMI79 = RP.{SoASeqNum}:PersonNameDetails.OtherGivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI79, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000318",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Other Given Name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000136" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI79", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI79) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000136

                    #region VR.ATO.TRTAMI.000137

                    /*  VR.ATO.TRTAMI.000137
                    Other given name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF ([TRTAMI79] = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI79] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.OtherGivenName.Text
        
                    Technical Business Rule Format:
                    (FoundSet(^TRTAMI79, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^TRTAMI79 = RP.{SoASeqNum}:PersonNameDetails.OtherGivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI79, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000137",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Other Given Names cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000137" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI79", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI79) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000137

                    #region VR.ATO.TRTAMI.000138

                    /*  VR.ATO.TRTAMI.000138
                    Other given name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF ([TRTAMI79] = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI79] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.OtherGivenName.Text
        
                    Technical Business Rule Format:
                    (FoundSet(^TRTAMI79, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^TRTAMI79 = RP.{SoASeqNum}:PersonNameDetails.OtherGivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI79, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000138",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Other Given Names cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000138" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI79", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI79) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000138

                    #region VR.ATO.TRTAMI.000139

                    /*  VR.ATO.TRTAMI.000139
                    Other given name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF ([TRTAMI79] CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI79] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.OtherGivenName.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI79, ' - ')
            
                    Data Elements:
            
                    ^TRTAMI79 = RP.{SoASeqNum}:PersonNameDetails.OtherGivenName.Text
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI79 == null ? false : rpSoASeqNumContext.TRTAMI79.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000139",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Other Given Names cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000139" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI79", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI79) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000139

                    #region VR.ATO.TRTAMI.000140

                    /*  VR.ATO.TRTAMI.000140
                    Other given name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF ([TRTAMI79] CONTAINS SET("--","''","  "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI79] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.OtherGivenName.Text
        
                    Technical Business Rule Format:
                    (ContainsSet(^TRTAMI79, '"--","''","  "'))
            
                    Data Elements:
            
                    ^TRTAMI79 = RP.{SoASeqNum}:PersonNameDetails.OtherGivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI79, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000140",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Other Given Names cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000140" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI79", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI79) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000140

                    #region VR.ATO.TRTAMI.000141

                    /*  VR.ATO.TRTAMI.000141
                    Family name or non-individual name must be present.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum})
                    IF ([TRTAMI81] <> NULL) AND ([TRTAMI77] = NULLORBLANK) AND ([TRTAMI80] = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    [TRTAMI81] = TRTAMI:RP.{SoASeqNum}:rvctc1.02.21:IncomeTax.AssessmentCalculation.Code
        
                    Technical Business Rule Format:
                    (^TRTAMI81 <> NULL) AND (^TRTAMI77 = BLANK) AND (^TRTAMI80 = BLANK)
            
                    Data Elements:
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
            
                    ^TRTAMI81 = RP.{SoASeqNum}:IncomeTax.AssessmentCalculation.Code
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI81 != null && string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI77) == true && string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI80) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000141",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Name of the Statement of Attribution member must be present",
                            LongDescription = @"If the member is an individual, provide the name of the individual. If the member is a company, provide the name of the company.",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000141" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI81", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI81) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000141

                    #region VR.ATO.TRTAMI.000142

                    /*  VR.ATO.TRTAMI.000142
                    Non-individual name and Family name details are both present. Only one name can be present.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum})
                    IF ([TRTAMI80] <> NULLORBLANK) AND ([TRTAMI76] <> NULL OR [TRTAMI77] <> NULLORBLANK OR [TRTAMI78] <> NULLORBLANK OR [TRTAMI79] <> NULLORBLANK OR [TRTAMI124] <> NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI76] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.Title.Text
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                    [TRTAMI78] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.GivenName.Text
                    [TRTAMI79] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.OtherGivenName.Text
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    [TRTAMI124] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonDemographicDetails.Birth.Date
        
                    Technical Business Rule Format:
                    (^TRTAMI80 <> BLANK) AND (^TRTAMI76 <> NULL OR ^TRTAMI77 <> BLANK OR ^TRTAMI78 <> BLANK OR ^TRTAMI79 <> BLANK OR ^TRTAMI124 <> NULL)
            
                    Data Elements:
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
            
                    ^TRTAMI76 = RP.{SoASeqNum}:PersonNameDetails.Title.Text
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
            
                    ^TRTAMI78 = RP.{SoASeqNum}:PersonNameDetails.GivenName.Text
            
                    ^TRTAMI79 = RP.{SoASeqNum}:PersonNameDetails.OtherGivenName.Text
            
                    ^TRTAMI124 = RP.{SoASeqNum}:PersonDemographicDetails.Birth.Date
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI80) != true && (rpSoASeqNumContext.TRTAMI76 != null || string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI77) != true || string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI78) != true || string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI79) != true || rpSoASeqNumContext.TRTAMI124 != null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000142",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Non-Individual Name and Member Family Name details must not both be present",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000142" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI76", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI76) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI78", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI78) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI79", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI79) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI124", Value = GetValueOrEmptyDate(rpSoASeqNumContext.TRTAMI124) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000142

                    #region VR.ATO.TRTAMI.000143

                    /*  VR.ATO.TRTAMI.000143
                    Organisation name must contain at least one alpha or numeric character
    
                    Legacy Rule Format:
                    IF ([TRTAMI80] <> NULLORBLANK) AND ([TRTAMI80] DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI80 <> BLANK) AND (NotContainsSet(^TRTAMI80, '"a-z", "A-Z", "0-9"'))
            
                    Data Elements:
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI80) != true && !(IsMatch(rpSoASeqNumContext.TRTAMI80, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000143",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Non-Individual Name must contain at least one alpha or numeric character",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000143" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000143

                    #region VR.ATO.TRTAMI.000144

                    /*  VR.ATO.TRTAMI.000144
                    Non Individual Name cannot contain space hyphen space combinations
    
                    Legacy Rule Format:
                    IF ([TRTAMI80] CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI80, ' - ')
            
                    Data Elements:
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI80 == null ? false : rpSoASeqNumContext.TRTAMI80.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000144",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Non-Individual Name cannot contain space hyphen space combinations",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000144" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000144

                    #region VR.ATO.TRTAMI.000145

                    /*  VR.ATO.TRTAMI.000145
                    Non Individual Name cannot contain "P/L"
    
                    Legacy Rule Format:
                    IF ([TRTAMI80] CONTAINS "P/L")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI80, 'P/L')
            
                    Data Elements:
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI80 == null ? false : rpSoASeqNumContext.TRTAMI80.ToUpperInvariant().Contains(@"P/L"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000145",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Non-Individual Name cannot contain ""P/L""",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000145" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000145

                    #region VR.ATO.TRTAMI.000146

                    /*  VR.ATO.TRTAMI.000146
                    Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF ([TRTAMI80] CONTAINS SET("--","''","  "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
        
                    Technical Business Rule Format:
                    (ContainsSet(^TRTAMI80, '"--","''","  "'))
            
                    Data Elements:
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI80, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000146",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Non-Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000146" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000146

                    #region VR.ATO.TRTAMI.000147

                    /*  VR.ATO.TRTAMI.000147
                    Non Individual Name cannot start with "T/A" and end with "Pship", "P'ship" or "P/Ship"
    
                    Legacy Rule Format:
                    IF ([TRTAMI80] STARTSWITH "T/A ") AND ([TRTAMI80] ENDSWITH SET(" Pship"," P'ship"," P/ship"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
        
                    Technical Business Rule Format:
                    (StartsWith(^TRTAMI80, 'T/A ')) AND (EndsWithSet(^TRTAMI80, '" Pship"," P'ship"," P/ship"'))
            
                    Data Elements:
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (IsMatch(rpSoASeqNumContext.TRTAMI80, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(rpSoASeqNumContext.TRTAMI80, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000147",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Non-Individual Name cannot start with ""T/A"" and end with ""Pship"", ""P'ship"" or ""P/Ship""",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000147" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000147

                    #region VR.ATO.TRTAMI.000149

                    /*  VR.ATO.TRTAMI.000149
                    Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
                    Legacy Rule Format:
                    IF ([TRTAMI80] ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
        
                    Technical Business Rule Format:
                    (EndsWithSet(^TRTAMI80, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
            
                    Data Elements:
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI80, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000149",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Non-Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000149" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000149

                    #region VR.ATO.TRTAMI.000166

                    /*  VR.ATO.TRTAMI.000166
                    The Tax File Number has failed the Tax File Number algorithm check.
    
                    Legacy Rule Format:
                    IF (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier <> NULLORBLANK) AND (TFNALGORITHM (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier) = FALSE)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRTAMI75 <> BLANK) AND (FailsTFNAlgorithm(^TRTAMI75))
            
                    Data Elements:
            
                    ^TRTAMI75 = RP.{SoASeqNum}:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI75) != true && FailsTFNAlgorithm(rpSoASeqNumContext.TRTAMI75));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000166",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Tax File Number has failed the algorithm check",
                            Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumber.Identifier[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000166" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI75", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI75) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000166

                    #region VR.ATO.TRTAMI.000168

                    /*  VR.ATO.TRTAMI.000168
                    The member TFN reported must not be reported more than once.
    
                    Legacy Rule Format:
                    IF [TRTAMI75] <> NULL AND [TRTAMI75] = ANY OTHER OCCURRENCE OF ([TRTAMI75])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI75] = TRTAMI:RP.{SoASeqNum}:pyid.02.00:Identifiers.TaxFileNumber.Identifier
        
                    Technical Business Rule Format:
                    ^TRTAMI75 <> NULL AND HasDuplicateValues(^TRTAMI75)
            
                    Data Elements:
            
                    ^TRTAMI75 = RP.{SoASeqNum}:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI75 != null && HasDuplicateValues(report.RPSoASeqNumContextCollection == null ? null : report.RPSoASeqNumContextCollection.Select(f => f.TRTAMI75).Cast<object>().ToArray()));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000168",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Tax File Number must not be repeated",
                            Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumber.Identifier[@contextRef='" + report.RPSoASeqNumContextCollection[DuplicateValueIndex(report.RPSoASeqNumContextCollection.Select(f => f.TRTAMI75))].Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000168" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI75", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI75) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000168

                    #region VR.ATO.TRTAMI.000171

                    /*  VR.ATO.TRTAMI.000171
                    In the Statement of Attribution, if the assessment calculation code is 138, 139 or 140, the S98 assessable amounts must be provided.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum})
                    IF [TRTAMI81] <> NULL AND [TRTAMI82] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI81] = TRTAMI:RP.{SoASeqNum}:rvctc1.02.21:IncomeTax.AssessmentCalculation.Code
                    [TRTAMI82] = TRTAMI:RP.{SoASeqNum}:rvctc4.02.10:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount
        
                    Technical Business Rule Format:
                    ^TRTAMI81 <> NULL AND ^TRTAMI82 = NULL
            
                    Data Elements:
            
                    ^TRTAMI82 = RP.{SoASeqNum}:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount
            
                    ^TRTAMI81 = RP.{SoASeqNum}:IncomeTax.AssessmentCalculation.Code
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI81 != null && rpSoASeqNumContext.TRTAMI82 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000171",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Taxable Determined Member Component amount must be present",
                            LongDescription = @"When the Assessment Calculation Code of Member is present, the Taxable Determined Member Component amount must be provided",
                            Location = "/xbrli:xbrl/tns:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000171" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI81", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI81) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI82", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI82) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000171

                    #region VR.ATO.TRTAMI.000172

                    /*  VR.ATO.TRTAMI.000172
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRTAMI82] <> NULL AND [TRTAMI82] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI82] = TRTAMI:RP.{SoASeqNum}:rvctc4.02.10:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount
        
                    Technical Business Rule Format:
                    NotMonetary(^TRTAMI82, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRTAMI82 = RP.{SoASeqNum}:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount
                    */
                    assertion = NotMonetary(rpSoASeqNumContext.TRTAMI82, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000172",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Taxable Determined Member Component is not in a valid monetary format",
                            LongDescription = @"Taxable Determined Member Component must be unsigned, not exceed 11 digits and not contain a decimal point or any other characters",
                            Location = "/xbrli:xbrl/tns:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000172" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI82", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI82) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000172

                    #region VR.ATO.TRTAMI.000193

                    /*  VR.ATO.TRTAMI.000193
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI77] <> NULLORBLANK) AND ([TRTAMI77] CONTAINS SET("{","}","|","_","\","#"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
        
                    Technical Business Rule Format:
                    ContainsSet(^TRTAMI77, '"{","}","|","_","\","#"')
            
                    Data Elements:
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI77, @"\{|\}|\||_|\\|#", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000193",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Family Name contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.FamilyName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000193" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000193

                    #region VR.ATO.TRTAMI.000194

                    /*  VR.ATO.TRTAMI.000194
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI78] <> NULLORBLANK) AND ([TRTAMI78] CONTAINS SET("{","}","|","_","\","#"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI78] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.GivenName.Text
        
                    Technical Business Rule Format:
                    ContainsSet(^TRTAMI78, '"{","}","|","_","\","#"')
            
                    Data Elements:
            
                    ^TRTAMI78 = RP.{SoASeqNum}:PersonNameDetails.GivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI78, @"\{|\}|\||_|\\|#", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000194",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member First Given Name contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.GivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000194" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI78", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI78) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000194

                    #region VR.ATO.TRTAMI.000195

                    /*  VR.ATO.TRTAMI.000195
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI79] <> NULLORBLANK) AND ([TRTAMI79] CONTAINS SET("{","}","|","_","\","#"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI79] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.OtherGivenName.Text
        
                    Technical Business Rule Format:
                    ContainsSet(^TRTAMI79, '"{","}","|","_","\","#"')
            
                    Data Elements:
            
                    ^TRTAMI79 = RP.{SoASeqNum}:PersonNameDetails.OtherGivenName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI79, @"\{|\}|\||_|\\|#", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000195",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Other Given Names contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:PersonNameDetails.OtherGivenName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000195" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI79", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI79) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000195

                    #region VR.ATO.TRTAMI.000196

                    /*  VR.ATO.TRTAMI.000196
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ # $ % & * ( ) - _ = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI80] <> NULLORBLANK) AND ([TRTAMI80] CONTAINS SET("{","}","|","\"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
        
                    Technical Business Rule Format:
                    ContainsSet(^TRTAMI80, '"{","}","|","\"')
            
                    Data Elements:
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = IsMatch(rpSoASeqNumContext.TRTAMI80, @"\{|\}|\||\\", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000196",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Non-Individual Name contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000196" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000196

                    #region VR.ATO.TRTAMI.000197

                    /*  VR.ATO.TRTAMI.000197
                    In the Statement of Attribution, if the S98 assessable amount is positive, the assessment calculation code must be provided.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum})
                    IF [TRTAMI82] > 0 AND [TRTAMI81] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI81] = TRTAMI:RP.{SoASeqNum}:rvctc1.02.21:IncomeTax.AssessmentCalculation.Code
                    [TRTAMI82] = TRTAMI:RP.{SoASeqNum}:rvctc4.02.10:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount
        
                    Technical Business Rule Format:
                    ^TRTAMI82 > 0 AND ^TRTAMI81 = NULL
            
                    Data Elements:
            
                    ^TRTAMI81 = RP.{SoASeqNum}:IncomeTax.AssessmentCalculation.Code
            
                    ^TRTAMI82 = RP.{SoASeqNum}:InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI82.GetValueOrDefault() > 0 && rpSoASeqNumContext.TRTAMI81 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000197",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Assessment Calculation Code of Member must be present",
                            LongDescription = @"When a positive value for Trustee Liability for Non-Resident Amounts is present, the Assessment Calculation Code of Member must be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.AssessmentCalculation.Code[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000197" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI82", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI82) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI81", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI81) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000197

                    #region VR.ATO.TRTAMI.000225

                    /*  VR.ATO.TRTAMI.000225
                    Date of Birth must be provided for individuals
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum})
                    IF [TRTAMI77] <> NULLORBLANK AND [TRTAMI124] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                    [TRTAMI124] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonDemographicDetails.Birth.Date
        
                    Technical Business Rule Format:
                    ^TRTAMI77 <> BLANK AND ^TRTAMI124 = NULL
            
                    Data Elements:
            
                    ^TRTAMI124 = RP.{SoASeqNum}:PersonDemographicDetails.Birth.Date
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI77) != true && rpSoASeqNumContext.TRTAMI124 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000225",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Date of Birth must be provided for individuals",
                            Location = "/xbrli:xbrl/tns:PersonDemographicDetails.Birth.Date[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000225" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI124", Value = GetValueOrEmptyDate(rpSoASeqNumContext.TRTAMI124) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000225

                    #region VR.ATO.TRTAMI.000241

                    /*  VR.ATO.TRTAMI.000241
                    For individuals, the only valid Assessment Calculation Code of Member value is "138".
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum})
                    IF [TRTAMI81] <> "138" AND [TRTAMI77] <> NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI77] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:PersonNameDetails.FamilyName.Text
                    [TRTAMI81] = TRTAMI:RP.{SoASeqNum}:rvctc1.02.21:IncomeTax.AssessmentCalculation.Code
        
                    Technical Business Rule Format:
                    ^TRTAMI81 <> '138' AND ^TRTAMI77 <> BLANK
            
                    Data Elements:
            
                    ^TRTAMI81 = RP.{SoASeqNum}:IncomeTax.AssessmentCalculation.Code
            
                    ^TRTAMI77 = RP.{SoASeqNum}:PersonNameDetails.FamilyName.Text
                    */
                    assertion = (rpSoASeqNumContext.TRTAMI81 != @"138" && string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI77) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000241",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Assessment Calculation Code of Member is invalid",
                            LongDescription = @"For individuals, the only valid Assessment Calculation Code of Member value is ""138""",
                            Location = "/xbrli:xbrl/tns:IncomeTax.AssessmentCalculation.Code[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000241" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI81", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI81) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI77", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI77) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000241

                    #region VR.ATO.TRTAMI.000242

                    /*  VR.ATO.TRTAMI.000242
                    For non-individuals, the valid Assessment Calculation Code of Member values are "139" or "140".
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum})
                    IF [TRTAMI81] <> SET("139","140") AND [TRTAMI80] <> NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI80] = TRTAMI:RP.{SoASeqNum}:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    [TRTAMI81] = TRTAMI:RP.{SoASeqNum}:rvctc1.02.21:IncomeTax.AssessmentCalculation.Code
        
                    Technical Business Rule Format:
                    NotInSet(^TRTAMI81, '"139","140"') AND ^TRTAMI80 <> BLANK
            
                    Data Elements:
            
                    ^TRTAMI81 = RP.{SoASeqNum}:IncomeTax.AssessmentCalculation.Code
            
                    ^TRTAMI80 = RP.{SoASeqNum}:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (!(IsMatch(rpSoASeqNumContext.TRTAMI81, @"^(139|140)$", RegexOptions.IgnoreCase)) && string.IsNullOrWhiteSpace(rpSoASeqNumContext.TRTAMI80) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000242",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Assessment Calculation Code of Member is invalid",
                            LongDescription = @"For non-individuals, the valid Assessment Calculation Code of Member values are ""139"" or ""140""",
                            Location = "/xbrli:xbrl/tns:IncomeTax.AssessmentCalculation.Code[@contextRef='" + rpSoASeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000242" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI81", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI81) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI80", Value = GetValueOrEmpty(rpSoASeqNumContext.TRTAMI80) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000242
                }
            }

            #endregion // Foreach loop

            #region Repeating RPSoASeqNumBUSContext
            if (report.RPSoASeqNumBUSContextCollection != null)
            {
                int itemIndexContext3 = 0;
                foreach (TRTAMI2022.RPSoASeqNumBUSContext rpSoASeqNumBUSContext in report.RPSoASeqNumBUSContextCollection)
                {
                    itemIndexContext3++;

                    #region VR.ATO.GEN.001023

                    /*  VR.ATO.GEN.001023
                    TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
                    Legacy Rule Format:
                    IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^Context <> NULL AND ^OtherTFN <> ^TFN
            
                    Data Elements:
            
                    ^Context = RP.{SoASeqNum}:RP.{SoASeqNum}.BUS
            
                    ^OtherTFN = RP.{SoASeqNum}:RP.{SoASeqNum}.BUS
            
                    ^TFN = RP
                    */
                    assertion = (rpSoASeqNumBUSContext.Exists != false && rpSoASeqNumBUSContext.IdentifierTFN != report.RPIdentifierTFN);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.001023",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Unexpected TFN mismatch with Reporting Party",
                            LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpSoASeqNumBUSContext.OccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.001023

                    #region VR.ATO.GEN.438029

                    /*  VR.ATO.GEN.438029
                    Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
                    Legacy Rule Format:
                    IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
            
                    Data Elements:
            
                    ^IdentifierScheme = RP.{SoASeqNum}:RP.{SoASeqNum}.BUS
            
                    ^Context = RP.{SoASeqNum}:RP.{SoASeqNum}.BUS
                    */
                    assertion = (rpSoASeqNumBUSContext.Exists != false && rpSoASeqNumBUSContext.IdentifierScheme != @"http://www.ato.gov.au/tfn");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438029",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpSoASeqNumBUSContext.OccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.438029

                    #region VR.ATO.GEN.500030

                    /*  VR.ATO.GEN.500030
                    Invalid context. Period start date must be the same across all duration contexts.
    
                    Legacy Rule Format:
                    IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
            
                    Data Elements:
            
                    ^StartDate = RP
            
                    ^StartDate1 = RP.{SoASeqNum}:RP.{SoASeqNum}.BUS
                    */
                    assertion = (rpSoASeqNumBUSContext.StartDate != null && rpSoASeqNumBUSContext.StartDate != report.RPStartDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500030",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500030

                    #region VR.ATO.GEN.500033

                    /*  VR.ATO.GEN.500033
                    Invalid context. Period end date must be the same across all duration contexts.
    
                    Legacy Rule Format:
                    IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
            
                    Data Elements:
            
                    ^EndDate = RP
            
                    ^EndDate1 = RP.{SoASeqNum}:RP.{SoASeqNum}.BUS
                    */
                    assertion = (rpSoASeqNumBUSContext.EndDate != null && rpSoASeqNumBUSContext.EndDate != report.RPEndDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500033",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500033

                    #region VR.ATO.TRTAMI.000230

                    /*  VR.ATO.TRTAMI.000230
                    Member Business Address - Address Line 1 has a maximum field length of 38
    
                    Legacy Rule Format:
                    IF LENGTH([TRTAMI131]) > 38
                          RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI131] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    Length(^TRTAMI131) > 38
            
                    Data Elements:
            
                    ^TRTAMI131 = RP.{SoASeqNum}.BUS:AddressDetails.Line1.Text
                    */
                    assertion = (Length(rpSoASeqNumBUSContext.TRTAMI131) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000230",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Address Line 1 must not exceed 38 characters",
                            LongDescription = @"For Member Business Address - Address Line 1, the maximum allowable characters for this form is 38",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000230" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI131", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI131) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000230

                    #region VR.ATO.TRTAMI.000231

                    /*  VR.ATO.TRTAMI.000231
                    Member Business Address - Address Line 2 has a maximum field length of 38
    
                    Legacy Rule Format:
                    IF LENGTH([TRTAMI132]) > 38
                          RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI132] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    Length(^TRTAMI132) > 38
            
                    Data Elements:
            
                    ^TRTAMI132 = RP.{SoASeqNum}.BUS:AddressDetails.Line2.Text
                    */
                    assertion = (Length(rpSoASeqNumBUSContext.TRTAMI132) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000231",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Address Line 2 must not exceed 38 characters",
                            LongDescription = @"For Member Business Address - Address Line 2, the maximum allowable characters for this form is 38",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000231" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI132", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI132) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000231

                    #region VR.ATO.TRTAMI.000232

                    /*  VR.ATO.TRTAMI.000232
                    Member Business Address - City/Town has a maximum field length of 46
    
                    Legacy Rule Format:
                    IF LENGTH([TRTAMI133]) > 46
                          RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI133] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.LocalityName.Text
        
                    Technical Business Rule Format:
                    Length(^TRTAMI133) > 46
            
                    Data Elements:
            
                    ^TRTAMI133 = RP.{SoASeqNum}.BUS:AddressDetails.LocalityName.Text
                    */
                    assertion = (Length(rpSoASeqNumBUSContext.TRTAMI133) > 46);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000232",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - City/Town must not exceed 46 characters",
                            LongDescription = @"For Member Business Address - City/Town, the maximum allowable characters for this form is 46",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000232" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI133", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI133) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000232

                    #region VR.ATO.TRTAMI.000266

                    /*  VR.ATO.TRTAMI.000266
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI131] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI131] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI131] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI131 <> BLANK) AND (NotCharacterInSet(^TRTAMI131, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
            
                    Data Elements:
            
                    ^TRTAMI131 = RP.{SoASeqNum}.BUS:AddressDetails.Line1.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumBUSContext.TRTAMI131) != true && !(IsMatch(rpSoASeqNumBUSContext.TRTAMI131, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000266",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Address Line 1 contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000266" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI131", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI131) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000266

                    #region VR.ATO.TRTAMI.000267

                    /*  VR.ATO.TRTAMI.000267
                    Address Line 1 must not contain 'AS ABOVE'
    
                    Legacy Rule Format:
                    IF [TRTAMI131] = FOUND("AS ABOVE")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI131] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    FoundSet(^TRTAMI131, '"AS ABOVE"')
            
                    Data Elements:
            
                    ^TRTAMI131 = RP.{SoASeqNum}.BUS:AddressDetails.Line1.Text
                    */
                    assertion = IsMatch(rpSoASeqNumBUSContext.TRTAMI131, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000267",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Address Line 1 must not contain 'AS ABOVE'",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000267" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI131", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI131) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000267

                    #region VR.ATO.TRTAMI.000269

                    /*  VR.ATO.TRTAMI.000269
                    Address line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-.
    
                    Legacy Rule Format:
                    IF [TRTAMI131] = FOUND("C/O","C/","Care Of","CO")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI131] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    FoundSet(^TRTAMI131, '"C/O","C/","Care Of","CO"')
            
                    Data Elements:
            
                    ^TRTAMI131 = RP.{SoASeqNum}.BUS:AddressDetails.Line1.Text
                    */
                    assertion = IsMatch(rpSoASeqNumBUSContext.TRTAMI131, @"(^(C/O|C/|Care Of|CO)$)|(^(.*)( C/O | C/ | Care Of | CO )(.*)$)|(^(C/O |C/ |Care Of |CO )(.*)$)|(^(.*)( C/O| C/| Care Of| CO)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000269",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Address Line 1 is invalid",
                            LongDescription = @"Member Business Address - Address Line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000269" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI131", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI131) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000269

                    #region VR.ATO.TRTAMI.000270

                    /*  VR.ATO.TRTAMI.000270
                    Address line 1 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF [TRTAMI131] CONTAINS "UNKNOWN"
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI131] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI131, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRTAMI131 = RP.{SoASeqNum}.BUS:AddressDetails.Line1.Text
                    */
                    assertion = (rpSoASeqNumBUSContext.TRTAMI131 == null ? false : rpSoASeqNumBUSContext.TRTAMI131.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000270",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Address Line 1 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000270" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI131", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI131) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000270

                    #region VR.ATO.TRTAMI.000271

                    /*  VR.ATO.TRTAMI.000271
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI132] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI132] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI132] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI132 <> BLANK) AND (NotCharacterInSet(^TRTAMI132, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
            
                    Data Elements:
            
                    ^TRTAMI132 = RP.{SoASeqNum}.BUS:AddressDetails.Line2.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumBUSContext.TRTAMI132) != true && !(IsMatch(rpSoASeqNumBUSContext.TRTAMI132, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000271",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Address Line 2 contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000271" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI132", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI132) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000271

                    #region VR.ATO.TRTAMI.000272

                    /*  VR.ATO.TRTAMI.000272
                    Care of' is not valid for address line 2
    
                    Legacy Rule Format:
                    IF [TRTAMI132] = FOUND("C/-","C/O","C/","Care Of","CO")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI132] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    FoundSet(^TRTAMI132, '"C/-","C/O","C/","Care Of","CO"')
            
                    Data Elements:
            
                    ^TRTAMI132 = RP.{SoASeqNum}.BUS:AddressDetails.Line2.Text
                    */
                    assertion = IsMatch(rpSoASeqNumBUSContext.TRTAMI132, @"(^(C/-|C/O|C/|Care Of|CO)$)|(^(.*)( C/- | C/O | C/ | Care Of | CO )(.*)$)|(^(C/- |C/O |C/ |Care Of |CO )(.*)$)|(^(.*)( C/-| C/O| C/| Care Of| CO)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000272",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Care of' is not valid for Member Business Address - Address Line 2",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000272" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI132", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI132) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000272

                    #region VR.ATO.TRTAMI.000273

                    /*  VR.ATO.TRTAMI.000273
                    Address line 2 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF [TRTAMI132] CONTAINS "UNKNOWN"
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI132] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI132, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRTAMI132 = RP.{SoASeqNum}.BUS:AddressDetails.Line2.Text
                    */
                    assertion = (rpSoASeqNumBUSContext.TRTAMI132 == null ? false : rpSoASeqNumBUSContext.TRTAMI132.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000273",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Address Line 2 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000273" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI132", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI132) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000273

                    #region VR.ATO.TRTAMI.000274

                    /*  VR.ATO.TRTAMI.000274
                    Address Line 2 must be present if Address Line 1 contains 'C/-'
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                    IF ([TRTAMI131] CONTAINS "C/-") AND ([TRTAMI132] = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI131] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line1.Text
                    [TRTAMI132] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    (Contains(^TRTAMI131, 'C/-')) AND (^TRTAMI132 = BLANK)
            
                    Data Elements:
            
                    ^TRTAMI132 = RP.{SoASeqNum}.BUS:AddressDetails.Line2.Text
            
                    ^TRTAMI131 = RP.{SoASeqNum}.BUS:AddressDetails.Line1.Text
                    */
                    assertion = ((rpSoASeqNumBUSContext.TRTAMI131 == null ? false : rpSoASeqNumBUSContext.TRTAMI131.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(rpSoASeqNumBUSContext.TRTAMI132) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000274",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Address Line 2 must be provided",
                            LongDescription = @"When Member Business Address - Address Line 1 contains 'C/-', Member Business Address - Address Line 2 must be provided",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000274" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI131", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI131) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI132", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI132) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000274

                    #region VR.ATO.TRTAMI.000275

                    /*  VR.ATO.TRTAMI.000275
                    State code [QLD,NSW,VIC,SA,WA,NT,ACT,TAS] or a leading space or combination of spaces with a State code is present in the suburb/town field.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                    IF ([TRTAMI136] = NULL OR [TRTAMI136] = "au") AND ([TRTAMI133] = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI133] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.LocalityName.Text
                    [TRTAMI136] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.08:AddressDetails.Country.Code
        
                    Technical Business Rule Format:
                    (^TRTAMI136 = NULL OR ^TRTAMI136 = 'au') AND (FoundSet(^TRTAMI133, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
            
                    Data Elements:
            
                    ^TRTAMI133 = RP.{SoASeqNum}.BUS:AddressDetails.LocalityName.Text
            
                    ^TRTAMI136 = RP.{SoASeqNum}.BUS:AddressDetails.Country.Code
                    */
                    assertion = ((rpSoASeqNumBUSContext.TRTAMI136 == null || rpSoASeqNumBUSContext.TRTAMI136 == @"au") && IsMatch(rpSoASeqNumBUSContext.TRTAMI133, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000275",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - City/Town cannot contain state code",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000275" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI136", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI136) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI133", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI133) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000275

                    #region VR.ATO.TRTAMI.000276

                    /*  VR.ATO.TRTAMI.000276
                    Suburb/Town/City cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF [TRTAMI133] CONTAINS "UNKNOWN"
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI133] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.LocalityName.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI133, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRTAMI133 = RP.{SoASeqNum}.BUS:AddressDetails.LocalityName.Text
                    */
                    assertion = (rpSoASeqNumBUSContext.TRTAMI133 == null ? false : rpSoASeqNumBUSContext.TRTAMI133.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000276",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - City/Town cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000276" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI133", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI133) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000276

                    #region VR.ATO.TRTAMI.000278

                    /*  VR.ATO.TRTAMI.000278
                    Numerics [0 to 9] are not allowed in the suburb/town field.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.BUS)
                    IF ([TRTAMI136] = NULL OR [TRTAMI136] = "au") AND ([TRTAMI133] CONTAINS SET(0-9))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI133] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.LocalityName.Text
                    [TRTAMI136] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.08:AddressDetails.Country.Code
        
                    Technical Business Rule Format:
                    (^TRTAMI136 = NULL OR ^TRTAMI136 = 'au') AND (ContainsSet(^TRTAMI133, '0-9'))
            
                    Data Elements:
            
                    ^TRTAMI133 = RP.{SoASeqNum}.BUS:AddressDetails.LocalityName.Text
            
                    ^TRTAMI136 = RP.{SoASeqNum}.BUS:AddressDetails.Country.Code
                    */
                    assertion = ((rpSoASeqNumBUSContext.TRTAMI136 == null || rpSoASeqNumBUSContext.TRTAMI136 == @"au") && IsMatch(rpSoASeqNumBUSContext.TRTAMI133, @"[0-9]", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000278",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - City/Town is invalid",
                            LongDescription = @"When an Australian address is provided, Member Business Address - City/Town must not contain numeric characters",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000278" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI136", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI136) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI133", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI133) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000278

                    #region VR.ATO.TRTAMI.000286

                    /*  VR.ATO.TRTAMI.000286
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI133] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI133] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI133] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.00:AddressDetails.LocalityName.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI133 <> BLANK) AND (NotCharacterInSet(^TRTAMI133, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
            
                    Data Elements:
            
                    ^TRTAMI133 = RP.{SoASeqNum}.BUS:AddressDetails.LocalityName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumBUSContext.TRTAMI133) != true && !(IsMatch(rpSoASeqNumBUSContext.TRTAMI133, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000286",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - City/Town contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000286" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI133", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI133) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000286

                    #region VR.ATO.TRTAMI.000307

                    /*  VR.ATO.TRTAMI.000307
                    Member Business Address - Postal/Zip Code has a maximum field length of 12
    
                    Legacy Rule Format:
                    IF LENGTH([TRTAMI144]) > 12
                          RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI144] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.27:AddressDetails.PostalCode.Text
        
                    Technical Business Rule Format:
                    Length(^TRTAMI144) > 12
            
                    Data Elements:
            
                    ^TRTAMI144 = RP.{SoASeqNum}.BUS:AddressDetails.PostalCode.Text
                    */
                    assertion = (Length(rpSoASeqNumBUSContext.TRTAMI144) > 12);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000307",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Postal/Zip Code must not exceed 12 characters",
                            LongDescription = @"For Member Business Address - Postal/Zip Code, the maximum allowable characters for this form is 12",
                            Location = "/xbrli:xbrl/tns:AddressDetails.PostalCode.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000307" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI144", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI144) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000307

                    #region VR.ATO.TRTAMI.000311

                    /*  VR.ATO.TRTAMI.000311
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI144] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI144] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI144] = TRTAMI:RP.{SoASeqNum}.BUS:pyde.02.27:AddressDetails.PostalCode.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI144 <> BLANK) AND (NotCharacterInSet(^TRTAMI144, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
            
                    Data Elements:
            
                    ^TRTAMI144 = RP.{SoASeqNum}.BUS:AddressDetails.PostalCode.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumBUSContext.TRTAMI144) != true && !(IsMatch(rpSoASeqNumBUSContext.TRTAMI144, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000311",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Business Address - Postal/Zip Code contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.PostalCode.Text[@contextRef='" + rpSoASeqNumBUSContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000311" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI144", Value = GetValueOrEmpty(rpSoASeqNumBUSContext.TRTAMI144) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000311
                }
            }

            #endregion // Foreach loop

            #region Repeating RPSoASeqNumRESContext
            if (report.RPSoASeqNumRESContextCollection != null)
            {
                int itemIndexContext3 = 0;
                foreach (TRTAMI2022.RPSoASeqNumRESContext rpSoASeqNumRESContext in report.RPSoASeqNumRESContextCollection)
                {
                    itemIndexContext3++;

                    #region VR.ATO.GEN.001023

                    /*  VR.ATO.GEN.001023
                    TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
                    Legacy Rule Format:
                    IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^Context <> NULL AND ^OtherTFN <> ^TFN
            
                    Data Elements:
            
                    ^Context = RP.{SoASeqNum}:RP.{SoASeqNum}.RES
            
                    ^OtherTFN = RP.{SoASeqNum}:RP.{SoASeqNum}.RES
            
                    ^TFN = RP
                    */
                    assertion = (rpSoASeqNumRESContext.Exists != false && rpSoASeqNumRESContext.IdentifierTFN != report.RPIdentifierTFN);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.001023",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Unexpected TFN mismatch with Reporting Party",
                            LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpSoASeqNumRESContext.OccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.001023

                    #region VR.ATO.GEN.438029

                    /*  VR.ATO.GEN.438029
                    Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
                    Legacy Rule Format:
                    IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
            
                    Data Elements:
            
                    ^IdentifierScheme = RP.{SoASeqNum}:RP.{SoASeqNum}.RES
            
                    ^Context = RP.{SoASeqNum}:RP.{SoASeqNum}.RES
                    */
                    assertion = (rpSoASeqNumRESContext.Exists != false && rpSoASeqNumRESContext.IdentifierScheme != @"http://www.ato.gov.au/tfn");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438029",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpSoASeqNumRESContext.OccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.438029

                    #region VR.ATO.GEN.500030

                    /*  VR.ATO.GEN.500030
                    Invalid context. Period start date must be the same across all duration contexts.
    
                    Legacy Rule Format:
                    IF period.startDate WHERE CONTEXT(ALL) <> RP:period.startDate
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^StartDate1 <> NULL AND ^StartDate1 <> ^StartDate
            
                    Data Elements:
            
                    ^StartDate = RP
            
                    ^StartDate1 = RP.{SoASeqNum}:RP.{SoASeqNum}.RES
                    */
                    assertion = (rpSoASeqNumRESContext.StartDate != null && rpSoASeqNumRESContext.StartDate != report.RPStartDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500030",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500030" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500030

                    #region VR.ATO.GEN.500033

                    /*  VR.ATO.GEN.500033
                    Invalid context. Period end date must be the same across all duration contexts.
    
                    Legacy Rule Format:
                    IF period.endDate WHERE CONTEXT(ALL) <> RP:period.endDate
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^EndDate1 <> NULL AND ^EndDate1 <> ^EndDate
            
                    Data Elements:
            
                    ^EndDate = RP
            
                    ^EndDate1 = RP.{SoASeqNum}:RP.{SoASeqNum}.RES
                    */
                    assertion = (rpSoASeqNumRESContext.EndDate != null && rpSoASeqNumRESContext.EndDate != report.RPEndDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500033",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500033" } },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500033

                    #region VR.ATO.TRTAMI.000226

                    /*  VR.ATO.TRTAMI.000226
                    Member Residential Address - Address Line 1 has a maximum field length of 38
    
                    Legacy Rule Format:
                    IF LENGTH([TRTAMI125]) > 38
                          RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI125] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    Length(^TRTAMI125) > 38
            
                    Data Elements:
            
                    ^TRTAMI125 = RP.{SoASeqNum}.RES:AddressDetails.Line1.Text
                    */
                    assertion = (Length(rpSoASeqNumRESContext.TRTAMI125) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000226",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Address Line 1 must not exceed 38 characters",
                            LongDescription = @"For Member Residential Address - Address Line 1, the maximum allowable characters for this form is 38",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000226" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI125", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI125) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000226

                    #region VR.ATO.TRTAMI.000227

                    /*  VR.ATO.TRTAMI.000227
                    Member Residential Address - Address Line 2 has a maximum field length of 38
    
                    Legacy Rule Format:
                    IF LENGTH([TRTAMI126]) > 38
                          RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI126] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    Length(^TRTAMI126) > 38
            
                    Data Elements:
            
                    ^TRTAMI126 = RP.{SoASeqNum}.RES:AddressDetails.Line2.Text
                    */
                    assertion = (Length(rpSoASeqNumRESContext.TRTAMI126) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000227",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Address Line 2 must not exceed 38 characters",
                            LongDescription = @"For Member Residential Address - Address Line 2, the maximum allowable characters for this form is 38",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000227" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI126", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI126) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000227

                    #region VR.ATO.TRTAMI.000228

                    /*  VR.ATO.TRTAMI.000228
                    Member Residential Address - City/Town has a maximum field length of 46
    
                    Legacy Rule Format:
                    IF LENGTH([TRTAMI127]) > 46
                          RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI127] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.LocalityName.Text
        
                    Technical Business Rule Format:
                    Length(^TRTAMI127) > 46
            
                    Data Elements:
            
                    ^TRTAMI127 = RP.{SoASeqNum}.RES:AddressDetails.LocalityName.Text
                    */
                    assertion = (Length(rpSoASeqNumRESContext.TRTAMI127) > 46);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000228",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - City/Town must not exceed 46 characters",
                            LongDescription = @"For Member Residential Address - City/Town, the maximum allowable characters for this form is 46",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000228" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI127", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI127) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000228

                    #region VR.ATO.TRTAMI.000247

                    /*  VR.ATO.TRTAMI.000247
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI125] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI125] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI125] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI125 <> BLANK) AND (NotCharacterInSet(^TRTAMI125, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
            
                    Data Elements:
            
                    ^TRTAMI125 = RP.{SoASeqNum}.RES:AddressDetails.Line1.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumRESContext.TRTAMI125) != true && !(IsMatch(rpSoASeqNumRESContext.TRTAMI125, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000247",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Address Line 1 contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000247" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI125", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI125) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000247

                    #region VR.ATO.TRTAMI.000248

                    /*  VR.ATO.TRTAMI.000248
                    Address Line 1 must not contain 'AS ABOVE'
    
                    Legacy Rule Format:
                    IF [TRTAMI125] = FOUND("AS ABOVE")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI125] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    FoundSet(^TRTAMI125, '"AS ABOVE"')
            
                    Data Elements:
            
                    ^TRTAMI125 = RP.{SoASeqNum}.RES:AddressDetails.Line1.Text
                    */
                    assertion = IsMatch(rpSoASeqNumRESContext.TRTAMI125, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000248",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Address Line 1 must not contain 'AS ABOVE'",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000248" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI125", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI125) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000248

                    #region VR.ATO.TRTAMI.000250

                    /*  VR.ATO.TRTAMI.000250
                    Address line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-.
    
                    Legacy Rule Format:
                    IF [TRTAMI125] = FOUND("C/O","C/","Care Of","CO")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI125] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    FoundSet(^TRTAMI125, '"C/O","C/","Care Of","CO"')
            
                    Data Elements:
            
                    ^TRTAMI125 = RP.{SoASeqNum}.RES:AddressDetails.Line1.Text
                    */
                    assertion = IsMatch(rpSoASeqNumRESContext.TRTAMI125, @"(^(C/O|C/|Care Of|CO)$)|(^(.*)( C/O | C/ | Care Of | CO )(.*)$)|(^(C/O |C/ |Care Of |CO )(.*)$)|(^(.*)( C/O| C/| Care Of| CO)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000250",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Address Line 1 is invalid",
                            LongDescription = @"Member Residential Address - Address Line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000250" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI125", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI125) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000250

                    #region VR.ATO.TRTAMI.000251

                    /*  VR.ATO.TRTAMI.000251
                    Address line 1 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF [TRTAMI125] CONTAINS "UNKNOWN"
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI125] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line1.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI125, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRTAMI125 = RP.{SoASeqNum}.RES:AddressDetails.Line1.Text
                    */
                    assertion = (rpSoASeqNumRESContext.TRTAMI125 == null ? false : rpSoASeqNumRESContext.TRTAMI125.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000251",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Address Line 1 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line1.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000251" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI125", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI125) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000251

                    #region VR.ATO.TRTAMI.000252

                    /*  VR.ATO.TRTAMI.000252
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI126] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI126] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI126] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI126 <> BLANK) AND (NotCharacterInSet(^TRTAMI126, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
            
                    Data Elements:
            
                    ^TRTAMI126 = RP.{SoASeqNum}.RES:AddressDetails.Line2.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumRESContext.TRTAMI126) != true && !(IsMatch(rpSoASeqNumRESContext.TRTAMI126, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000252",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Address Line 2 contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000252" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI126", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI126) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000252

                    #region VR.ATO.TRTAMI.000253

                    /*  VR.ATO.TRTAMI.000253
                    Care of' is not valid for address line 2
    
                    Legacy Rule Format:
                    IF [TRTAMI126] = FOUND("C/-","C/O","C/","Care Of","CO")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI126] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    FoundSet(^TRTAMI126, '"C/-","C/O","C/","Care Of","CO"')
            
                    Data Elements:
            
                    ^TRTAMI126 = RP.{SoASeqNum}.RES:AddressDetails.Line2.Text
                    */
                    assertion = IsMatch(rpSoASeqNumRESContext.TRTAMI126, @"(^(C/-|C/O|C/|Care Of|CO)$)|(^(.*)( C/- | C/O | C/ | Care Of | CO )(.*)$)|(^(C/- |C/O |C/ |Care Of |CO )(.*)$)|(^(.*)( C/-| C/O| C/| Care Of| CO)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000253",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Care of' is not valid for Member Residential Address - Address Line 2",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000253" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI126", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI126) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000253

                    #region VR.ATO.TRTAMI.000254

                    /*  VR.ATO.TRTAMI.000254
                    Address line 2 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF [TRTAMI126] CONTAINS "UNKNOWN"
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI126] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI126, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRTAMI126 = RP.{SoASeqNum}.RES:AddressDetails.Line2.Text
                    */
                    assertion = (rpSoASeqNumRESContext.TRTAMI126 == null ? false : rpSoASeqNumRESContext.TRTAMI126.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000254",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Address Line 2 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000254" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI126", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI126) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000254

                    #region VR.ATO.TRTAMI.000255

                    /*  VR.ATO.TRTAMI.000255
                    Address Line 2 must be present if Address Line 1 contains 'C/-'
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                    IF ([TRTAMI125] CONTAINS "C/-") AND ([TRTAMI126] = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI125] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line1.Text
                    [TRTAMI126] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.Line2.Text
        
                    Technical Business Rule Format:
                    (Contains(^TRTAMI125, 'C/-')) AND (^TRTAMI126 = BLANK)
            
                    Data Elements:
            
                    ^TRTAMI126 = RP.{SoASeqNum}.RES:AddressDetails.Line2.Text
            
                    ^TRTAMI125 = RP.{SoASeqNum}.RES:AddressDetails.Line1.Text
                    */
                    assertion = ((rpSoASeqNumRESContext.TRTAMI125 == null ? false : rpSoASeqNumRESContext.TRTAMI125.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(rpSoASeqNumRESContext.TRTAMI126) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000255",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Address Line 2 must be provided",
                            LongDescription = @"When Member Residential Address - Address Line 1 contains 'C/-', Member Residential Address - Address Line 2 must be provided",
                            Location = "/xbrli:xbrl/tns:AddressDetails.Line2.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000255" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI125", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI125) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI126", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI126) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000255

                    #region VR.ATO.TRTAMI.000256

                    /*  VR.ATO.TRTAMI.000256
                    State code [QLD,NSW,VIC,SA,WA,NT,ACT,TAS] or a leading space or combination of spaces with a State code is present in the suburb/town field.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                    IF ([TRTAMI130] = NULL OR [TRTAMI130] = "au") AND ([TRTAMI127] = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI127] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.LocalityName.Text
                    [TRTAMI130] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.08:AddressDetails.Country.Code
        
                    Technical Business Rule Format:
                    (^TRTAMI130 = NULL OR ^TRTAMI130 = 'au') AND (FoundSet(^TRTAMI127, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
            
                    Data Elements:
            
                    ^TRTAMI127 = RP.{SoASeqNum}.RES:AddressDetails.LocalityName.Text
            
                    ^TRTAMI130 = RP.{SoASeqNum}.RES:AddressDetails.Country.Code
                    */
                    assertion = ((rpSoASeqNumRESContext.TRTAMI130 == null || rpSoASeqNumRESContext.TRTAMI130 == @"au") && IsMatch(rpSoASeqNumRESContext.TRTAMI127, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000256",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - City/Town cannot contain state code",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000256" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI130", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI130) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI127", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI127) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000256

                    #region VR.ATO.TRTAMI.000257

                    /*  VR.ATO.TRTAMI.000257
                    Suburb/Town/City cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF [TRTAMI127] CONTAINS "UNKNOWN"
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI127] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.LocalityName.Text
        
                    Technical Business Rule Format:
                    Contains(^TRTAMI127, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRTAMI127 = RP.{SoASeqNum}.RES:AddressDetails.LocalityName.Text
                    */
                    assertion = (rpSoASeqNumRESContext.TRTAMI127 == null ? false : rpSoASeqNumRESContext.TRTAMI127.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000257",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - City/Town cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000257" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI127", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI127) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000257

                    #region VR.ATO.TRTAMI.000259

                    /*  VR.ATO.TRTAMI.000259
                    Numerics [0 to 9] are not allowed in the suburb/town field.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SoASeqNum}.RES)
                    IF ([TRTAMI130] = NULL OR [TRTAMI130] = "au") AND ([TRTAMI127] CONTAINS SET(0-9))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI127] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.LocalityName.Text
                    [TRTAMI130] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.08:AddressDetails.Country.Code
        
                    Technical Business Rule Format:
                    (^TRTAMI130 = NULL OR ^TRTAMI130 = 'au') AND (ContainsSet(^TRTAMI127, '0-9'))
            
                    Data Elements:
            
                    ^TRTAMI127 = RP.{SoASeqNum}.RES:AddressDetails.LocalityName.Text
            
                    ^TRTAMI130 = RP.{SoASeqNum}.RES:AddressDetails.Country.Code
                    */
                    assertion = ((rpSoASeqNumRESContext.TRTAMI130 == null || rpSoASeqNumRESContext.TRTAMI130 == @"au") && IsMatch(rpSoASeqNumRESContext.TRTAMI127, @"[0-9]", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000259",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - City/Town is invalid",
                            LongDescription = @"When an Australian address is provided, Member Residential Address - City/Town must not contain numeric characters",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000259" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI130", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI130) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI127", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI127) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000259

                    #region VR.ATO.TRTAMI.000285

                    /*  VR.ATO.TRTAMI.000285
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI127] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI127] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI127] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.00:AddressDetails.LocalityName.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI127 <> BLANK) AND (NotCharacterInSet(^TRTAMI127, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
            
                    Data Elements:
            
                    ^TRTAMI127 = RP.{SoASeqNum}.RES:AddressDetails.LocalityName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumRESContext.TRTAMI127) != true && !(IsMatch(rpSoASeqNumRESContext.TRTAMI127, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000285",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - City/Town contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.LocalityName.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000285" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI127", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI127) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000285

                    #region VR.ATO.TRTAMI.000304

                    /*  VR.ATO.TRTAMI.000304
                    Member Residential Address - Postal/Zip Code has a maximum field length of 12
    
                    Legacy Rule Format:
                    IF LENGTH([TRTAMI143]) > 12
                          RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI143] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.27:AddressDetails.PostalCode.Text
        
                    Technical Business Rule Format:
                    Length(^TRTAMI143) > 12
            
                    Data Elements:
            
                    ^TRTAMI143 = RP.{SoASeqNum}.RES:AddressDetails.PostalCode.Text
                    */
                    assertion = (Length(rpSoASeqNumRESContext.TRTAMI143) > 12);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000304",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Postal/Zip Code must not exceed 12 characters",
                            LongDescription = @"For Member Residential Address - Postal/Zip Code, the maximum allowable characters for this form is 12",
                            Location = "/xbrli:xbrl/tns:AddressDetails.PostalCode.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000304" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI143", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI143) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000304

                    #region VR.ATO.TRTAMI.000310

                    /*  VR.ATO.TRTAMI.000310
                    A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' " , . ? / or a space character.
    
                    Legacy Rule Format:
                    IF ([TRTAMI143] <> NULLORBLANK) AND (ANY CHARACTER OF [TRTAMI143] <> SET("a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI143] = TRTAMI:RP.{SoASeqNum}.RES:pyde.02.27:AddressDetails.PostalCode.Text
        
                    Technical Business Rule Format:
                    (^TRTAMI143 <> BLANK) AND (NotCharacterInSet(^TRTAMI143, '"a-z","A-Z","0-9","!","@","$","%","&","*","(",")","-","=","[","]",";",":","'",""",",",".","?","/"," "'))
            
                    Data Elements:
            
                    ^TRTAMI143 = RP.{SoASeqNum}.RES:AddressDetails.PostalCode.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpSoASeqNumRESContext.TRTAMI143) != true && !(IsMatch(rpSoASeqNumRESContext.TRTAMI143, @"^[a-zA-Z0-9!@\$%&\*\(\)\-=\[\];:'"",\.\?/ ]*$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000310",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Member Residential Address - Postal/Zip Code contains invalid text",
                            LongDescription = @"A text character must be one of the following: A to Z a to z 0 to 9 ! @ $ % & * ( ) - = [ ] ; : ' "" , . ? / or a space character.",
                            Location = "/xbrli:xbrl/tns:AddressDetails.PostalCode.Text[@contextRef='" + rpSoASeqNumRESContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000310" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI143", Value = GetValueOrEmpty(rpSoASeqNumRESContext.TRTAMI143) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000310
                }
            }

            #endregion // Foreach loop

            #region Repeating TRTAMI139Collection
            if (report.TRTAMI139Collection != null)
            {
                int itemIndex1 = 0;
                foreach (string TRTAMI139 in report.TRTAMI139Collection)
                {
                    itemIndex1++;

                    #region VR.ATO.TRTAMI.000293

                    /*  VR.ATO.TRTAMI.000293
                    The Stapled Entity Australian Business Number has failed the ABN algorithm check.
    
                    Legacy Rule Format:
                    IF ANY OCCURRENCE OF([TRTAMI139]) <> NULL AND ABNALGORITHM([TRTAMI139]) = FALSE
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI139] = TRTAMI:RP.StapledEntity:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
        
                    Technical Business Rule Format:
                    AnyOccurrence(^TRTAMI139, ^TRTAMI139 <> NULL) AND FailsABNAlgorithm(^TRTAMI139)
            
                    Data Elements:
            
                    ^TRTAMI139 = RP.StapledEntity:Identifiers.AustralianBusinessNumber.Identifier
                    */
                    assertion = ((report.TRTAMI139Collection == null ? false : report.TRTAMI139Collection.Any(TRTAMI139Repeat => TRTAMI139Repeat != null)) && FailsABNAlgorithm(TRTAMI139));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000293",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Stapled Entity Australian Business Number is invalid",
                            LongDescription = @"The Stapled Entity Australian Business Number has failed the ABN algorithm check",
                            Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000293" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI139", Value = "TRTAMI139" });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000293

                    #region VR.ATO.TRTAMI.000294

                    /*  VR.ATO.TRTAMI.000294
                    Stapled Entity Australian Business Number must differ from the Trust Australian Business Number.
    
                    Legacy Rule Format:
                    IF ANY OCCURRENCE OF([TRTAMI139]) <> NULL AND [TRTAMI139] = [TRTAMI5]
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI5] = TRTAMI:RP:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
                    [TRTAMI139] = TRTAMI:RP.StapledEntity:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
        
                    Technical Business Rule Format:
                    AnyOccurrence(^TRTAMI139, ^TRTAMI139 <> NULL) AND ^TRTAMI139 = ^TRTAMI5
            
                    Data Elements:
            
                    ^TRTAMI139 = RP.StapledEntity:Identifiers.AustralianBusinessNumber.Identifier
            
                    ^TRTAMI5 = RP:Identifiers.AustralianBusinessNumber.Identifier
                    */
                    assertion = ((report.TRTAMI139Collection == null ? false : report.TRTAMI139Collection.Any(TRTAMI139Repeat => TRTAMI139Repeat != null)) && TRTAMI139 == report.TRTAMI5);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000294",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Stapled Entity Australian Business Number must differ from the Trust Australian Business Number",
                            Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000294" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI139", Value = "TRTAMI139" });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI5", Value = report.TRTAMI5 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000294

                    #region VR.ATO.TRTAMI.000295

                    /*  VR.ATO.TRTAMI.000295
                    Stapled Entity Australian Business Number must differ from the Trustee Australian Business Number.
    
                    Legacy Rule Format:
                    IF ANY OCCURRENCE OF([TRTAMI139]) <> NULL AND [TRTAMI139] = [TRTAMI29]
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRTAMI29] = TRTAMI:RP.Trustee:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
                    [TRTAMI139] = TRTAMI:RP.StapledEntity:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
        
                    Technical Business Rule Format:
                    AnyOccurrence(^TRTAMI139, ^TRTAMI139 <> NULL) AND ^TRTAMI139 = ^TRTAMI29
            
                    Data Elements:
            
                    ^TRTAMI139 = RP.StapledEntity:Identifiers.AustralianBusinessNumber.Identifier
            
                    ^TRTAMI29 = RP.Trustee:Identifiers.AustralianBusinessNumber.Identifier
                    */
                    assertion = ((report.TRTAMI139Collection == null ? false : report.TRTAMI139Collection.Any(TRTAMI139Repeat => TRTAMI139Repeat != null)) && TRTAMI139 == report.TRTAMI29);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRTAMI.000295",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Stapled Entity Australian Business Number must differ from the Trustee Australian Business Number",
                            Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000295" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI139", Value = "TRTAMI139" });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRTAMI29", Value = GetValueOrEmpty(report.TRTAMI29) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.TRTAMI.000295
                }
            }

            #endregion // Foreach loop

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Value = _emptyParameterValue;
                        }

                        if (currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }


    }
}
